@@ -12,7 +12,11 @@ extern void reiser4_fs_close(reiser4_fs_t *fs);
 
 extern errno_t reiser4_fs_root_key(reiser4_fs_t *fs, reiser4_key_t *key);
 
+#ifndef ENABLE_STAND_ALONE
 extern reiser4_fs_t *reiser4_fs_open(aal_device_t *device, bool_t check);
+#else
+extern reiser4_fs_t *reiser4_fs_open(aal_device_t *device);
+#endif
 
 #ifndef ENABLE_STAND_ALONE
 
@@ -45,8 +49,6 @@ extern reiser4_fs_t *reiser4_fs_create(aal_device_t *device,
 
 extern errno_t reiser4_fs_backup(reiser4_fs_t *fs, aal_stream_t *stream);
 
-extern errno_t reiser4_fs_init_params(reiser4_fs_t *fs);
-
 #endif
 
 #endif
