@@ -17,9 +17,9 @@ extern void reiser4_format_mkclean(reiser4_format_t *format);
 extern bool_t reiser4_format_isdirty(reiser4_format_t *format);
 
 extern reiser4_format_t *reiser4_format_create(reiser4_fs_t *fs,
-					       count_t blocks,
-					       uint16_t policy,
-					       rid_t pid);
+					       reiser4_plug_t *plug,
+					       reiser4_plug_t *policy,
+					       count_t blocks);
 
 extern errno_t reiser4_format_backup(reiser4_format_t *format,
 				     aal_stream_t *stream);
@@ -60,11 +60,10 @@ extern count_t reiser4_format_get_free(reiser4_format_t *format);
 extern const char *reiser4_format_name(reiser4_format_t *format);
 extern uint32_t reiser4_format_get_stamp(reiser4_format_t *format);
 extern uint16_t reiser4_format_get_policy(reiser4_format_t *format);
+extern blk_t reiser4_format_get_root(reiser4_format_t *format);
 #endif
 
 extern void reiser4_format_close(reiser4_format_t *format);
-extern rid_t reiser4_format_key_pid(reiser4_format_t *format);
-extern blk_t reiser4_format_get_root(reiser4_format_t *format);
 extern uint16_t reiser4_format_get_height(reiser4_format_t *format);
 #endif
 
