@@ -9,31 +9,32 @@
 #include <aal/libaal.h>
 
 /* Leaf and twig levels. */
-#define LEAF_LEVEL	        1
-#define TWIG_LEVEL	        (LEAF_LEVEL + 1)
+#define LEAF_LEVEL			1
+#define TWIG_LEVEL			(LEAF_LEVEL + 1)
 
 /* Master related stuff like magic and offset in bytes. These are used by both
    plugins and library itself. */
-#define REISER4_MASTER_MAGIC	("ReIsEr4")
-#define REISER4_MASTER_OFFSET	(65536)
+#define REISER4_MASTER_MAGIC		("ReIsEr4")
+#define REISER4_MASTER_OFFSET		(65536)
+#define REISER4_MASTER_BLOCKNR(blksize)	(REISER4_MASTER_OFFSET/blksize)
 
 /* The same for fs stat block. */
-#define REISER4_STATUS_BLOCK    (21)
-#define REISER4_STATUS_MAGIC    ("ReiSeR4StATusBl")
+#define REISER4_STATUS_BLOCKNR(blksize)	(REISER4_MASTER_BLOCKNR(blksize) + 5)
+#define REISER4_STATUS_MAGIC		("ReiSeR4StATusBl")
 
-/* Max number of backups on the reiser4. */
-#define REISER4_BACKUPS_MAX	16
+/* Where the backup starts. */
+#define REISER4_BACKUP_START(blksize)	(REISER4_MASTER_BLOCKNR(blksize) + 6)
 
 /* Root key locality and objectid. This is actually defined in oid plugin,
    but hardcoded here to exclude oid plugin from the stand alone mode at all,
    nothing but these oids is needed there. */
-#define REISER4_ROOT_LOCALITY   (0x29)
-#define REISER4_ROOT_OBJECTID   (0x2a)
+#define REISER4_ROOT_LOCALITY		(0x29)
+#define REISER4_ROOT_OBJECTID		(0x2a)
 
 /* Macros for hole and unallocated extents. Used by both plugins (extent40) and
    library itself. */
-#define EXTENT_HOLE_UNIT        (0)
-#define EXTENT_UNALLOC_UNIT     (1)
+#define EXTENT_HOLE_UNIT		(0)
+#define EXTENT_UNALLOC_UNIT		(1)
 
 /* Defining the types for disk structures. All types like f32_t are fake ones
    and needed to avoid gcc-2.95.x bug with size of typedefined aligned types. */
@@ -43,7 +44,7 @@ typedef uint32_t f32_t; typedef f32_t d32_t __attribute__((aligned(4)));
 typedef uint64_t f64_t; typedef f64_t d64_t __attribute__((aligned(8)));
 
 /* Basic reiser4 types used by both library and plugins. */
-typedef uint8_t rid_t;
+typedef uint32_t rid_t;
 typedef uint64_t oid_t;
 
 /* Type for position in node (item and unit component). */
@@ -68,26 +69,24 @@ typedef int32_t lookup_t;
 /* Known by library plugin types. */
 enum reiser4_plug_type {
 	OBJECT_PLUG_TYPE        = 0x0,
-	/* 0x1 is reserved to be compatable with the kernel that distingushes
-	   file and dir plugin types. */
-	ITEM_PLUG_TYPE          = 0x2,
-	NODE_PLUG_TYPE          = 0x3,
-	HASH_PLUG_TYPE          = 0x4,
-	FIBRE_PLUG_TYPE		= 0x5,
-	POLICY_PLUG_TYPE        = 0x6,
-	PERM_PLUG_TYPE          = 0x7,
-	SDEXT_PLUG_TYPE         = 0x8,
-	FORMAT_PLUG_TYPE        = 0x9,
-	OID_PLUG_TYPE           = 0xa,
-	JNODE_PLUG_TYPE         = 0xb,
-	CRYPTO_PLUG_TYPE	= 0xc,
-	DIGEST_PLUG_TYPE	= 0xd,
-	COMPRESS_PLUG_TYPE	= 0xe,
+	ITEM_PLUG_TYPE          = 0x1,
+	NODE_PLUG_TYPE          = 0x2,
+	HASH_PLUG_TYPE          = 0x3,
+	FIBRE_PLUG_TYPE		= 0x4,
+	POLICY_PLUG_TYPE        = 0x5,
+	PERM_PLUG_TYPE          = 0x6,
+	SDEXT_PLUG_TYPE         = 0x7,
+	FORMAT_PLUG_TYPE        = 0x8,
+	OID_PLUG_TYPE           = 0x9,
+	JNODE_PLUG_TYPE         = 0xa,
+	CRYPTO_PLUG_TYPE	= 0xb,
+	DIGEST_PLUG_TYPE	= 0xc,
+	COMPRESS_PLUG_TYPE	= 0xd,
 	
 	/* These are not plugins in the kernel. */
-	ALLOC_PLUG_TYPE         = 0xf,
-	JOURNAL_PLUG_TYPE       = 0x10,
-	KEY_PLUG_TYPE           = 0x11,
+	ALLOC_PLUG_TYPE         = 0xe,
+	JOURNAL_PLUG_TYPE       = 0xf,
+	KEY_PLUG_TYPE           = 0x10,
 	LAST_PLUG_TYPE
 };
 
@@ -115,7 +114,7 @@ typedef enum reiser4_object_group reiser4_object_group_t;
 
 /* Known item plugin ids. */
 enum reiser4_item_plug_id {
-	ITEM_STATDATA40_ID	= 0x0,
+	ITEM_STAT40_ID		= 0x0,
 	ITEM_SDE40_ID	        = 0x1,
 	ITEM_CDE40_ID	        = 0x2,
 	ITEM_NODEPTR40_ID	= 0x3,
@@ -129,9 +128,9 @@ enum reiser4_item_plug_id {
 
 /* Known item groups. */
 enum reiser4_item_group {
-	STATDATA_ITEM		= 0x0,
-	NODEPTR_ITEM		= 0x1,
-	DIRENTRY_ITEM		= 0x2,
+	STAT_ITEM		= 0x0,
+	PTR_ITEM		= 0x1,
+	DIR_ITEM		= 0x2,
 	TAIL_ITEM		= 0x3,
 	EXTENT_ITEM		= 0x4,
 	PERMISSION_ITEM		= 0x5, /* not used yet */
@@ -165,9 +164,8 @@ typedef enum reiser4_hash_plug_id reiser4_hash_plug_id_t;
 /* Know tail policy plugin ids. */
 enum reiser4_tail_plug_id {
 	TAIL_NEVER_ID		= 0x0,
-	TAIL_SUPPRESS_OLD_ID	= 0x1,
-	TAIL_ALWAYS_ID		= 0x3,
-	TAIL_SMART_ID		= 0x4,
+	TAIL_ALWAYS_ID		= 0x1,
+	TAIL_SMART_ID		= 0x2,
 	TAIL_LAST_ID
 };
 
@@ -185,8 +183,9 @@ enum reiser4_sdext_plug_id {
 	SDEXT_SYMLINK_ID	= 0x3,
 	SDEXT_PLUG_ID		= 0x4,
 	SDEXT_FLAGS_ID          = 0x5,
-	SDEXT_CAPS_ID           = 0x6,
-	SDEXT_LARGE_TIMES_ID    = 0x7,
+	SDEXT_CAPS_ID		= 0x6,
+	SDEXT_CLUSTER_ID	= 0x7,
+	SDEXT_CRYPTO_ID		= 0x8,
 	SDEXT_LAST_ID
 };
 
@@ -244,7 +243,9 @@ typedef enum reiser4_fibre_plug_id reiser4_fibre_plug_id_t;
 struct reiser4_key {
 	reiser4_plug_t *plug;
 	d64_t body[4];
+#ifndef ENABLE_STAND_ALONE
 	uint32_t adjust;
+#endif
 };
 
 typedef struct reiser4_key reiser4_key_t;
@@ -261,10 +262,50 @@ enum key_type {
 
 typedef enum key_type key_type_t;
 
+/* Tree Plugin SET index. */
+enum reiser4_tpset_id {
+	TPSET_KEY		= 0x0,
+	TPSET_NODE		= 0x1,
+	TPSET_NODEPTR		= 0x2,
+
+	TPSET_LAST
+};
+
+/* Object Plugin SET index. */
+enum reiser4_opset_id {
+	OPSET_OBJ		= 0x0,
+	OPSET_DIR		= 0x1,
+	OPSET_PERM		= 0x2,
+	OPSET_POLICY		= 0x3,
+	OPSET_HASH		= 0x4,
+	OPSET_FIBRE		= 0x5,
+	OPSET_STAT		= 0x6,
+	OPSET_DIRITEM		= 0x7,
+	OPSET_CRYPTO		= 0x8,
+	OPSET_DIGEST		= 0x9,
+	OPSET_COMPRES		= 0xa,
+	
+	OPSET_STORE_LAST        = (OPSET_COMPRES + 1),
+	
+	/* These are not stored on disk in the current implementation. */
+	OPSET_CREATE		= 0xb,
+	OPSET_MKDIR		= 0xc,
+	OPSET_SYMLINK		= 0xd,
+	OPSET_MKNODE		= 0xe,
+	
+#ifndef ENABLE_STAND_ALONE
+	OPSET_TAIL		= 0xf,
+	OPSET_EXTENT		= 0x10,
+	OPSET_ACL		= 0x11,
+#endif
+	OPSET_LAST
+};
+
 /* Known print options for key. */
 enum print_options {
 	PO_DEFAULT              = 0x0,
-	PO_INODE                = 0x1
+	PO_INODE                = 0x1,
+	PO_UNIT_OFFSETS         = 0x2
 };
 
 typedef enum print_options print_options_t;
@@ -277,9 +318,20 @@ struct generic_entity {
 
 typedef struct generic_entity generic_entity_t;
 
+struct tree_entity {
+	/* Plugin SET. Set of plugins needed for the reiser4progs work.
+	   Consists of tree-specific plugins and object-specific plugins. */
+	reiser4_plug_t *tpset[TPSET_LAST];
+	reiser4_plug_t *opset[OPSET_LAST];
+};
+
+typedef struct tree_entity tree_entity_t;
+
 typedef struct reiser4_node reiser4_node_t;
 typedef struct reiser4_place reiser4_place_t;
 
+#define place_blknr(place) ((place)->node->block->nr)
+
 /* Tree coord struct. */
 struct reiser4_place {
 	/* Item/unit pos and node it lies in. These fields should be always
@@ -292,13 +344,13 @@ struct reiser4_place {
 	   method fetch() and stored here to make work with them simpler. */
 	void *body;
 	uint32_t len;
-	uint16_t flags;
 	reiser4_key_t key;
 	reiser4_plug_t *plug;
 };
 
 enum node_flags {
 	NF_HEARD_BANSHEE  = 1 << 0,
+	NF_LAST
 };
 
 typedef enum node_flags node_flags_t;
@@ -313,7 +365,7 @@ struct reiser4_node {
 
 	/* Reference to tree if node is attached to tree. Sometimes node needs
 	   access tree and tree functions. */
-	void *tree;
+	tree_entity_t *tree;
 	
 	/* Place in parent node. */
 	reiser4_place_t p;
@@ -332,6 +384,9 @@ struct reiser4_node {
 	/* Key plugin. */
 	reiser4_plug_t *kplug;
 
+	/* Key policy. Optimisation to not call key->bodysize all the time. */
+	uint8_t keypol;
+	
 	/* Node state flags. */
 	uint32_t state;
 
@@ -345,35 +400,16 @@ struct reiser4_node {
 #endif
 };
 
-/* Object info struct contains the main information about a reiser4
-   object. These are: its key, parent key and coord of first item. */
-struct object_info {
-	void *tree;
-	reiser4_place_t start;
-	reiser4_key_t object;
-	reiser4_key_t parent;
-};
-
-typedef struct object_info object_info_t;
-
-/* Object plugin entity. */
-struct object_entity {
-	reiser4_plug_t *plug;
-	object_info_t info;
-};
-
-typedef struct object_entity object_entity_t;
-
 /* Stat data extension entity. */
-struct sdext_entity {
-	reiser4_plug_t *plug;
-
-	void *body;
-	uint32_t sdlen;
+struct stat_entity {
+	reiser4_place_t *place;
+	reiser4_plug_t *ext_plug;
 	uint32_t offset;
 };
 
-typedef struct sdext_entity sdext_entity_t;
+typedef struct stat_entity stat_entity_t;
+
+#define stat_body(stat) ((char *)(stat)->place->body + (stat)->offset)
 
 /* Shift flags control shift process */
 enum shift_flags {
@@ -452,7 +488,7 @@ struct ptr_hint {
 
 typedef struct ptr_hint ptr_hint_t;
 
-struct sdext_unix_hint {
+struct sdhint_unix {
 	uint32_t uid;
 	uint32_t gid;
 	uint32_t atime;
@@ -462,42 +498,50 @@ struct sdext_unix_hint {
 	uint64_t bytes;
 };
 
-typedef struct sdext_unix_hint sdext_unix_hint_t;
+typedef struct sdhint_unix sdhint_unix_t;
 
-struct sdext_lw_hint {
+struct sdhint_lw {
 	uint16_t mode;
 	uint32_t nlink;
 	uint64_t size;
 };
 
-typedef struct sdext_lw_hint sdext_lw_hint_t;
+typedef struct sdhint_lw sdhint_lw_t;
 
-struct sdext_lt_hint {
+struct sdhint_lt {
 	uint32_t atime;
 	uint32_t mtime;
 	uint32_t ctime;
 };
 
-typedef struct sdext_lt_hint sdext_lt_hint_t;
+typedef struct sdhint_lt sdhint_lt_t;
 
-struct sdext_flags_hint {
+struct sdhint_flags {
 	uint32_t flags;
 };
 
-typedef struct sdext_flags_hint sdext_flags_hint_t;
+typedef struct sdhint_flags sdhint_flags_t;
+
+struct sdhint_plug {
+	reiser4_plug_t *plug[OPSET_LAST];
+	uint64_t mask;
+};
+
+typedef struct sdhint_plug sdhint_plug_t;
+
+typedef sdhint_plug_t reiser4_opset_t;
 
 /* These fields should be changed to what proper description of needed
    extensions. */
-struct statdata_hint {
-	
+struct stat_hint {
 	/* Extensions mask */
 	uint64_t extmask;
     
 	/* Stat data extensions */
-	void *ext[60];
+	void *ext[SDEXT_LAST_ID];
 };
 
-typedef struct statdata_hint statdata_hint_t;
+typedef struct stat_hint stat_hint_t;
 
 enum entry_type {
 	ET_NAME	= 0,
@@ -506,53 +550,50 @@ enum entry_type {
 
 typedef enum entry_type entry_type_t;
 
+/* Object info struct contains the main information about a reiser4
+   object. These are: its key, parent key and coord of first item. */
+struct object_info {
+	reiser4_opset_t opset;
+	
+	tree_entity_t *tree;
+	reiser4_place_t start;
+	reiser4_key_t object;
+	reiser4_key_t parent;
+};
+
+typedef struct object_info object_info_t;
+
+/* Object plugin entity. */
+/*
+struct object_entity {
+	object_info_t info;
+	reiser4_plug_t *plug;
+};
+
+typedef struct object_entity object_entity_t;
+*/
+
+typedef object_info_t object_entity_t;
+
+
+
 /* Object hint. It is used to bring all about object information to object
    plugin to create appropriate object by it. */
 struct object_hint {
-
-	/* Stat data related fields. */
-	struct {
-		/* Additional mode to be masked to stat data mode. This is
-		   needed for special files, but mat be used somewhere else. */
-		uint32_t mode;
-		
-		/* Stata plugin id. */
-		rid_t statdata;
-	} label;
-
-	/* Hint for a file body */
-	union {
-
-		/* Plugin ids for the directory body */
-		struct {
-			rid_t hash;
-			rid_t fibre;
-			rid_t direntry;
-		} dir;
+	/* Object info. */
+	object_info_t info;
 	
-		/* Plugin id for the regular file body */
-		struct {
-			rid_t tail;
-			rid_t extent;
-			rid_t policy;
-		} reg;
-
-		/* rdev field for special file. Nevertheless, it is stored
-		   inside special file stat data field, we put it to @body
-		   definition, because it is the special file essence. */
-		struct {
-			uint64_t rdev;
-		} spl;
-
-		/* Symlink data */
-		char *sym;
-	} body;
-    
-	/* The plugin in use */
-	reiser4_plug_t *plug;
-
-	/* Parent object key. */
-	reiser4_key_t *parent;
+	/* Additional mode to be masked to stat data mode. This is
+	   needed for special files, but mat be used somewhere else. */
+	uint32_t mode;
+	
+	/* rdev field for special file. Nevertheless, it is stored
+	   inside special file stat data field, we put it to @body
+	   definition, because it is the special file essence. */
+	uint64_t rdev;
+	
+	/* SymLink name. */
+	char *name;
 };
 
 typedef struct object_hint object_hint_t;
@@ -569,13 +610,18 @@ typedef enum entity_state entity_state_t;
 typedef errno_t (*region_func_t) (void *, uint64_t,
 				  uint64_t, void *);
 
-/* Type for object place enumeration functions. */
+/* Type for on-place functions. */
 typedef errno_t (*place_func_t) (reiser4_place_t *, void *);
 
+/* Type for on-node functions. */
+typedef errno_t (*node_func_t) (reiser4_node_t *, void *);
+
 /* Function definitions for enumeration item metadata and data. */
 typedef errno_t (*layout_func_t) (void *, region_func_t, void *);
 typedef errno_t (*metadata_func_t) (void *, place_func_t, void *);
 
+#define REISER4_MAX_BLKSIZE (8192)
+
 struct entry_hint {
 	/* Entry metadata size. Filled by rem_entry and add_entry. */
 	uint16_t len;
@@ -590,21 +636,24 @@ struct entry_hint {
 	reiser4_key_t object;
 
 	/* Entry type (name or special), filled by readdir */
-	entry_type_t type;
+	uint8_t type;
 
 	/* Name of entry */
-	char name[256];
+	char name[REISER4_MAX_BLKSIZE];
 
+#ifndef ENABLE_STAND_ALONE
 	/* Hook called onto each create item during write flow. */
 	place_func_t place_func;
 
 	/* Related opaque data. May be used for passing something to
 	   region_func() and place_func(). */
 	void *data;
+#endif
 };
 
 typedef struct entry_hint entry_hint_t;
 
+#ifndef ENABLE_STAND_ALONE
 enum slink_type {
 	SL_UNLINK,   /* safe-link for unlink */
 	SL_TRUNCATE, /* safe-link for truncate */
@@ -626,6 +675,7 @@ struct slink_hint {
 };
 
 typedef struct slink_hint slink_hint_t;
+#endif
 
 /* This structure contains fields which describe an item or unit to be inserted
    into the tree. This is used for all tree modification purposes like
@@ -656,8 +706,8 @@ struct trans_hint {
 	/* Max real key. Set by estimate and needed for file body items. */
 	reiser4_key_t maxkey;
 
-	/* Flags specific for the operation, set at prepare stage. */
-	uint16_t merge_flags;
+	/* Flags specific for the insert (raw for now), set at prepare stage. */
+	uint16_t insert_flags;
 
 	/* Shift flags for shift operation. */
 	uint32_t shift_flags;
@@ -711,7 +761,7 @@ struct conv_hint {
 typedef struct conv_hint conv_hint_t;
 
 struct coll_hint {
-	int type;
+	uint8_t type;
 	void *specific;
 };
 
@@ -729,7 +779,9 @@ typedef enum lookup_bias lookup_bias_t;
 
 typedef struct lookup_hint lookup_hint_t;
 
-typedef lookup_t (*coll_func_t) (void *, reiser4_place_t *, coll_hint_t *);
+typedef lookup_t (*coll_func_t) (tree_entity_t *, 
+				 reiser4_place_t *, 
+				 coll_hint_t *);
 
 /* Hint to be used when looking for data in tree. */
 struct lookup_hint {
@@ -749,9 +801,16 @@ struct lookup_hint {
 #endif
 };
 
+struct repair_hint {
+	int64_t len;
+	uint8_t mode;
+};
+
+typedef struct repair_hint repair_hint_t;
+
 /* Filesystem description. */
 struct fs_desc {
-	uint16_t policy;
+	reiser4_plug_t *policy;
 	uint32_t blksize;
 	aal_device_t *device;
 };
@@ -759,10 +818,6 @@ struct fs_desc {
 typedef struct fs_desc fs_desc_t;
 
 struct reiser4_key_ops {
-	/* Cleans key up. Actually it just memsets it by zeros, but more smart
-	   behavior may be implemented. */
-	void (*clean) (reiser4_key_t *);
-
 	/* Function for dermining is key contains direntry name hashed or
 	   not? */
 	int (*hashed) (reiser4_key_t *);
@@ -803,14 +858,21 @@ struct reiser4_key_ops {
 	errno_t (*build_hashed) (reiser4_key_t *, reiser4_plug_t *,
 				 reiser4_plug_t *, uint64_t, uint64_t, char *);
 	
+#ifndef ENABLE_STAND_ALONE
+	
 	/* Gets/sets key type (minor in reiser4 notation). */	
 	void (*set_type) (reiser4_key_t *, key_type_t);
 	key_type_t (*get_type) (reiser4_key_t *);
 
+	/* Gets/sets key full objectid */
+	void (*set_fobjectid) (reiser4_key_t *, uint64_t);
+	uint64_t (*get_fobjectid) (reiser4_key_t *);
+	
 	/* Gets/sets key locality. */
 	void (*set_locality) (reiser4_key_t *, uint64_t);
+#endif
 	uint64_t (*get_locality) (reiser4_key_t *);
-    
+	
 	/* Gets/sets key locality. */
 	void (*set_ordering) (reiser4_key_t *, uint64_t);
 	uint64_t (*get_ordering) (reiser4_key_t *);
@@ -819,10 +881,6 @@ struct reiser4_key_ops {
 	void (*set_objectid) (reiser4_key_t *, uint64_t);
 	uint64_t (*get_objectid) (reiser4_key_t *);
 
-	/* Gets/sets key full objectid */
-	void (*set_fobjectid) (reiser4_key_t *, uint64_t);
-	uint64_t (*get_fobjectid) (reiser4_key_t *);
-
 	/* Gets/sets key offset */
 	void (*set_offset) (reiser4_key_t *, uint64_t);
 	uint64_t (*get_offset) (reiser4_key_t *);
@@ -848,7 +906,7 @@ typedef struct reiser4_key_ops reiser4_key_ops_t;
 
 struct reiser4_object_ops {
 	/* Loads object stat data to passed hint. */
-	errno_t (*stat) (object_entity_t *, statdata_hint_t *);
+	errno_t (*stat) (object_entity_t *, stat_hint_t *);
 
 #ifndef ENABLE_STAND_ALONE
 	/* These methods change @nlink value of passed @entity. */
@@ -861,10 +919,10 @@ struct reiser4_object_ops {
 	errno_t (*detach) (object_entity_t *, object_entity_t *);
 
 	/* Updates object stat data from passed hint. */
-	errno_t (*update) (object_entity_t *, statdata_hint_t *);
+	errno_t (*update) (object_entity_t *, stat_hint_t *);
 	
 	/* Creates new file with passed parent and object keys. */
-	object_entity_t *(*create) (object_info_t *, object_hint_t *);
+	object_entity_t *(*create) (object_hint_t *);
 
 	/* Delete file body and stat data if any. */
 	errno_t (*clobber) (object_entity_t *);
@@ -907,9 +965,6 @@ struct reiser4_object_ops {
 	/* Creates the fake object by the gived @info. Needed to recover "/" and
 	   "lost+found" direcories if their SD are broken. */
 	object_entity_t *(*fake) (object_info_t *);
-
-	/* Forms the correct object from what was openned/checked. */
-	errno_t (*form) (object_entity_t *);
 #endif
 	
 	/* Change current position to passed value. */
@@ -928,8 +983,7 @@ struct reiser4_object_ops {
 	uint64_t (*offset) (object_entity_t *);
 
 	/* Makes lookup inside file */
-	lookup_t (*lookup) (object_entity_t *, char *,
-			    entry_hint_t *);
+	lookup_t (*lookup) (object_entity_t *, char *, entry_hint_t *);
 
 	/* Finds actual file stat data (used in symlinks). */
 	errno_t (*follow) (object_entity_t *, reiser4_key_t *,
@@ -993,9 +1047,6 @@ struct item_balance_ops {
 typedef struct item_balance_ops item_balance_ops_t;
 
 struct item_object_ops {
-	/* Get plugin id of specified type if stored in stat data item. */
-	rid_t (*object_plug) (reiser4_place_t *, rid_t);
-		
 	/* Reads passed amount of bytes from the item. */
 	int64_t (*read_units) (reiser4_place_t *, trans_hint_t *);
 		
@@ -1041,35 +1092,35 @@ struct item_object_ops {
 
 typedef struct item_object_ops item_object_ops_t;
 
-struct item_repair_ops {
 #ifndef ENABLE_STAND_ALONE
+struct item_repair_ops {
 	/* Estimate merge operation. */
-	errno_t (*prep_merge) (reiser4_place_t *, trans_hint_t *);
+	errno_t (*prep_insert_raw) (reiser4_place_t *, trans_hint_t *);
 
 	/* Copies some amount of units from @src to @dst with partial
 	   overwritting. */
-	errno_t (*merge) (reiser4_place_t *, trans_hint_t *);
+	errno_t (*insert_raw) (reiser4_place_t *, trans_hint_t *);
 
 	/* Checks the item structure. */
-	errno_t (*check_struct) (reiser4_place_t *, uint8_t);
+	errno_t (*check_struct) (reiser4_place_t *, repair_hint_t *);
 	
 	/* Does some specific actions if a block the item points to is wrong. */
-	errno_t (*check_layout) (reiser4_place_t *, region_func_t,
-				 void *, uint8_t);
+	errno_t (*check_layout) (reiser4_place_t *, repair_hint_t *,
+				 region_func_t, void *);
 
 	errno_t (*pack) (reiser4_place_t *, aal_stream_t *);
 	errno_t (*unpack) (reiser4_place_t *, aal_stream_t *);
-#endif
 };
+#endif
 
 typedef struct item_repair_ops item_repair_ops_t;
 
-struct item_debug_ops {
 #ifndef ENABLE_STAND_ALONE
+struct item_debug_ops {
 	/* Prints item into specified buffer. */
 	void (*print) (reiser4_place_t *, aal_stream_t *, uint16_t);
-#endif
 };
+#endif
 
 typedef struct item_debug_ops item_debug_ops_t;
 
@@ -1087,10 +1138,12 @@ typedef struct item_tree_ops item_tree_ops_t;
 
 struct reiser4_item_ops {
 	item_tree_ops_t *tree;
-	item_debug_ops_t *debug;
 	item_object_ops_t *object;
-	item_repair_ops_t *repair;
 	item_balance_ops_t *balance;
+#ifndef ENABLE_STAND_ALONE
+	item_debug_ops_t *debug;
+	item_repair_ops_t *repair;
+#endif
 };
 
 typedef struct reiser4_item_ops reiser4_item_ops_t;
@@ -1099,19 +1152,19 @@ typedef struct reiser4_item_ops reiser4_item_ops_t;
 struct reiser4_sdext_ops {
 #ifndef ENABLE_STAND_ALONE
 	/* Initialize stat data extension data at passed pointer. */
-	errno_t (*init) (void *, void *);
+	errno_t (*init) (stat_entity_t *, void *);
 
 	/* Prints stat data extension data into passed buffer. */
-	void (*print) (void *, aal_stream_t *, uint16_t);
+	void (*print) (stat_entity_t *, aal_stream_t *, uint16_t);
 
 	/* Checks sd extension content. */
-	errno_t (*check_struct) (sdext_entity_t *, uint8_t);
+	errno_t (*check_struct) (stat_entity_t *, repair_hint_t *);
 #endif
 	/* Reads stat data extension data. */
-	errno_t (*open) (void *, void *);
+	errno_t (*open) (stat_entity_t *, void *);
 
 	/* Returns length of the extension. */
-	uint16_t (*length) (void *);
+	uint32_t (*length) (stat_entity_t *, void *);
 };
 
 typedef struct reiser4_sdext_ops reiser4_sdext_ops_t;
@@ -1176,8 +1229,8 @@ struct reiser4_node_ops {
 
 	/* Merge 2 items -- insert/overwrite @src_entity parts to
 	   @dst_entity. */
-	errno_t (*merge) (reiser4_node_t *, pos_t *, 
-			  trans_hint_t *);
+	errno_t (*insert_raw) (reiser4_node_t *, pos_t *, 
+			       trans_hint_t *);
 
 	/* Copies items from @src_entity to @dst_entity. */
 	errno_t (*copy) (reiser4_node_t *, pos_t *,
@@ -1432,6 +1485,9 @@ struct reiser4_alloc_ops {
 
 	/* Checks blocks allocator on validness */
 	errno_t (*valid) (generic_entity_t *);
+	
+	/* Checks blocks allocator on validness */
+	errno_t (*check_struct) (generic_entity_t *, uint8_t mode);
 
 	/* Prints block allocator data */
 	void (*print) (generic_entity_t *, aal_stream_t *, uint16_t);
@@ -1524,7 +1580,6 @@ struct reiser4_policy_ops {
 typedef struct reiser4_policy_ops reiser4_policy_ops_t;
 #endif
 
-#define PLUG_MAX_PATH   1024
 #define PLUG_MAX_DESC	64
 #define PLUG_MAX_LABEL	22
 
@@ -1543,11 +1598,6 @@ struct plug_class {
 	
 	/* Plugin finalization routine. */
 	plug_fini_t fini;
-
-#ifndef ENABLE_STAND_ALONE
-	/* Plugin location, that is address of its init() function. */
-	char loc[PLUG_MAX_PATH];
-#endif
 };
 
 typedef struct plug_class plug_class_t;
@@ -1555,17 +1605,13 @@ typedef struct plug_class plug_class_t;
 struct plug_ident {
 	/* Plugin id, type and group. */
 	rid_t id;
-	rid_t group;
-	rid_t type;
+	uint8_t group;
+	uint8_t type;
 };
 
 typedef struct plug_ident plug_ident_t;
 
-#ifndef ENABLE_STAND_ALONE
-#  define class_init {NULL, NULL, ""}
-#else
-#  define class_init {NULL, NULL}
-#endif
+#define class_init {NULL, NULL}
 
 struct reiser4_plug {
 	/* Plugin class. This will be used by plugin factory for initializing
@@ -1615,17 +1661,17 @@ struct reiser4_plug {
 
 struct flow_ops {
 	/* Reads data from the tree. */
-	int64_t (*read) (void *, trans_hint_t *);
+	int64_t (*read) (tree_entity_t *, trans_hint_t *);
 
 #ifndef ENABLE_STAND_ALONE
 	/* Writes data to tree. */
-	int64_t (*write) (void *, trans_hint_t *);
+	int64_t (*write) (tree_entity_t *, trans_hint_t *);
 
 	/* Truncates data from tree. */
-	int64_t (*truncate) (void *, trans_hint_t *);
+	int64_t (*truncate) (tree_entity_t *, trans_hint_t *);
 	
 	/* Convert some particular place to another plugin. */
-	errno_t (*convert) (void *, conv_hint_t *);
+	errno_t (*convert) (tree_entity_t *, conv_hint_t *);
 #endif
 };
 
@@ -1634,31 +1680,33 @@ typedef struct flow_ops flow_ops_t;
 struct tree_ops {
 	/* Makes lookup in the tree in order to know where say stat data item of
 	   a file realy lies. It is used in all object plugins. */
-	lookup_t (*lookup) (void *, lookup_hint_t *, lookup_bias_t,
-			    reiser4_place_t *);
+	lookup_t (*lookup) (tree_entity_t *, lookup_hint_t *, 
+			    lookup_bias_t, reiser4_place_t *);
 
 #ifndef ENABLE_STAND_ALONE
 	/* Collisions handler. It takes start place and looks for actual data in
 	   collided array. */
-	lookup_t (*collision) (void *, reiser4_place_t *, coll_hint_t *);
+	lookup_t (*collision) (tree_entity_t *, reiser4_place_t *, 
+			       coll_hint_t *);
 	
 	/* Inserts item/unit in the tree by calling tree_insert() function, used
 	   by all object plugins (dir, file, etc). */
-	int64_t (*insert) (void *, reiser4_place_t *,
+	int64_t (*insert) (tree_entity_t *, reiser4_place_t *,
 			   trans_hint_t *, uint8_t);
 
 	/* Removes item/unit from the tree. It is used in all object plugins for
 	   modification purposes. */
-	errno_t (*remove) (void *, reiser4_place_t *, trans_hint_t *);
+	errno_t (*remove) (tree_entity_t *, reiser4_place_t *, trans_hint_t *);
 	
 	/* Update the key in the place and the node itsef. */
-	errno_t (*update_key) (void *, reiser4_place_t *, reiser4_key_t *);
+	errno_t (*update_key) (tree_entity_t *, reiser4_place_t *, reiser4_key_t *);
 
 	/* Get the safe link locality. */
-	uint64_t (*slink_locality) (void *);
+	uint64_t (*slink_locality) (tree_entity_t *);
 #endif
 	/* Returns the next item. */
-	errno_t (*next_item) (void *, reiser4_place_t *, reiser4_place_t *);
+	errno_t (*next_item) (tree_entity_t *, reiser4_place_t *, 
+			      reiser4_place_t *);
 };
 
 typedef struct tree_ops tree_ops_t;
@@ -1666,32 +1714,30 @@ typedef struct tree_ops tree_ops_t;
 struct factory_ops {
 	/* Finds plugin by its attributes (type and id). */
 	reiser4_plug_t *(*ifind) (rid_t, rid_t);
-	
-#ifndef ENABLE_STAND_ALONE
-	/* Finds plugin by its type and name. */
-	reiser4_plug_t *(*nfind) (char *);
-#endif
 };
 
 typedef struct factory_ops factory_ops_t;
 
 #ifdef ENABLE_SYMLINKS
 struct object_ops {
-	errno_t (*resolve) (void *, char *, reiser4_key_t *,
-			    reiser4_key_t *);
+	errno_t (*resolve) (tree_entity_t *, char *, 
+			    reiser4_key_t *, reiser4_key_t *);
 };
 
 typedef struct object_ops object_ops_t;
 #endif
 
+struct pset_ops {
+	/* Obtains the plugin from the profile by its profile index. */
+	reiser4_plug_t *(*find) (rid_t, rid_t);
 #ifndef ENABLE_STAND_ALONE
-struct param_ops {
-	/* Obtains the param value by its name. */
-	uint64_t (*value) (char *);
+	void (*diff) (tree_entity_t *, reiser4_opset_t *);
+#endif
 };
 
-typedef struct param_ops param_ops_t;
+typedef struct pset_ops pset_ops_t;
 
+#ifndef ENABLE_STAND_ALONE
 struct key_ops {
 	char *(*print) (reiser4_key_t *, uint16_t);
 };
@@ -1712,11 +1758,8 @@ struct reiser4_core {
 	flow_ops_t flow_ops;
 	tree_ops_t tree_ops;
 	factory_ops_t factory_ops;
-	
-#ifndef ENABLE_STAND_ALONE
-	param_ops_t param_ops;
-#endif
-	
+	pset_ops_t pset_ops;
+
 #ifdef ENABLE_SYMLINKS
 	object_ops_t object_ops;
 #endif
