@@ -8,10 +8,9 @@
 
 #include <reiser4/types.h>
 
-extern reiser4_plug_t *reiser4_semantic_plug(reiser4_tree_t *tree,
-					     reiser4_place_t *place);
+extern reiser4_object_t *reiser4_semantic_open(reiser4_tree_t *tree,
+					       char *path, 
+					       reiser4_key_t *from,
+					       bool_t follow);
 
-extern object_entity_t *reiser4_semantic_resolve(reiser4_tree_t *tree,
-						 char *path, reiser4_key_t *from,
-						 bool_t follow);
 #endif
