@@ -13,7 +13,7 @@
 /* Minimal block number needed for a reiser4 filesystem: Master, Format40,
    Bitmap, JHeader, JFooter, Status, Backup, Twig, Leaf + skipped ones. */
 #define REISER4_FS_MIN_SIZE(blksize) \
-	(9 + REISER4_MASTER_OFFSET / blksize)
+	(9 + REISER4_MASTER_BLOCKNR(blksize))
 
 /* Master super block structure. It is the same for all reiser4 filesystems,
    so, we can declare it here. It contains common for all format fields like
@@ -123,46 +123,14 @@ struct reiser4_status {
 
 typedef struct reiser4_status reiser4_status_t;
 
-enum reiser4_pid_flags {
-	PF_OVERRIDDEN = 0x0,
-	PF_READ       = 0x1
-};
-
-struct reiser4_pid {
-	char name[255];
-	uint32_t type;
-	uint64_t value;
-	uint8_t flags;
-};
-
-typedef struct reiser4_pid reiser4_pid_t;
-
-#define PARAM_NR 20
-
-/* Profile structure. It describes what plugins will be used for every part of
-   the filesystem. */
-struct reiser4_param {
-	char name[10];
-	reiser4_pid_t pid[PARAM_NR];
-};
-
-typedef struct reiser4_param reiser4_param_t;
 typedef struct reiser4_tree reiser4_tree_t;
 
-#define OBJECT_NAME_SIZE 1024
-
 /* Reiser4 file structure (regular file, directory, symlinks, etc) */
 struct reiser4_object {
-	/* Object info pointer */
-	object_info_t *info;
-	
 	/* Object entity. It is initialized by object plugin */
-	object_entity_t *entity;
+	object_entity_t *ent;
 
 #ifndef ENABLE_STAND_ALONE
-	/* Full file name or printed key */
-	char name[OBJECT_NAME_SIZE];
-
 	/* Applications using this library sometimes need to embed information
 	   into the objects of our library for their own use. */
 	void *data;
@@ -171,13 +139,13 @@ struct reiser4_object {
 
 typedef struct reiser4_object reiser4_object_t;
 
+#define objplug(object) (object->ent->opset.plug[OPSET_OBJ])
+
 /* Calback types used in object code. */
-typedef errno_t (*object_init_t) (reiser4_object_t *, 
-				  reiser4_object_t *);
+typedef object_entity_t *(*object_init_t) (object_info_t *);
 
 typedef reiser4_object_t *(*object_open_func_t) (reiser4_object_t *, 
-						 entry_hint_t *, 
-						 void *);
+						 entry_hint_t *, void *);
 
 #ifndef ENABLE_STAND_ALONE
 enum reiser4_owner {
@@ -201,7 +169,7 @@ struct reiser4_format {
 	
 	/* Disk-format entity. It is initialized by disk-format plugin during
 	   initialization. */
-	generic_entity_t *entity;
+	generic_entity_t *ent;
 };
 
 typedef struct reiser4_format reiser4_format_t;
@@ -218,7 +186,7 @@ struct reiser4_journal {
 	aal_device_t *device;
 
 	/* Journal entity. Initializied by plugin */
-	generic_entity_t *entity;
+	generic_entity_t *ent;
 };
 
 typedef struct reiser4_journal reiser4_journal_t;
@@ -231,7 +199,7 @@ typedef errno_t (*hook_alloc_t) (reiser4_alloc_t *,
 /* Block allocator structure */
 struct reiser4_alloc {
 	reiser4_fs_t *fs;
-	generic_entity_t *entity;
+	generic_entity_t *ent;
 
 	struct {
 		hook_alloc_t alloc;
@@ -245,7 +213,7 @@ struct reiser4_alloc {
 /* Oid allocator structure */
 struct reiser4_oid {
 	reiser4_fs_t *fs;
-	generic_entity_t *entity;
+	generic_entity_t *ent;
 };
 
 typedef struct reiser4_oid reiser4_oid_t;
@@ -262,6 +230,8 @@ typedef int (*mpc_func_t) (reiser4_tree_t *);
 
 /* Tree structure. */
 struct reiser4_tree {
+	tree_entity_t ent;
+	
 	/* Flag that shows, that tree adjusting is running now and should not be
 	   called again until this flag is turned off. */
 	int adjusting;
@@ -278,22 +248,6 @@ struct reiser4_tree {
 	/* Memory pressure detect function. */
 	mpc_func_t mpc_func;
 
-#ifndef ENABLE_STAND_ALONE
-	/* Minimal tree level we have to allocate something on it. In current
-	   tree implementation this is TWIG_LEVEL, that is LEAF_LEVEL + 1. In
-	   tree like reiser3 uses it is LEAF_LEVEL.
-
-	   Few words about why do we need it. We need it not because of
-	   reiser3. We really don't think, that libreiser4 will support reiser3
-	   some day, we need it just to make tree code more flexible and avoid
-	   to use any kind of hardcoding. */
-	uint32_t bottom;
-
-	/* Saved nodeptr plugin is stored here. This is needed for speedup
-	   attaching new nodes to tree. */
-	reiser4_plug_t *nodeptr;
-#endif
-	
 	/* Formatted nodes hash table. */
 	aal_hash_table_t *nodes;
 
@@ -303,19 +257,13 @@ struct reiser4_tree {
 #endif
 };
 
+
 #ifndef ENABLE_STAND_ALONE
+
 /* Callback function type for opening node. */
 typedef reiser4_node_t *(*tree_open_func_t) (reiser4_tree_t *, 
-				     reiser4_place_t *, void *);
-
-/* Callback function type for preparing per-node traverse data. */
-typedef errno_t (*tree_edge_func_t) (reiser4_tree_t *, 
-				     reiser4_node_t *, void *);
-
-/* Callback function type for preparing per-item traverse data. */
-typedef errno_t (*tree_update_func_t) (reiser4_tree_t *, 
-				       reiser4_place_t *,
-				       void *);
+					     reiser4_place_t *, 
+					     void *);
 #endif
 
 /* Filesystem compound structure */
@@ -370,6 +318,7 @@ typedef struct fs_hint fs_hint_t;
 
 typedef void (*uuid_unparse_t) (char *uuid, char *string);
 typedef errno_t (*walk_func_t) (reiser4_tree_t *, reiser4_node_t *);
+typedef errno_t (*walk_on_func_t) (reiser4_tree_t *, reiser4_place_t *);
 
 /* Number of bit to test it in format flags in order check if large keys policy
    in use. Large keys in use if bit is set. */
