@@ -15,6 +15,8 @@ typedef struct repair_ds_stat {
 	uint64_t read_nodes;
 	uint64_t good_nodes, good_leaves, good_twigs;
 	uint64_t fixed_nodes, fixed_leaves, fixed_twigs;
+
+	uint64_t *files, tmp;
 	time_t time;
 } repair_ds_stat_t;
 
@@ -28,10 +30,7 @@ typedef struct repair_ds {
 	aux_bitmap_t *bm_leaf;	/* Found leaves. */
 	aux_bitmap_t *bm_twig;	/* Fount twigs. */
 
-	repair_progress_handler_t *progress_handler;    
-	repair_progress_t *progress;
 	repair_ds_stat_t stat;
-
 	bool_t *check_node;
 } repair_ds_t;
 
