@@ -14,8 +14,10 @@ extern reiser4_node_t *repair_node_open(reiser4_tree_t *tree,
 extern errno_t repair_node_check_level(reiser4_node_t *node,
 				       uint8_t mode);
 
-extern errno_t repair_node_check_struct(reiser4_node_t *node,
-					uint8_t mode);
+extern errno_t repair_node_check_struct(reiser4_node_t *node, 
+					place_func_t func,
+					uint8_t mode,
+					void *data);
 
 extern errno_t repair_node_clear_flags(reiser4_node_t *node);
 
