@@ -7,147 +7,56 @@
 #include <aux/aux.h>
 #include <reiser4/libreiser4.h>
 
-/* Helper funtion, which initializes @object->entity by @object->info. */
-static errno_t reiser4_object_init(reiser4_object_t *object,
-				   reiser4_object_t *parent)
-{
-	reiser4_place_t *place;
-	reiser4_plug_t *plug;
-
-	aal_assert("umka-2380", object != NULL);
-
-	/* Getting object plugin by first item coord. */
-	place = object_start(object);
+/* Init object pset by its SD. */
+errno_t reiser4_object_init(object_info_t *info) {
+	sdhint_plug_t plugh;
+	trans_hint_t trans;
+	stat_hint_t stat;
+	errno_t res;
 	
-	if (!(plug = reiser4_semantic_plug(object->info->tree, place)))
+	aal_assert("umka-2380", info != NULL);
+
+	aal_memset(&stat, 0, sizeof(stat));
+	aal_memset(&plugh, 0, sizeof(plugh));
+	
+	/* Preparing hint and mask */
+	trans.specific = &stat;
+	trans.place_func = NULL;
+	trans.region_func = NULL;
+	trans.shift_flags = SF_DEFAULT;
+	stat.ext[SDEXT_PLUG_ID] = &plugh;
+	
+	if (reiser4_place_valid(&info->start)) {
+		if ((res = reiser4_place_fetch(&info->start)))
+			return res;
+
+		/* Init is allowed on a SD item only. */
+		if (info->start.plug->id.group != STAT_ITEM)
+			return -EINVAL;
+	} else {
+		/* Init is allowed on a SD item only. */
 		return -EINVAL;
-	    
+	}
+	
+	/* Getting object plugin by first item coord. */
+	if ((res = plug_call(info->start.plug->o.item_ops->object,
+			     fetch_units, &info->start, &trans)) != 1)
+		return res;
+	
+	aal_memcpy(&info->opset.plug, &plugh, sizeof(plugh));
+	
+	/* Object plugin must be detected. */
+	return info->opset.plug[OPSET_OBJ] ? 0 : -EINVAL;
+}
+
+/* Helper funtion, which initializes @object->ent by @object->info. */
+object_entity_t *reiser4_object_recognize(object_info_t *info) {
+	if (reiser4_object_init(info))
+		return INVAL_PTR;
+
 	/* Requesting object plugin to open the object on passed @tree and
 	   @place. If it fails, we will continue lookup. */
-	if (!(object->entity = plug_call(plug->o.object_ops,
-					 open, object->info)))
-	{
-		return -EINVAL;
-	}
-
-	return 0;
-}
-
-/* Closes object entity. */
-static void reiser4_object_fini(reiser4_object_t *object) {
-	plug_call(object->entity->plug->o.object_ops,
-		  close, object->entity);
-	
-	object->entity = NULL;
-}
-
-/* Returns object size. That is stat data field st_size. Actually it might be
-   got by means of using object_stat() function, but, we implemented this
-   function as helper, because using object_stat() is rather complicated due to
-   somplex initializing stat data extensions to be loaded by it. */
-uint64_t reiser4_object_size(reiser4_object_t *object) {
-	statdata_hint_t hint;
-	sdext_lw_hint_t lw_hint;
-	
-	aal_assert("umka-1961", object != NULL);
-
-	/* Initializing stat data hint. And namely extension mask of extension
-	   slot we are interested in. Size lies in light weight extension. */
-
-	/* FIXME-UMKA: Why object (on API abstraction level) knows, that size
-	   lies in LW extension? What if someone will move it to another one? */
-	hint.extmask = 1 << SDEXT_LW_ID;
-	hint.ext[SDEXT_LW_ID] = &lw_hint;
-
-	/* Calling objects stat() method. */
-	if (plug_call(object->entity->plug->o.object_ops,
-		      stat, object->entity, &hint))
-	{
-		return 0;
-	}
-
-	return lw_hint.size;
-}
-
-/* Updates object stat data coord by means of using tree_lookup(). */
-errno_t reiser4_object_refresh(reiser4_object_t *object) {
-	lookup_hint_t hint;
-	object_info_t *info;
-
-	info = object->info;
-	hint.level = LEAF_LEVEL;
-	hint.key = &info->object;
-
-#ifndef ENABLE_STAND_ALONE
-	hint.collision = NULL;
-#endif
-
-	switch (reiser4_tree_lookup(info->tree, &hint, FIND_EXACT,
-				    object_start(object)))
-	{
-	case PRESENT:
-		return 0;
-	default:
-		return -EINVAL;
-	}
-}
-
-/* Resolve passed @path. */
-errno_t reiser4_object_resolve(reiser4_object_t *object,
-			       reiser4_tree_t *tree,
-			       char *path, bool_t follow)
-{
-	aal_assert("umka-2247", path != NULL);
-	aal_assert("umka-2246", object != NULL);
-	aal_assert("vpf-1422", tree != NULL);
-
-	/* Resolving object by @path starting from root key. */
-	if (!(object->entity = reiser4_semantic_resolve(tree, path,
-							&tree->key,
-							follow)))
-	{
-		return -EINVAL;
-	}
-
-	/* Assigning info reference to entity info instance */
-	object->info = &object->entity->info;
-	
-	return 0;
-}
-
-/* This function opens object by its name */
-reiser4_object_t *reiser4_object_open(
-	reiser4_tree_t *tree,		/* tree object will be opened on */
-	char *path,                     /* name of object to be opened */
-	bool_t follow)                  /* follow symlinks */
-{
-#ifndef ENABLE_STAND_ALONE
-	char *name;
-#endif
-	reiser4_object_t *object;
-    
-	aal_assert("umka-678", tree != NULL);
-	aal_assert("umka-789", path != NULL);
-
-	if (!(object = aal_calloc(sizeof(*object), 0)))
-		return NULL;
-    
-	/* Semantic resolve of @path. */
-	if (reiser4_object_resolve(object, tree, path, follow))
-		goto error_free_object;
-	
-
-	/* Initializing object name. It is stat data key as string. */
-#ifndef ENABLE_STAND_ALONE
-	name = reiser4_print_key(&object->info->object, PO_INODE);
-	aal_strncpy(object->name, name, sizeof(object->name));
-#endif
-
-	return object;
-    
- error_free_object:
-	aal_free(object);
-	return NULL;
+	return plug_call(info->opset.plug[OPSET_OBJ]->o.object_ops, open, info);
 }
 
 /* Tries to open object at @place. Uses @init_func for initializing object
@@ -155,16 +64,14 @@ reiser4_object_t *reiser4_object_open(
    entity initializing and librepair another one, but both they use some amount
    of common code, which was moved to this function and used by both in such a
    manner. */
-reiser4_object_t *reiser4_object_guess(reiser4_tree_t *tree,
-				       reiser4_object_t *parent,
-				       reiser4_key_t *okey,
-				       reiser4_place_t *place,
-				       object_init_t init_func)
-
+reiser4_object_t *reiser4_object_form(reiser4_tree_t *tree,
+				      reiser4_object_t *parent,
+				      reiser4_key_t *okey,
+				      reiser4_place_t *place,
+				      object_init_t init_func)
 {
-	errno_t res;
-	object_info_t info;
 	reiser4_object_t *object;
+	object_info_t info;
 	
 	aal_assert("umka-1508", tree != NULL);
 	aal_assert("umka-1509", place != NULL);
@@ -177,63 +84,48 @@ reiser4_object_t *reiser4_object_guess(reiser4_tree_t *tree,
 		return INVAL_PTR;
 
 	/* Initializing object info. */
-	object->info = &info;
-	object->info->tree = tree;
-
-	/* Parent is not passed. Using object's key as parent's one. */
-	if (parent) {
-		reiser4_key_assign(&object->info->parent, 
-				   &parent->info->object);
-	}
+	info.tree = (tree_entity_t *)tree;
 
 	/* Putting object key to info struct. We may want to open and fix 
 	   the object even if @place->key does not match @okey. */
-	reiser4_key_assign(&object->info->object, okey);
+	reiser4_key_assign(&info.object, okey);
 
 	/* Copying item coord. */
-	aal_memcpy(&object->info->start, place, sizeof(*place));
+	aal_memcpy(&info.start, place, sizeof(*place));
+
+	/* Parent is not passed. Using object's key as parent's one. */
+	if (parent)
+		reiser4_key_assign(&info.parent, &parent->ent->object);
 
 	/* Calling @init_func. It returns zero for success. */
-	if ((res = init_func(object, parent)))
+	if (!(object->ent = init_func(&info)) || object->ent == INVAL_PTR)
 		goto error_free_object;
-
-	object->info = &object->entity->info;
-
-	/* Initializing object name. */
-#ifndef ENABLE_STAND_ALONE
-	{
-		char *name = reiser4_print_key(&object->info->object,
-					       PO_INODE);
-		
-		aal_strncpy(object->name, name, sizeof(object->name));
-	}
-#endif
 	
 	return object;
-	
+
  error_free_object:
 	aal_free(object);
 	return NULL;
 }
 
 /* This function opens object by its @place. */
-reiser4_object_t *reiser4_object_realize(reiser4_tree_t *tree, 
-					 reiser4_object_t *parent,
-					 reiser4_place_t *place)
+reiser4_object_t *reiser4_object_open(reiser4_tree_t *tree, 
+				      reiser4_object_t *parent,
+				      reiser4_place_t *place)
 {
 	reiser4_object_t *object;
 	
 	aal_assert("vpf-1223", place != NULL);
-	
-	object = reiser4_object_guess(tree, parent, &place->key,
-				      place, reiser4_object_init);
+
+	object = reiser4_object_form(tree, parent, &place->key, place, 
+				     reiser4_object_recognize);
 
 	return (object == INVAL_PTR ? NULL : object);
 }
 
-#ifndef ENABLE_STAND_ALONE
-/* Try to open the object on the base of the given key. */
-reiser4_object_t *reiser4_object_launch(reiser4_tree_t *tree,
+/* Try to open the object on the base of the given key. 
+   Lookup by @key + object_form. */
+reiser4_object_t *reiser4_object_obtain(reiser4_tree_t *tree,
 					reiser4_object_t *parent,
 					reiser4_key_t *key) 
 {
@@ -245,46 +137,73 @@ reiser4_object_t *reiser4_object_launch(reiser4_tree_t *tree,
 
 	hint.key = key;
 	hint.level = LEAF_LEVEL;
+#ifndef ENABLE_STAND_ALONE
 	hint.collision = NULL;
-	
-	switch (reiser4_tree_lookup(tree, &hint, FIND_EXACT, &place)) {
-	case PRESENT:
-		/* The key must point to the start of the object. */
-		if (reiser4_key_compfull(&place.key, key))
-			return NULL;
-	
-		/* If the pointed item was found, object must be
-		   openable. @parent probably should be passed here. */
-		return reiser4_object_realize(tree, parent, &place);
-	default:
+#endif
+
+	if (reiser4_tree_lookup(tree, &hint, FIND_EXACT, &place) != PRESENT)
 		return NULL;
-	}
+
+	/* The key must point to the start of the object. */
+	if (reiser4_key_compfull(&place.key, key))
+		return NULL;
+
+	/* If the pointed item was found, object must be
+	   openable. @parent probably should be passed here. */
+	return reiser4_object_open(tree, parent, &place);
 }
 
-errno_t reiser4_object_truncate(
-	reiser4_object_t *object,           /* object for truncating */
-	uint64_t n)			    /* the number of entries */
+/* Returns object size. That is stat data field st_size. Actually it might be
+   got by means of using object_stat() function, but, we implemented this
+   function as helper, because using object_stat() is rather complicated due to
+   somplex initializing stat data extensions to be loaded by it. */
+uint64_t reiser4_object_size(reiser4_object_t *object) {
+	stat_hint_t hint;
+	sdhint_lw_t lwh;
+
+	aal_assert("umka-1961", object != NULL);
+
+	/* Initializing stat data hint. And namely extension mask of extension
+	   slot we are interested in. Size lies in light weight extension. */
+
+	/* FIXME-UMKA: Why object (on API abstraction level) knows, that size
+	   lies in LW extension? What if someone will move it to another one? */
+	hint.extmask = 1 << SDEXT_LW_ID;
+	hint.ext[SDEXT_LW_ID] = &lwh;
+
+	/* Calling objects stat() method. */
+	if (plug_call(objplug(object)->o.object_ops,
+		      stat, object->ent, &hint))
+		return 0;
+
+	return lwh.size;
+}
+
+/* Closes specified object */
+void reiser4_object_close(
+	reiser4_object_t *object)    /* object to be closed */
 {
-	aal_assert("umka-1154", object != NULL);
-	aal_assert("umka-1155", object->entity != NULL);
-    
-	return plug_call(object->entity->plug->o.object_ops, 
-			 truncate, object->entity, n);
+	aal_assert("umka-680", object != NULL);
+	aal_assert("umka-1149", object->ent != NULL);
+
+	plug_call(objplug(object)->o.object_ops, close, object->ent);
+	aal_free(object);
 }
 
+#ifndef ENABLE_STAND_ALONE
 /* Adds @entry to @object */
 errno_t reiser4_object_add_entry(
 	reiser4_object_t *object,           /* object for adding entry */
 	entry_hint_t *entry)                /* entry hint to be added */
 {
 	aal_assert("umka-1975", object != NULL);
-	aal_assert("umka-1976", object->entity != NULL);
+	aal_assert("umka-1976", object->ent != NULL);
 
-	if (!object->entity->plug->o.object_ops->add_entry)
+	if (!objplug(object)->o.object_ops->add_entry)
 		return -EINVAL;
 	
-	return plug_call(object->entity->plug->o.object_ops, 
-			 add_entry, object->entity, entry);
+	return plug_call(objplug(object)->o.object_ops,
+			 add_entry, object->ent, entry);
 }
 
 /* Removes @entry to @object */
@@ -293,74 +212,93 @@ errno_t reiser4_object_rem_entry(
 	entry_hint_t *entry)                /* entry hint to be added */
 {
 	aal_assert("umka-1977", object != NULL);
-	aal_assert("umka-1978", object->entity != NULL);
+	aal_assert("umka-1978", object->ent != NULL);
     
-	if (!object->entity->plug->o.object_ops->rem_entry)
+	if (!objplug(object)->o.object_ops->rem_entry)
 		return -EINVAL;
 	
-	return plug_call(object->entity->plug->o.object_ops, 
-			 rem_entry, object->entity, entry);
+	return plug_call(objplug(object)->o.object_ops,
+			 rem_entry, object->ent, entry);
+}
+
+errno_t reiser4_object_truncate(
+	reiser4_object_t *object,           /* object for truncating */
+	uint64_t n)			    /* the number of entries */
+{
+	aal_assert("umka-1154", object != NULL);
+	aal_assert("umka-1155", object->ent != NULL);
+    
+	return plug_call(objplug(object)->o.object_ops,
+			 truncate, object->ent, n);
 }
 
 /* Adds speficied entry into passed opened dir */
 int64_t reiser4_object_write(
 	reiser4_object_t *object,           /* object for writing */
 	void *buff,			    /* new entries buffer */
-	uint64_t n)			    /* the number of entries to be created */
+	uint64_t n)			    /* the number of entries to 
+					       be created */
 {
 	aal_assert("umka-862", object != NULL);
-	aal_assert("umka-863", object->entity != NULL);
+	aal_assert("umka-863", object->ent != NULL);
     
-	if (!object->entity->plug->o.object_ops->write)
+	if (!objplug(object)->o.object_ops->write)
 		return -EINVAL;
 	
-	return plug_call(object->entity->plug->o.object_ops, 
-			 write, object->entity, buff, n);
+	return plug_call(objplug(object)->o.object_ops,
+			 write, object->ent, buff, n);
 }
 
-/* Loads object stat data to @hint. */
-errno_t reiser4_object_stat(reiser4_object_t *object,
-			    statdata_hint_t *hint)
-{
-	aal_assert("umka-2570", object != NULL);
-	aal_assert("umka-2571", object->entity != NULL);
+/* Updates object stat data coord by means of using tree_lookup(). */
+errno_t reiser4_object_refresh(reiser4_object_t *object) {
+	reiser4_tree_t *tree;
+	lookup_hint_t hint;
 
-	return plug_call(object->entity->plug->o.object_ops,
-			 stat, object->entity, hint);
+	hint.level = LEAF_LEVEL;
+	hint.key = &object->ent->object;
+
+	hint.collision = NULL;
+
+	tree = (reiser4_tree_t *)object->ent->tree;
+	
+	switch (reiser4_tree_lookup(tree, &hint, FIND_EXACT, 
+				    object_start(object)))
+	{
+	case PRESENT:
+		return 0;
+	default:
+		return -EINVAL;
+	}
 }
 
 /* Saves stat data described by @hint to @object stat data item in tree. */
-errno_t reiser4_object_update(reiser4_object_t *object,
-			      statdata_hint_t *hint)
-{
+errno_t reiser4_object_update(reiser4_object_t *object, stat_hint_t *hint) {
 	aal_assert("umka-2572", object != NULL);
-	aal_assert("umka-2573", object->entity != NULL);
+	aal_assert("umka-2573", object->ent != NULL);
 
-	return plug_call(object->entity->plug->o.object_ops,
-			 update, object->entity, hint);
+	return plug_call(objplug(object)->o.object_ops,
+			 update, object->ent, hint);
 }
 
 /* Helper function for prepare object key to be used for creating new object. */
 static void reiser4_object_maintain(reiser4_tree_t *tree,
 				    entry_hint_t *entry,
-				    object_hint_t *hint,
-				    object_info_t *info) 
+				    object_hint_t *hint) 
 {
 	oid_t locality;
 	oid_t ordering;
 	oid_t objectid;
 	
 	/* Initializing fields and preparing the keys */
-	info->tree = tree;
+	hint->info.tree = (tree_entity_t *)tree;
 
-	if (hint->parent) {
+	if (hint->info.parent.plug) {
 		/* Parent if defined, getting locality from it. */
-		reiser4_key_assign(&info->parent, hint->parent);
 		objectid = reiser4_oid_allocate(tree->fs->oid);
-		locality = reiser4_key_get_objectid(&info->parent);
+		locality = reiser4_key_get_objectid(&hint->info.parent);
 	} else {
 		/* Parent is not defined, root key is used. */
-		reiser4_key_assign(&info->parent, &tree->key);
+		reiser4_key_assign(&hint->info.parent, &tree->key);
 		locality = reiser4_key_get_locality(&tree->key);
 		objectid = reiser4_key_get_objectid(&tree->key);
 	}
@@ -368,13 +306,13 @@ static void reiser4_object_maintain(reiser4_tree_t *tree,
 	/* New object is identified by its locality and objectid. Set them to
 	   the @object->info.object key and plugin create method will build the
 	   whole key there. */
-	info->object.plug = tree->key.plug;
+	hint->info.object.plug = tree->key.plug;
 
 	/* Ordering component of key to be used for object. */
 	ordering = reiser4_key_get_ordering(&entry->offset);
 
 	/* Building object stat data key. */
-	reiser4_key_build_generic(&info->object, KEY_STATDATA_TYPE,
+	reiser4_key_build_generic(&hint->info.object, KEY_STATDATA_TYPE,
 				  locality, ordering, objectid, 0);
 }
 
@@ -384,35 +322,27 @@ reiser4_object_t *reiser4_object_create(
 	entry_hint_t *entry,                 /* entry hint object to be used */
 	object_hint_t *hint)                 /* object hint */
 {
-	char *name;
-	object_info_t info;
 	reiser4_object_t *object;
+	reiser4_plug_t *plug;
 	
 	aal_assert("umka-790", tree != NULL);
 	aal_assert("umka-1128", hint != NULL);
-	aal_assert("umka-1917", hint->plug != NULL);
+
+	plug = hint->info.opset.plug[OPSET_OBJ];
+	
+	aal_assert("umka-1917", plug != NULL);
 
 	/* Allocating the memory for object instance */
 	if (!(object = aal_calloc(sizeof(*object), 0)))
 		return NULL;
 
 	/* Preparing object info. */
-	reiser4_object_maintain(tree, entry, hint, &info);
+	reiser4_object_maintain(tree, entry, hint);
 
 	/* Calling object plugin to create its body in the tree */
-	if (!(object->entity = plug_call(hint->plug->o.object_ops,
-					 create, &info, hint)))
-	{
+	if (!(object->ent = plug_call(plug->o.object_ops, create, hint)))
 		goto error_free_object;
-	}
 
-	/* Assigning into to entity's info. */
-	object->info = &object->entity->info;
-	
-	/* @hint->object key is built by plugin create method. */
-	name = reiser4_print_key(&object->info->object, PO_INODE);
-	aal_strncpy(object->name, name, sizeof(object->name));
-	
 	return object;
 	
  error_free_object:
@@ -424,16 +354,8 @@ reiser4_object_t *reiser4_object_create(
 errno_t reiser4_object_clobber(reiser4_object_t *object) {
 	aal_assert("umka-2297", object != NULL);
 
-	return plug_call(object->entity->plug->o.object_ops,
-			 clobber, object->entity);
-}
-
-/* Returns @nlink value from the stat data if any */
-uint32_t reiser4_object_links(reiser4_object_t *object) {
-	aal_assert("umka-2293", object != NULL);
-
-	return plug_call(object->entity->plug->o.object_ops,
-			 links, object->entity);
+	return plug_call(objplug(object)->o.object_ops,
+			 clobber, object->ent);
 }
 
 /* Links @child to @object if it is a directory */
@@ -447,31 +369,31 @@ errno_t reiser4_object_link(reiser4_object_t *object,
 
 	/* Check if we need to add entry in parent @object */
 	if (entry && object) {
-		reiser4_key_assign(&entry->object, &child->info->object);
+		reiser4_key_assign(&entry->object, 
+				   &child->ent->object);
 
 		if ((res = reiser4_object_add_entry(object, entry))) {
-			aal_error("Can't add entry %s to %s.",
-				  entry->name, object->name);
+			aal_error("Can't add entry %s to %s.", entry->name, 
+				  reiser4_print_key(&object->ent->object, 
+						    PO_INODE));
 			return res;
 		}
 	}
 
 	/* Add one hard link to @child. */
-	if (child->entity->plug->o.object_ops->link) {
-		if ((res = plug_call(child->entity->plug->o.object_ops,
-				     link, child->entity)))
-		{
-			return res;
-		}
+	if (objplug(child)->o.object_ops->link) {
+		if ((res = plug_call(objplug(child)->o.object_ops, 
+				     link, child->ent)))
+		     return res;
 	}
 
 	/* Attach @child to @parent. */
-	if (child->entity->plug->o.object_ops->attach) {
+	if (objplug(child)->o.object_ops->attach) {
 		object_entity_t *parent = object ?
-			object->entity : NULL;
+			object->ent : NULL;
 
-		return plug_call(child->entity->plug->o.object_ops,
-				 attach, child->entity, parent);
+		return plug_call(objplug(child)->o.object_ops,
+				 attach, child->ent, parent);
 	}
 
 	return 0;
@@ -481,26 +403,34 @@ errno_t reiser4_object_link(reiser4_object_t *object,
 errno_t reiser4_object_unlink(reiser4_object_t *object,
 			      entry_hint_t *entry)
 {
-	errno_t res = 0;
-	lookup_hint_t hint;
-	reiser4_place_t place;
 	reiser4_object_t *child;
+	reiser4_place_t place;
+	reiser4_tree_t *tree;
+	lookup_hint_t hint;
+	errno_t res = 0;
 	
 	aal_assert("umka-1910", object != NULL);
 
+	if (entry->type == ET_SPCL) {
+		aal_error("Can't remove the special name '%s' "
+			  "from the directory %s.", entry->name, 
+			  reiser4_print_key(&object->ent->object, PO_INODE));
+		return -EINVAL;
+	}
+	
 	/* Getting child statdata key */
 	if (reiser4_object_lookup(object, entry->name,
 				  entry) != PRESENT)
 	{
-		aal_error("Can't find entry %s in %s.",
-			  entry->name, object->name);
+		aal_error("Can't find entry %s in %s.", entry->name, 
+			  reiser4_print_key(&object->ent->object, PO_INODE));
 		return -EINVAL;
 	}
 
 	/* Removing entry from @object. */
 	if ((res = reiser4_object_rem_entry(object, entry))) {
-		aal_error("Can't remove entry %s in %s.",
-			  entry->name, object->name);
+		aal_error("Can't remove entry %s in %s.",entry->name, 
+			  reiser4_print_key(&object->ent->object, PO_INODE));
 		return res;
 	}
 
@@ -508,42 +438,38 @@ errno_t reiser4_object_unlink(reiser4_object_t *object,
 	hint.key = &entry->object;
 	hint.collision = NULL;
 	
+	tree = (reiser4_tree_t *)object->ent->tree;
+	
 	/* Looking up for the victim's statdata place */
-	if (reiser4_tree_lookup(object->info->tree, &hint,
-				FIND_EXACT, &place) != PRESENT)
-	{
+	if (reiser4_tree_lookup(tree, &hint, FIND_EXACT, &place) != PRESENT) {
 		char *key = reiser4_print_key(&entry->object, PO_DEFAULT);
 		aal_error("Can't find an item pointed by %s. "
-			  "Entry %s/%s points to nowere.",
-			  key, object->name, entry->name);
+			  "Entry %s/%s points to nowere.", key, 
+			  reiser4_print_key(&object->ent->object, PO_INODE), 
+			  entry->name);
 		return -EINVAL;
 	}
 
 	/* Opening victim object by found place */
-	if (!(child = reiser4_object_realize(object->info->tree, 
-					     object, &place))) 
-	{
+	if (!(child = reiser4_object_open(tree, object, &place))) {
 		aal_error("Can't open %s/%s. Object is corrupted?",
-			  object->name, entry->name);
+			  reiser4_print_key(&object->ent->object, PO_INODE), 
+			  entry->name);
 		return -EINVAL;
 	}
 
 	/* Remove one hard link from child. */
-	if (child->entity->plug->o.object_ops->unlink) {
-		if ((res = plug_call(child->entity->plug->o.object_ops,
-				     unlink, child->entity)))
-		{
+	if (objplug(child)->o.object_ops->unlink) {
+		if ((res = plug_call(objplug(child)->o.object_ops,
+				     unlink, child->ent)))
 			return res;
-		}
 	}
 
 	/* Detach @child from parent. */
-	if (child->entity->plug->o.object_ops->detach) {
-		if ((res = plug_call(child->entity->plug->o.object_ops,
-				     detach, child->entity, object->entity)))
-		{
+	if (objplug(child)->o.object_ops->detach) {
+		if ((res = plug_call(objplug(child)->o.object_ops, detach,
+				     child->ent, object->ent)))
 			return res;
-		}
 	}
 
 	reiser4_object_close(child);
@@ -556,18 +482,14 @@ errno_t reiser4_object_layout(
 	region_func_t region_func,  /* layout callback function */
 	void *data)                 /* user-specified data */
 {
-	reiser4_plug_t *plug;
-	
 	aal_assert("umka-1469", object != NULL);
 	aal_assert("umka-1470", region_func != NULL);
 
-	plug = object->entity->plug;
-	
-	if (!plug->o.object_ops->layout)
+	if (!objplug(object)->o.object_ops->layout)
 		return 0;
 	
-	return plug_call(plug->o.object_ops, layout,
-			 object->entity, region_func, data);
+	return plug_call(objplug(object)->o.object_ops, layout, 
+			 object->ent, region_func, data);
 }
 
 /* Enumerates all items object consists of */
@@ -576,61 +498,37 @@ errno_t reiser4_object_metadata(
 	place_func_t place_func,    /* metadata layout callback */
 	void *data)                 /* user-spaecified data */
 {
-	reiser4_plug_t *plug;
-	
 	aal_assert("umka-1714", object != NULL);
 	aal_assert("umka-1715", place_func != NULL);
 
-	plug = object->entity->plug;
-	
-	if (!plug->o.object_ops->metadata)
+	if (!objplug(object)->o.object_ops->metadata)
 		return 0;
 	
-	return plug_call(plug->o.object_ops, metadata,
-			 object->entity, place_func, data);
+	return plug_call(objplug(object)->o.object_ops, metadata, 
+			 object->ent, place_func, data);
 }
 
 /* Makes lookup inside the @object */
 lookup_t reiser4_object_lookup(reiser4_object_t *object,
-			       const char *name,
-			       entry_hint_t *entry)
+			       const char *name, entry_hint_t *entry)
 {
 	aal_assert("umka-1919", object != NULL);
 	aal_assert("umka-1920", name != NULL);
 
-	if (!object->entity->plug->o.object_ops->lookup)
+	if (!objplug(object)->o.object_ops->lookup)
 		return -EINVAL;
 	
-	return plug_call(object->entity->plug->o.object_ops,
-			 lookup, object->entity, (char *)name,
-			 (void *)entry);
-}
-#endif
-
-/* Sets directory current position to passed pos */
-errno_t reiser4_object_seek(
-	reiser4_object_t *object,    /* object position to be changed in */
-	uint32_t offset)	     /* offset for seeking */
-{
-	aal_assert("umka-1129", object != NULL);
-	aal_assert("umka-1153", object->entity != NULL);
-    
-	if (!object->entity->plug->o.object_ops->seek)
-		return -EINVAL;
-	
-	return plug_call(object->entity->plug->o.object_ops, 
-			 seek, object->entity, offset);
+	return plug_call(objplug(object)->o.object_ops, lookup, 
+			 object->ent, (char *)name, (void *)entry);
 }
 
-/* Closes specified object */
-void reiser4_object_close(
-	reiser4_object_t *object)    /* object to be closed */
-{
-	aal_assert("umka-680", object != NULL);
-	aal_assert("umka-1149", object->entity != NULL);
+/* Loads object stat data to @hint. */
+errno_t reiser4_object_stat(reiser4_object_t *object, stat_hint_t *hint) {
+	aal_assert("umka-2570", object != NULL);
+	aal_assert("umka-2571", object->ent != NULL);
 
-	reiser4_object_fini(object);
-	aal_free(object);
+	return plug_call(objplug(object)->o.object_ops,
+			 stat, object->ent, hint);
 }
 
 /* Resets directory position */
@@ -638,10 +536,64 @@ errno_t reiser4_object_reset(
 	reiser4_object_t *object)    /* dir to be reset */
 {
 	aal_assert("umka-842", object != NULL);
-	aal_assert("umka-843", object->entity != NULL);
+	aal_assert("umka-843", object->ent != NULL);
 
-	return plug_call(object->entity->plug->o.object_ops, 
-			 reset, object->entity);
+	return plug_call(objplug(object)->o.object_ops,
+			 reset, object->ent);
+}
+
+/* Sets directory current position to passed pos */
+errno_t reiser4_object_seek(
+	reiser4_object_t *object,    /* object position to be changed in */
+	uint32_t offset)	     /* offset for seeking */
+{
+	aal_assert("umka-1129", object != NULL);
+	aal_assert("umka-1153", object->ent != NULL);
+    
+	if (!objplug(object)->o.object_ops->seek)
+		return -EINVAL;
+	
+	return plug_call(objplug(object)->o.object_ops,
+			 seek, object->ent, offset);
+}
+
+/* Change current position in passed @object if it is a directory */
+errno_t reiser4_object_seekdir(reiser4_object_t *object,
+			       reiser4_key_t *offset)
+{
+	aal_assert("umka-1979", object != NULL);
+	aal_assert("umka-1980", offset != NULL);
+
+	if (!objplug(object)->o.object_ops->seekdir)
+		return -EINVAL;
+
+	return plug_call(objplug(object)->o.object_ops,
+			 seekdir, object->ent, offset);
+}
+
+/* Returns current position in directory */
+uint32_t reiser4_object_offset(
+	reiser4_object_t *object)    /* dir position will be obtained from */
+{
+	aal_assert("umka-875", object != NULL);
+	aal_assert("umka-876", object->ent != NULL);
+
+	return plug_call(objplug(object)->o.object_ops, 
+			 offset, object->ent);
+}
+
+/* Return current position in passed @object if it is a directory */
+errno_t reiser4_object_telldir(reiser4_object_t *object,
+			       reiser4_key_t *offset)
+{
+	aal_assert("umka-1981", object != NULL);
+	aal_assert("umka-1982", offset != NULL);
+
+	if (!objplug(object)->o.object_ops->telldir)
+		return -EINVAL;
+
+	return plug_call(objplug(object)->o.object_ops,
+			 telldir, object->ent, offset);
 }
 
 /* Reads @n bytes of data at the current offset of @object to passed
@@ -652,24 +604,13 @@ int64_t reiser4_object_read(
 	uint64_t n)                 /* buffer size */
 {
 	aal_assert("umka-860", object != NULL);
-	aal_assert("umka-861", object->entity != NULL);
+	aal_assert("umka-861", object->ent != NULL);
 
-	if (!object->entity->plug->o.object_ops->read)
+	if (!objplug(object)->o.object_ops->read)
 		return -EINVAL;
 	
-	return plug_call(object->entity->plug->o.object_ops, 
-			 read, object->entity, buff, n);
-}
-
-/* Returns current position in directory */
-uint32_t reiser4_object_offset(
-	reiser4_object_t *object)    /* dir position will be obtained from */
-{
-	aal_assert("umka-875", object != NULL);
-	aal_assert("umka-876", object->entity != NULL);
-
-	return plug_call(object->entity->plug->o.object_ops, 
-			 offset, object->entity);
+	return plug_call(objplug(object)->o.object_ops, 
+			 read, object->ent, buff, n);
 }
 
 /* Reads entry at current @object offset to passed @entry hint */
@@ -679,40 +620,11 @@ errno_t reiser4_object_readdir(reiser4_object_t *object,
 	aal_assert("umka-1973", object != NULL);
 	aal_assert("umka-1974", entry != NULL);
 
-	if (!object->entity->plug->o.object_ops->readdir)
+	if (!objplug(object)->o.object_ops->readdir)
 		return -EINVAL;
 	
-	return plug_call(object->entity->plug->o.object_ops, 
-			 readdir, object->entity, entry);
-}
-
-#ifndef ENABLE_STAND_ALONE
-/* Change current position in passed @object if it is a directory */
-errno_t reiser4_object_seekdir(reiser4_object_t *object,
-			       reiser4_key_t *offset)
-{
-	aal_assert("umka-1979", object != NULL);
-	aal_assert("umka-1980", offset != NULL);
-
-	if (!object->entity->plug->o.object_ops->seekdir)
-		return -EINVAL;
-
-	return plug_call(object->entity->plug->o.object_ops,
-			 seekdir, object->entity, offset);
-}
-
-/* Return current position in passed @object if it is a directory */
-errno_t reiser4_object_telldir(reiser4_object_t *object,
-			       reiser4_key_t *offset)
-{
-	aal_assert("umka-1981", object != NULL);
-	aal_assert("umka-1982", offset != NULL);
-
-	if (!object->entity->plug->o.object_ops->telldir)
-		return -EINVAL;
-
-	return plug_call(object->entity->plug->o.object_ops,
-			 telldir, object->entity, offset);
+	return plug_call(objplug(object)->o.object_ops, 
+			 readdir, object->ent, entry);
 }
 
 /* Completes object creating. */
@@ -726,15 +638,16 @@ static reiser4_object_t *reiser4_obj_create(reiser4_tree_t *tree,
 	/* Preparing @entry to be used for object creating and linking to parent
 	   object. This is name and offset key. */
 	if (parent) {
-		if (!parent->entity->plug->o.object_ops->build_entry) {
-			aal_error("Object %s has not build_entry() "
-				  "method implemented. Is it dir "
-				  "object at all?", parent->name);
+		if (!objplug(parent)->o.object_ops->build_entry) {
+			aal_error("Object %s has not build_entry() method "
+				  "implemented. Is it dir object at all?", 
+				  reiser4_print_key(&parent->ent->object, 
+						    PO_INODE));
 			return NULL;
 		}
 		
-		plug_call(parent->entity->plug->o.object_ops,
-			  build_entry, parent->entity, entry);
+		plug_call(objplug(parent)->o.object_ops, 
+			  build_entry, parent->ent, entry);
 	} else {
 		reiser4_key_assign(&entry->offset, &tree->key);
 	}
@@ -747,8 +660,10 @@ static reiser4_object_t *reiser4_obj_create(reiser4_tree_t *tree,
 
 	if (parent) {
 		if (reiser4_object_link(parent, object, entry)) {
-			aal_warn("Removing not attached object %s "
-				 "from tree.", object->name);
+			aal_warn("Removing not attached object %s from tree.", 
+				 reiser4_print_key(&object->ent->object, 
+						   PO_INODE));
+
 			reiser4_object_clobber(object);
 			reiser4_object_close(object);
 			return NULL;
@@ -771,7 +686,7 @@ errno_t reiser4_object_traverse(reiser4_object_t *object,
 	aal_assert("vpf-1103", open_func != NULL);
 
 	/* Check if object has readdir() method implemented. */
-	if (!object->entity->plug->o.object_ops->readdir)
+	if (!objplug(object)->o.object_ops->readdir)
 		return 0;
 
 	/* Main loop until all entries enumerated. */
@@ -803,36 +718,30 @@ reiser4_object_t *reiser4_dir_create(reiser4_fs_t *fs,
 				     reiser4_object_t *parent,
 				     const char *name)
 {
-	rid_t pid;
 	entry_hint_t entry;
 	object_hint_t hint;
+	tree_entity_t *tent;
 	
 	aal_assert("vpf-1053", fs != NULL);
+	aal_assert("vpf-1611", fs->tree != NULL);
 	
-	pid = reiser4_param_value("directory");
+	aal_memset(&hint, 0, sizeof(hint));
+
+	tent = &fs->tree->ent;
 	
 	/* Preparing object hint */
-	hint.plug = reiser4_factory_ifind(OBJECT_PLUG_TYPE, pid);
-
-	if (!hint.plug) {
-		aal_error("Can't find directory plugin "
-			  "by its id 0x%x.", pid);
-		return NULL;
+	aal_memcpy(hint.info.opset.plug, tent->opset, 
+		   OPSET_LAST * sizeof(reiser4_plug_t *));
+	hint.info.opset.plug[OPSET_OBJ] = tent->opset[OPSET_MKDIR];
+	hint.mode = 0;
+	
+	if (parent) {
+		reiser4_key_assign(&hint.info.parent, 
+				   &parent->ent->object);
 	}
 
-	/* Preparing directory label. */
-	hint.label.mode = 0;
-	hint.label.statdata = reiser4_param_value("statdata");
-	
-	/* Preparing directory body. */
-	hint.body.dir.hash = reiser4_param_value("hash");
-	hint.body.dir.fibre = reiser4_param_value("fibre");
-	hint.body.dir.direntry = reiser4_param_value("direntry");
-	hint.parent = (parent ? &parent->info->object : NULL);
-
 	if (name) {
-		aal_strncpy(entry.name, name,
-			    sizeof(entry.name));
+		aal_strncpy(entry.name, name, sizeof(entry.name));
 	} else {
 		entry.name[0] = '\0';
 	}
@@ -846,36 +755,29 @@ reiser4_object_t *reiser4_reg_create(reiser4_fs_t *fs,
 				     reiser4_object_t *parent,
 				     const char *name)
 {
-	rid_t regular;
 	entry_hint_t entry;
 	object_hint_t hint;
+	tree_entity_t *tent;
 	
 	aal_assert("vpf-1054", fs != NULL);
 	
-	regular = reiser4_param_value("regular");
+	aal_memset(&hint, 0, sizeof(hint));
+
+	tent = &fs->tree->ent;
 	
 	/* Preparing object hint */
-	hint.plug = reiser4_factory_ifind(OBJECT_PLUG_TYPE, regular);
+	aal_memcpy(hint.info.opset.plug, tent->opset, 
+		   OPSET_LAST * sizeof(reiser4_plug_t *));
+	hint.info.opset.plug[OPSET_OBJ] = tent->opset[OPSET_CREATE];
+	hint.mode = 0;
 
-	if (!hint.plug) {
-		aal_error("Can't find regual file plugin "
-			  "by its id 0x%x.", regular);
-		return NULL;
+	if (parent) {
+		reiser4_key_assign(&hint.info.parent, 
+				   &parent->ent->object);
 	}
 
-	/* Preparing label fields. */
-	hint.label.mode = 0;
-	hint.label.statdata = reiser4_param_value("statdata");
-
-	/* Preparing body fields. */
-	hint.body.reg.tail = reiser4_param_value("tail");
-	hint.body.reg.extent = reiser4_param_value("extent");
-	hint.body.reg.policy = reiser4_param_value("policy");
-	hint.parent = (parent ? &parent->info->object : NULL);
-	
 	if (name) {
-		aal_strncpy(entry.name, name,
-			    sizeof(entry.name));
+		aal_strncpy(entry.name, name, sizeof(entry.name));
 	} else {
 		entry.name[0] = '\0';
 	}
@@ -889,35 +791,29 @@ reiser4_object_t *reiser4_sym_create(reiser4_fs_t *fs,
 		                     const char *name,
 		                     const char *target)
 {
-	rid_t symlink;
 	entry_hint_t entry;
 	object_hint_t hint;
+	tree_entity_t *tent;
 	
 	aal_assert("vpf-1186", fs != NULL);
 	aal_assert("vpf-1057", target != NULL);
 	
-	symlink = reiser4_param_value("symlink");
+	tent = &fs->tree->ent;
 	
 	/* Preparing object hint */
-	hint.plug = reiser4_factory_ifind(OBJECT_PLUG_TYPE, symlink);
-
-	if (!hint.plug) {
-		aal_error("Can't find symlink plugin by "
-			  "its id 0x%x.", symlink);
-		return NULL;
+	aal_memcpy(hint.info.opset.plug, tent->opset, 
+		   OPSET_LAST * sizeof(reiser4_plug_t *));
+	hint.info.opset.plug[OPSET_OBJ] = tent->opset[OPSET_SYMLINK];
+	hint.mode = 0;
+	hint.name = (char *)target;
+	
+	if (parent) {
+		reiser4_key_assign(&hint.info.parent, 
+				   &parent->ent->object);
 	}
 
-	/* Preparing label fields. */
-	hint.label.mode = 0;
-	hint.label.statdata = reiser4_param_value("statdata");
-
-	/* Preparing body fields. */
-	hint.body.sym = (char *)target;
-	hint.parent = (parent ? &parent->info->object : NULL);
-	
 	if (name) {
-		aal_strncpy(entry.name, name,
-			    sizeof(entry.name));
+		aal_strncpy(entry.name, name, sizeof(entry.name));
 	} else {
 		entry.name[0] = '\0';
 	}
@@ -932,39 +828,34 @@ reiser4_object_t *reiser4_spl_create(reiser4_fs_t *fs,
 				     uint32_t mode,
 		                     uint64_t rdev)
 {
-	rid_t special;
 	entry_hint_t entry;
 	object_hint_t hint;
+	tree_entity_t *tent;
 	
 	aal_assert("umka-2534", fs != NULL);
 	aal_assert("umka-2535", rdev != 0);
 	
-	special = reiser4_param_value("special");
+	tent = &fs->tree->ent;
 	
 	/* Preparing object hint. */
-	hint.plug = reiser4_factory_ifind(OBJECT_PLUG_TYPE, special);
-
-	if (!hint.plug) {
-		aal_error("Can't find special file plugin "
-			  "by its id 0x%x.", special);
-		return NULL;
+	aal_memcpy(hint.info.opset.plug, tent->opset, 
+		   OPSET_LAST * sizeof(reiser4_plug_t *));
+	hint.info.opset.plug[OPSET_OBJ] = tent->opset[OPSET_MKNODE];
+	hint.mode = mode;
+	hint.rdev = rdev;
+	
+	if (parent) {
+		reiser4_key_assign(&hint.info.parent, 
+				   &parent->ent->object);
 	}
-
-	/* Preparing label fields. */
-	hint.label.mode = mode;
-	hint.label.statdata = reiser4_param_value("statdata");
-
-	/* Preparing body fields. */
-	hint.body.spl.rdev = rdev;
-	hint.parent = (parent ? &parent->info->object : NULL);
 	
 	if (name) {
-		aal_strncpy(entry.name, name,
-			    sizeof(entry.name));
+		aal_strncpy(entry.name, name, sizeof(entry.name));
 	} else {
 		entry.name[0] = '\0';
 	}
 
 	return reiser4_obj_create(fs->tree, parent, &entry, &hint);
 }
+
 #endif
