@@ -13,7 +13,6 @@ errno_t repair_fs_open(repair_data_t *repair,
 		       aal_device_t *jdevice)
 {
 	errno_t res = 0;
-	uint64_t len;
 
 	aal_assert("vpf-851",  repair != NULL);
 	aal_assert("vpf-159",  hdevice != NULL);
@@ -49,13 +48,10 @@ errno_t repair_fs_open(repair_data_t *repair,
 		goto error_format_close;
 	}
 	
-	len = reiser4_format_get_len(repair->fs->format);
+	res |= repair_alloc_open(repair->fs, repair->mode);
 
-	/* Block and oid allocator plugins are specified by format plugin 
-	 * unambiguously, so there is nothing to be checked here anymore. */
-	if (!(repair->fs->alloc = reiser4_alloc_open(repair->fs, len))) {
-		aal_fatal("Failed to open a block allocator.");
-		res = -EINVAL;
+	if (repair_error_fatal(res)) {
+		aal_fatal("Failed to open the block allocator.");
 		goto error_status_close;
 	}
 	
@@ -65,9 +61,6 @@ errno_t repair_fs_open(repair_data_t *repair,
 		goto error_alloc_close;
 	}
 
-	if ((res = reiser4_fs_init_params(repair->fs)))
-		goto error_alloc_close;
-	
 	res |= repair_journal_open(repair->fs, jdevice, repair->mode);
 	
 	if (repair_error_fatal(res)) {
@@ -76,10 +69,11 @@ errno_t repair_fs_open(repair_data_t *repair,
 	}
 	
 	if (!(repair->fs->tree = reiser4_tree_init(repair->fs))) {
+		aal_fatal("Failed to init the fs-global plugin set.");
 		res = -ENOMEM;
 		goto error_journal_close;
 	}
-		
+	
 	repair_error_count(repair, res);
 	return 0;
 
@@ -114,23 +108,6 @@ error_journal_close:
 	return res < 0 ? res : 0;
 }
 
-errno_t repair_fs_valid(reiser4_fs_t *fs, uint8_t mode) {
-	errno_t res;
-	
-	if (!(res = reiser4_alloc_valid(fs->alloc)))
-		return 0;
-	
-	if (res != -ESTRUCT) 
-		return res;
-
-	if (mode != RM_CHECK) {
-		aal_mess("Checksums will be fixed later.\n");
-		return 0;
-	}
-	
-	return RE_FIXABLE;
-}
-
 errno_t repair_fs_replay(reiser4_fs_t *fs) {
 	errno_t res;
 	
@@ -221,7 +198,8 @@ errno_t repair_fs_pack(reiser4_fs_t *fs,
 		if (!(node = reiser4_node_open(fs->tree, blk)))
 			continue;
 
-		if ((res = repair_node_check_struct(node, RM_CHECK)) < 0)
+		if ((res = repair_node_check_struct(node, NULL, 
+						    RM_CHECK, NULL)) < 0)
 			return res;
 
 		if (res) {
@@ -245,8 +223,8 @@ errno_t repair_fs_pack(reiser4_fs_t *fs,
 	return 0;
 }
 
-static errno_t callback_mark_used(void *object, uint64_t start,
-				  uint64_t count, void *data) 
+static errno_t cb_mark_used(void *object, uint64_t start,
+			    uint64_t count, void *data) 
 {
 	aux_bitmap_t *bitmap = (aux_bitmap_t *)data;
 
@@ -378,7 +356,7 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 		/* Resize the bitmap. */
 		aux_bitmap_resize(bitmap, len);
 		
-		if (reiser4_fs_layout(fs, callback_mark_used, bitmap)) {
+		if (reiser4_fs_layout(fs, cb_mark_used, bitmap)) {
 			aal_error("Can't to mark all frozen fs "
 				  "blocks as used in the bitmap.");
 			goto error_free_status;
@@ -412,8 +390,9 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 			goto error_free_status;
 		}
 
-		if (bitmap)
-			aux_bitmap_mark(bitmap, node_blocknr(node));
+		if (bitmap) {
+			aux_bitmap_mark(bitmap, node->block->nr);
+		}
 		
 		reiser4_node_close(node);
 	}
@@ -440,19 +419,11 @@ reiser4_fs_t *repair_fs_unpack(aal_device_t *device,
 errno_t repair_fs_lost_key(reiser4_fs_t *fs, reiser4_key_t *key) {
 	oid_t locality;
 	oid_t objectid;
-	rid_t pid;
 
 	aal_assert("vpf-1553", fs != NULL);
 	aal_assert("vpf-1554", key != NULL);
 
-	pid = reiser4_format_key_pid(fs->format);
-
-	/* Finding needed key plugin by its identifier. */
-	if (!(key->plug = reiser4_factory_ifind(KEY_PLUG_TYPE, pid))) {
-		aal_error("Can't find key plugin by its id 0x%x.", pid);
-		return -EINVAL;
-	}
-	
+	key->plug = fs->tree->ent.tpset[TPSET_KEY];
 	locality = reiser4_oid_root_objectid(fs->oid);
 	objectid = repair_oid_lost_objectid(fs->oid);
 	
