@@ -7,13 +7,16 @@
 #include <repair/librepair.h>
 
 /* Checks the blocksize. */
-static int callback_bs_check (int64_t val, void * data) {
+static int cb_bs_check (int64_t val, void * data) {
 	if (!aal_pow2(val))
 		return 0;
     
 	if (val < 512)
 		return 0;
 
+	if (val > REISER4_MAX_BLKSIZE)
+		return 0;
+
 	return 1;
 }
 
@@ -21,8 +24,8 @@ static int callback_bs_check (int64_t val, void * data) {
    one was opened. */
 static errno_t repair_master_check(reiser4_fs_t *fs, uint8_t mode) {
 	reiser4_plug_t *plug;
-	rid_t format, pid;
-	uint16_t blksize;
+	uint16_t size;
+	rid_t pid;
 	
 	aal_assert("vpf-730", fs != NULL);
 	aal_assert("vpf-161", fs->master != NULL || fs->device != NULL);
@@ -32,26 +35,24 @@ static errno_t repair_master_check(reiser4_fs_t *fs, uint8_t mode) {
 			return RE_FATAL;
 		
 		/* Master SB was not opened. Create a new one. */
-		if (aal_exception_throw(EXCEPTION_TYPE_MESSAGE, 
-					EXCEPTION_OPT_YESNO,
-					"Master super block cannot be found. Do"
-					" you want to build a new one on (%s)?",
-					fs->device->name) == EXCEPTION_OPT_NO)
+		if (aal_yesno("Master super block cannot be found. Do"
+			      " you want to build a new one on (%s)?",
+			      fs->device->name) == EXCEPTION_OPT_NO)
 		{
 			return -EINVAL;
 		}
 		
-		blksize = aal_ui_get_numeric(4096, callback_bs_check, NULL,
-					     "Which block size do you use?");
+		size = aal_ui_get_numeric(4096, cb_bs_check, NULL,
+					  "Which block size do you use?");
 		
 		/* Create a new master SB. */
-		if (!(fs->master = reiser4_master_create(fs->device, blksize)))
+		if (!(fs->master = reiser4_master_create(fs->device, size)))
 		{
-			aal_fatal("Failed to create a new master super block.");
+			aal_error("Failed to create a new master super block.");
 			return -EINVAL;
 		}
 
-		aal_info("A new master superblock is created on (%s).", 
+		aal_warn("A new master superblock is created on (%s).", 
 			 fs->device->name);
 		
 		reiser4_master_set_uuid(fs->master, NULL);
@@ -61,56 +62,49 @@ static errno_t repair_master_check(reiser4_fs_t *fs, uint8_t mode) {
 		/* Master SB was opened. Check it for validness. */
 		
 		/* Check the blocksize. */
-		if (!callback_bs_check(reiser4_master_get_blksize(fs->master), 
-				       NULL))
+		if (!cb_bs_check(reiser4_master_get_blksize(fs->master), NULL))
 		{
-			aal_fatal("Invalid blocksize found in the "
+			aal_error("Invalid blocksize found in the "
 				  "master super block (%u).",
 				  reiser4_master_get_blksize(fs->master));
 			
 			if (mode != RM_BUILD)
 				return RE_FATAL;
 			
-			blksize = aal_ui_get_numeric(4096, callback_bs_check,
-						     NULL, "Which block size "
-						     "do you use?");
+			size = aal_ui_get_numeric(4096, cb_bs_check, NULL,
+						  "Which block size do you "
+						  "use?");
 
-			reiser4_master_set_blksize(fs->master, blksize);
+			reiser4_master_set_blksize(fs->master, size);
 			reiser4_master_mkdirty(fs->master);
 		}
 	}
 
 	/* Setting actual used block size from master super block */
 	if (aal_device_set_bs(fs->device, reiser4_master_get_blksize(fs->master))) {
-		aal_fatal("Invalid block size was specified (%u). It "
+		aal_error("Invalid block size was specified (%u). It "
 			  "must be power of two.",
 			  reiser4_master_get_blksize(fs->master));
 		return -EINVAL;
 	}
 	
-	format = reiser4_param_value("format");
+	plug = reiser4_profile_plug(PROF_FORMAT);
 	pid = reiser4_master_get_format(fs->master);
 	
 	/* If the format is overridden, fix master accordingly to the specified 
 	   value. */ 
-	if (pid != format && reiser4_param_get_flag("format", PF_OVERRIDDEN)) {
-		/* The @format is correct value. */
-		if (!(plug = reiser4_factory_ifind(FORMAT_PLUG_TYPE, format))) {
-			aal_error("Can't find format plugin by its id 0x%x.", 
-				  format);
-			return -EINVAL;
-		}
-
-		aal_fatal("The specified reiser4 format on '%s' is '%s'. Its "
+	if (reiser4_profile_overridden(PROF_FORMAT) && pid != plug->id.id) {
+		/* The @plug is the correct one. */
+		aal_error("The specified reiser4 format on '%s' is '%s'. Its "
 			  "id (0x%x) does not match the on-disk id (0x%x).%s", 
-			  fs->device->name, plug->label, format, pid, 
+			  fs->device->name, plug->label, plug->id.id, pid, 
 			  mode == RM_BUILD ? " Fixed." : " Has effect in BUILD "
 			  "mode only.");
 
 		if (mode != RM_BUILD)
 			return RE_FATAL;
 
-		reiser4_master_set_format(fs->master, format);
+		reiser4_master_set_format(fs->master, plug->id.id);
 		reiser4_master_mkdirty(fs->master);
 	}
 	
@@ -180,8 +174,7 @@ reiser4_master_t *repair_master_unpack(aal_device_t *device,
 		return NULL;
 	
 	if (size != sizeof(master->ent)) {
-		aal_error("Invalid size %u is detected in stream.",
-			  size);
+		aal_error("Invalid size %u is detected in stream.", size);
 		goto error_free_master;
 	}
 
@@ -223,7 +216,7 @@ void repair_master_print(reiser4_master_t *master,
 	}
 	
 	aal_stream_format(stream, "Master super block (%lu):\n",
-			  REISER4_MASTER_OFFSET / blksize);
+			  REISER4_MASTER_BLOCKNR(blksize));
 	
 	aal_stream_format(stream, "magic:\t\t%s\n",
 			  reiser4_master_get_magic(master));
