@@ -10,20 +10,21 @@ static void repair_semantic_lost_name(reiser4_object_t *object, char *name) {
 	uint8_t len;
 
 	len = aal_strlen(LOST_PREFIX);
-	key = reiser4_print_key(&object->info->object, PO_INODE);
+	key = reiser4_print_key(&object->ent->object, PO_INODE);
 	
 	aal_memcpy(name, LOST_PREFIX, len);
 	aal_memcpy(name + len, key, aal_strlen(key));
 }
 
 /* Callback for repair_object_check_struct. Mark the passed item as CHECKED. */
-static errno_t callback_register_item(reiser4_place_t *place, void *data) {
+static errno_t cb_register_item(reiser4_place_t *place, void *data) {
         aal_assert("vpf-1115", place != NULL);
          
         if (reiser4_item_test_flag(place, OF_CHECKED)) {
-                aal_error("Node (%llu), item (%u): item registering "
+                fsck_mess("Node (%llu), item (%u), [%s]: item registering "
 			  "failed, it belongs to another object already.",
-			  place->node->block->nr, place->pos.item);
+			  place_blknr(place), place->pos.item,
+			  reiser4_print_key(&place->key, PO_DEFAULT));
                 return -EINVAL;
         }
          
@@ -46,7 +47,7 @@ static errno_t repair_semantic_check_struct(repair_semantic_t *sem,
 	aal_assert("vpf-1169", sem != NULL);
 	aal_assert("vpf-1170", object != NULL);
 	
-	oid = reiser4_key_get_objectid(&object->info->object);
+	oid = reiser4_key_get_objectid(&object->ent->object);
 	repair_semantic_register_oid(sem, oid);
 	
 	/* Check struct if not the BUILD mode, if the fake object or 
@@ -55,17 +56,18 @@ static errno_t repair_semantic_check_struct(repair_semantic_t *sem,
 	    !reiser4_item_test_flag(object_start(object), OF_CHECKED)) 
 	{
 		place_func_t place_func = sem->repair->mode == RM_BUILD ?
-			callback_register_item : NULL;
+			cb_register_item : NULL;
 		
 		res = repair_object_check_struct(object, place_func,
 						 sem->repair->mode, sem);
 		if (res < 0) return res;
 		
+		sem->stat.reached_files++;
 		repair_error_count(sem->repair, res);
 	}
 	
-	/* Update the @object->info. */
-	res |= repair_object_form(object);
+	/* Update the @object->info.parent. */
+	res |= repair_object_refresh(object);
 	return res;
 }
 
@@ -81,10 +83,10 @@ static errno_t repair_semantic_check_attach(repair_semantic_t *sem,
 	aal_assert("vpf-1255", parent != NULL);
 	
 	place_func = sem->repair->mode == RM_BUILD ? 
-		callback_register_item : NULL;
+		cb_register_item : NULL;
 	
-	/* Even if this object is ATTACHED already it may allow many names
-	   to itself -- check the attach with this @parent. */
+	/* Even if this object is ATTACHED already it may allow many 
+	   names to itself -- check the attach with this @parent. */
 	res = repair_object_check_attach(parent, object, place_func, 
 					 sem, sem->repair->mode);
 	
@@ -99,13 +101,13 @@ static errno_t repair_semantic_check_attach(repair_semantic_t *sem,
 		return res;
 	
 	/* Increment the link. */
-	if ((res = plug_call(object->entity->plug->o.object_ops, 
-			     link, object->entity)))
+	if ((res = plug_call(object->ent->opset.plug[OPSET_OBJ]->o.object_ops,
+			     link, object->ent)))
 		return res;
 
 	/* If @parent is "lost+found", do not mark as ATTACHED. */
-	if (sem->lost && !reiser4_key_compfull(&parent->info->object,
-					       &sem->lost->info->object))
+	if (sem->lost && !reiser4_key_compfull(&parent->ent->object,
+					       &sem->lost->ent->object))
 		return 0;
 
 	return repair_object_mark(object, OF_ATTACHED);
@@ -121,20 +123,21 @@ static errno_t repair_semantic_add_entry(reiser4_object_t *parent,
 	aal_memset(&entry, 0, sizeof(entry));
 	
 	aal_strncpy(entry.name, name, sizeof(entry.name));
-	reiser4_key_assign(&entry.object, &object->info->object);
-	entry.place_func = callback_register_item;
+	reiser4_key_assign(&entry.object, &object->ent->object);
+	entry.place_func = cb_register_item;
 	entry.data = NULL;
 	
 	if ((res = reiser4_object_add_entry(parent, &entry)))
-		aal_error("Can't add entry %s to %s.",
-			  name, parent->name);
+		aal_error("Can't add entry %s to %s.", name, 
+			  reiser4_print_key(&parent->ent->object, PO_INODE));
 	
 	return res;
 }
 
 static errno_t repair_semantic_unlink(repair_semantic_t *sem, 
 				      reiser4_object_t *parent,
-				      reiser4_object_t *object)
+				      reiser4_object_t *object,
+				      char *name)
 {
 	entry_hint_t entry;
 	
@@ -142,16 +145,18 @@ static errno_t repair_semantic_unlink(repair_semantic_t *sem,
 	aal_assert("vpf-1337", object != NULL);
 
 	if (!parent) {
+		reiser4_plug_t *plug = object->ent->opset.plug[OPSET_OBJ];
+		
 		/* If there is no parent, just detach the object. */
-		if (!object->entity->plug->o.object_ops->detach) 
+		if (!plug->o.object_ops->detach) 
 			return 0;
 
-		return plug_call(object->entity->plug->o.object_ops,  
-				 detach, object->entity, NULL);
+		return plug_call(plug->o.object_ops, detach, 
+				 object->ent, NULL);
 	}
 	
 	/* unlink from the parent. */
-	aal_strncpy(entry.name, object->name, sizeof(entry.name));
+	aal_strncpy(entry.name, name, sizeof(entry.name));
 	return reiser4_object_unlink(parent, &entry);
 }
 
@@ -160,20 +165,21 @@ static errno_t repair_semantic_link_lost(repair_semantic_t *sem,
 					 reiser4_object_t *object)
 {
 	errno_t res;
+	char buff[REISER4_MAX_BLKSIZE];
 
 	aal_assert("vpf-1178", sem != NULL);
 	aal_assert("vpf-1179", parent != NULL);
 	aal_assert("vpf-1180", object != NULL);
 	
 	/* Make the lost name. */
-	repair_semantic_lost_name(object, object->name);
+	repair_semantic_lost_name(object, buff);
 	
 	/* Detach if possible. */
-	if ((res = repair_semantic_unlink(sem, NULL, object)))
+	if ((res = repair_semantic_unlink(sem, NULL, object, NULL)))
 		return res;
 	
 	/* Add the entry to the @parent. */
-	if ((res = repair_semantic_add_entry(parent, object, object->name)))
+	if ((res = repair_semantic_add_entry(parent, object, buff)))
 		return res;
 	
 	/* Check the attach of the @object to the @parent. */
@@ -197,11 +203,13 @@ static reiser4_object_t *repair_semantic_uplink(repair_semantic_t *sem,
 	
 	aal_assert("vpf-1184", object != NULL);
 
-	if (!object->info->parent.plug)
+	if (!object->ent->parent.plug)
 		return NULL;
 	
-	if ((parent = repair_object_launch(object->info->tree, NULL, 
-					   &object->info->parent)) == INVAL_PTR)
+	parent = repair_object_obtain((reiser4_tree_t *)object->ent->tree,
+				      NULL, &object->ent->parent);
+	
+	if (parent == INVAL_PTR)
 		return INVAL_PTR;
 	
 	if (parent == NULL)
@@ -220,15 +228,15 @@ static reiser4_object_t *repair_semantic_uplink(repair_semantic_t *sem,
 	if (checked) {
 		/* If parent is "lost+found" (already checked), 
 		   detach from it. */
-		if (!reiser4_key_compfull(&parent->info->object,
-					  &sem->lost->info->object))
+		if (!reiser4_key_compfull(&parent->ent->object,
+					  &sem->lost->ent->object))
 		{
 			reiser4_object_close(parent);
 			goto error_object_detach;
 		}
 		
 		/* Init all needed info for the object. */
-		if ((res = repair_object_form(parent)))
+		if ((res = repair_object_refresh(parent)))
 			goto error_parent_close;
 		
 		if ((res = repair_semantic_link_lost(sem, parent, object)))
@@ -241,6 +249,11 @@ static reiser4_object_t *repair_semantic_uplink(repair_semantic_t *sem,
 		return NULL;
 	}
 	
+	sem->stat.statdatas++;
+	aal_gauge_set_value(sem->gauge, sem->stat.statdatas * 100 / 
+			    sem->stat.files);
+	aal_gauge_touch(sem->gauge);
+	
 	/* Some parent was found, check it and attach to it. */
 	if ((res = repair_semantic_check_struct(sem, parent)) < 0)
 		goto error_parent_close;
@@ -249,17 +262,17 @@ static reiser4_object_t *repair_semantic_uplink(repair_semantic_t *sem,
 	
 	/* Check that parent has a link to the object. */
 	while ((res = reiser4_object_readdir(parent, &entry)) > 0) {
-		if (reiser4_key_compfull(&object->info->object,
+		if (reiser4_key_compfull(&object->ent->object,
 					 &entry.object))
 			break;
 	}
 	
 	if (!res) {
-		/* EOF was reached. Add entry to the parent. */
-		repair_semantic_lost_name(object, object->name);
+		char buff[REISER4_MAX_BLKSIZE];
 		
-		if ((res = repair_semantic_add_entry(parent, object, 
-						     object->name)))
+		/* EOF was reached. Add entry to the parent. */
+		repair_semantic_lost_name(object, buff);
+		if ((res = repair_semantic_add_entry(parent, object, buff)))
 			goto error_parent_close;
 
 		/* Do not check attach here, as the pointer to parent 
@@ -288,7 +301,7 @@ static reiser4_object_t *repair_semantic_uplink(repair_semantic_t *sem,
 	
  error_object_detach:
 	/* Detach if possible. */
-	res = repair_semantic_unlink(sem, NULL, object);
+	res = repair_semantic_unlink(sem, NULL, object, NULL);
 	return res < 0 ? INVAL_PTR : NULL;
 	
  error_parent_close:
@@ -300,9 +313,8 @@ static errno_t repair_semantic_uptraverse(repair_semantic_t *sem,
 					  reiser4_object_t *parent,
 					  reiser4_object_t *object);
 
-static reiser4_object_t *callback_object_traverse(reiser4_object_t *parent, 
-						  entry_hint_t *entry,
-						  void *data)
+static reiser4_object_t *cb_object_traverse(reiser4_object_t *parent, 
+					    entry_hint_t *entry, void *data)
 {
 	repair_semantic_t *sem = (repair_semantic_t *)data;
 	reiser4_object_t *object;
@@ -319,16 +331,18 @@ static reiser4_object_t *callback_object_traverse(reiser4_object_t *parent,
 		return NULL;
 
 	/* Try to recognize the object by the key. */
-	if ((object = repair_object_launch(parent->info->tree, parent, 
-					   &entry->object)) == INVAL_PTR)
+	object = repair_object_obtain((reiser4_tree_t *)parent->ent->tree,
+				      parent, &entry->object);
+	
+	if (object == INVAL_PTR)
 		return INVAL_PTR;
 	
 	if (object == NULL) {
-		aal_error("Failed to find the object [%s] pointed "
-			  "by the entry [%s].%s",
+		fsck_mess("Directory [%s]: can't find the object "
+			  "[%s] pointed by the entry [%s].%s",
+			  reiser4_print_key(&parent->ent->object, PO_INODE),
 			  reiser4_print_key(&entry->object, PO_INODE),
-			  reiser4_print_key(&entry->offset, PO_INODE),
-			  sem->repair->mode != RM_CHECK ?
+			  entry->name, sem->repair->mode != RM_CHECK ? 
 			  " Entry is removed." : "");
 		
 		if (sem->repair->mode != RM_CHECK)
@@ -342,6 +356,15 @@ static reiser4_object_t *callback_object_traverse(reiser4_object_t *parent,
 	checked = reiser4_item_test_flag(start, OF_CHECKED);
 	attached = reiser4_item_test_flag(start, OF_ATTACHED);
 	
+	if (!checked) {
+		uint64_t val;
+		
+		sem->stat.statdatas++;
+		val = sem->stat.statdatas * 100 /sem->stat.files;
+		aal_gauge_set_value(sem->gauge, val > 100 ? 100 : val);
+		aal_gauge_touch(sem->gauge);
+	}
+	
 	res = repair_semantic_check_struct(sem, object);
 
 	if (repair_error_fatal(res))
@@ -353,13 +376,13 @@ static reiser4_object_t *callback_object_traverse(reiser4_object_t *parent,
 	while (sem->repair->mode == RM_BUILD && !attached) {
 		/* If @object knows nothing about its parent, just attach 
 		   it to the @parent. */
-		if (!object->info->parent.plug)
+		if (!object->ent->parent.plug)
 			break;
 		
 		/* If parent of the @object matches @parent, just 
 		   check_attach. */
-		if (!reiser4_key_compfull(&object->info->parent, 
-					  &parent->info->object))
+		if (!reiser4_key_compfull(&object->ent->parent, 
+					  &parent->ent->object))
 			break;
 		
 		if (!checked) {
@@ -371,15 +394,16 @@ static reiser4_object_t *callback_object_traverse(reiser4_object_t *parent,
 		} else {
 			/* If @object is checked and not attached, detach from
 			   the parent, if parent is "lost+found" -- unlink. */
+			char buff[REISER4_MAX_BLKSIZE];
 			int lost;
 
-			lost = !reiser4_key_compfull(&object->info->parent,
-						     &sem->lost->info->object);
+			lost = !reiser4_key_compfull(&object->ent->parent,
+						     &sem->lost->ent->object);
 			
-			repair_semantic_lost_name(object, object->name);
+			repair_semantic_lost_name(object, buff);
 			
 			if ((res = repair_semantic_unlink(sem, lost ? sem->lost
-							  : 0, object)))
+							  : 0, object, buff)))
 				goto error_close_object;
 		}
 
@@ -465,7 +489,7 @@ static errno_t repair_semantic_uptraverse(repair_semantic_t *sem,
 		goto error_close_upper;
 	
 	/* Traverse from the upper found object. */
-	res = reiser4_object_traverse(o, callback_object_traverse, sem);
+	res = reiser4_object_traverse(o, cb_object_traverse, sem);
 
  error_close_upper:
 	
@@ -474,7 +498,7 @@ static errno_t repair_semantic_uptraverse(repair_semantic_t *sem,
 	return res;
 }
 
-static errno_t callback_tree_scan(reiser4_place_t *place, void *data) {
+static errno_t cb_tree_scan(reiser4_place_t *place, void *data) {
 	repair_semantic_t *sem = (repair_semantic_t *)data;
 	reiser4_object_t *object;
 	errno_t res;
@@ -485,20 +509,26 @@ static errno_t callback_tree_scan(reiser4_place_t *place, void *data) {
 	/* Objects w/out SD get recovered only when reached from the parent. */
 	if (!reiser4_item_statdata(place))
 		return 0;
-		
+	
+	aal_gauge_touch(sem->gauge);
+
 	/* If this item was checked already, skip it. */
 	if (reiser4_item_test_flag(place, OF_CHECKED))
 		return 0;
 	
+	sem->stat.statdatas++;
+	aal_gauge_set_value(sem->gauge, sem->stat.statdatas * 100 /
+			    sem->stat.files);
+	
 	/* Try to open the object by its SD. */
-	object = repair_object_recognize(sem->repair->fs->tree, NULL, place);
+	object = repair_object_open(sem->repair->fs->tree, NULL, place);
 	
 	if (object == INVAL_PTR)
 		return -EINVAL;
 	else if (object == NULL)
 		return 0;
 	
-	/* Some object was openned. Check its structure and traverse from it. */
+	/* Some object was opened. Check its structure and traverse from it. */
 	res = repair_semantic_check_struct(sem, object);
 
 	if (repair_error_fatal(res))
@@ -526,53 +556,42 @@ static reiser4_object_t *repair_semantic_dir_open(repair_semantic_t *sem,
 						  reiser4_key_t *key)
 {
 	reiser4_object_t *object;
-	reiser4_plug_t *plug;
 	reiser4_tree_t *tree;
-	rid_t pid;
+	reiser4_plug_t *plug;
 	
 	aal_assert("vpf-1250", sem != NULL);
 	aal_assert("vpf-1251", key != NULL);
 	
 	tree = sem->repair->fs->tree;
 	
-	if ((object = repair_object_launch(tree, parent, key)) == INVAL_PTR)
+	if ((object = repair_object_obtain(tree, parent, key)) == INVAL_PTR)
 		return INVAL_PTR;
 	
 	if (object) {
 		/* Check that the object was recognized by the dir plugin. */
-		if (object->entity->plug->id.group == DIR_OBJECT)
+		if (object->ent->opset.plug[OPSET_OBJ]->id.group == DIR_OBJECT)
 			return object;
 
-		aal_error("The directory [%s] is recognized by the "
+		fsck_mess("The directory [%s] is recognized by the "
 			  "%s plugin which is not a directory one.", 
 			  reiser4_print_key(key, PO_INODE), 
-			  object->entity->plug->label);
+			  object->ent->opset.plug[OPSET_OBJ]->label);
 		
 		reiser4_object_close(object);
 	} else {
 		/* No plugin was recognized. */
-		aal_error("Failed to recognize the plugin for the directory "
+		fsck_mess("Failed to recognize the plugin for the directory "
 			  "[%s].", reiser4_print_key(key, PO_INODE));
 	}
 	
 	if (sem->repair->mode != RM_BUILD)
 		return NULL;
-	
-	if ((pid = reiser4_param_value("directory")) == INVAL_PID) {
-		aal_error("Can't get the valid plugin id "
-			  "for the directory plugin.");
-		return INVAL_PTR;
-	}
 
-	if (!(plug = reiser4_factory_ifind(OBJECT_PLUG_TYPE, pid))) {
-		aal_error("Can't find item plugin by its id 0x%x.", pid);
-		return INVAL_PTR;
-	}
+	plug = reiser4_profile_plug(PROF_DIR);
+	fsck_mess("Trying to recover the directory [%s] with the default "
+		  "plugin--%s.", reiser4_print_key(key, PO_INODE), plug->label);
 
-	aal_error("Trying to recover the directory [%s] with the default plugin"
-		  "--%s.", reiser4_print_key(key, PO_INODE), plug->label);
-
-	
+	sem->stat.files++;
 	return repair_object_fake(tree, parent, key, plug);
 }
 
@@ -586,6 +605,11 @@ static errno_t repair_semantic_object_check(repair_semantic_t *sem,
 	aal_assert("vpf-1268", object != NULL);
 	aal_assert("vpf-1419", parent != NULL);
 	
+	sem->stat.statdatas++;
+	aal_gauge_set_value(sem->gauge, sem->stat.statdatas * 100 /
+			    sem->stat.files);
+	aal_gauge_touch(sem->gauge);
+
 	/* Check the object. */
 	res = repair_semantic_check_struct(sem, object);
 	
@@ -593,11 +617,9 @@ static errno_t repair_semantic_object_check(repair_semantic_t *sem,
 		return res;
 	
 	while (sem->repair->mode == RM_BUILD) {
-		reiser4_plug_t *plug;
-		
 		/* Check the attach before. */
 		res = repair_object_check_attach(parent, object, 
-						 callback_register_item,
+						 cb_register_item, 
 						 sem, RM_FIX);
 		
 		if (res < 0)  
@@ -605,18 +627,15 @@ static errno_t repair_semantic_object_check(repair_semantic_t *sem,
 		else if (res == 0) 
 			break;
 		
-		/* Some problems found. Reattach it. */
-		plug = object->entity->plug;
-
-		aal_info("Object [%s]: detaching.", 
-			 reiser4_print_key(&object->info->object, PO_INODE));
+		fsck_mess("Object [%s]: detaching.", 
+			 reiser4_print_key(&object->ent->object, PO_INODE));
 		
-		if ((res = repair_semantic_unlink(sem, NULL, object)))
+		if ((res = repair_semantic_unlink(sem, NULL, object, NULL)))
 			return res;
 		
-		aal_info("Object [%s]: attaching to [%s].", 
-			 reiser4_print_key(&object->info->object, PO_INODE),
-			 reiser4_print_key(&object->info->parent, PO_INODE));
+		fsck_mess("Object [%s]: attaching to [%s].", 
+			 reiser4_print_key(&object->ent->object, PO_INODE),
+			 reiser4_print_key(&object->ent->parent, PO_INODE));
 
 		break;
 	}
@@ -637,13 +656,19 @@ static errno_t repair_semantic_root_prepare(repair_semantic_t *sem) {
 	
 	if (sem->root == NULL) {
 		sem->repair->fatal++;
-		aal_error("No root directory openned.");
+		fsck_mess("No root directory opened.");
 		return RE_FATAL;
 	} else if (sem->root == INVAL_PTR) {
 		sem->root = NULL;
 		return -EINVAL;
 	}
 	
+	reiser4_opset_profile(sem->root->ent->opset.plug);
+	if (!sem->root->ent->opset.plug[OPSET_OBJ]) {
+		sem->root->ent->opset.plug[OPSET_OBJ] = 
+			sem->root->ent->opset.plug[OPSET_MKDIR];
+	}
+	
 	if ((res = repair_semantic_object_check(sem, sem->root, sem->root))) {
 		reiser4_object_close(sem->root);
 		sem->root = NULL;
@@ -675,7 +700,7 @@ static errno_t repair_semantic_lost_prepare(repair_semantic_t *sem) {
 		if ((res = repair_fs_lost_key(fs, &lost)))
 			return res;
 		
-		aal_error("No 'lost+found' entry found. "
+		fsck_mess("No 'lost+found' entry found. "
 			  "Building a new object with the key %s.",
 			  reiser4_print_key(&lost, PO_INODE));
 	} else {
@@ -690,7 +715,7 @@ static errno_t repair_semantic_lost_prepare(repair_semantic_t *sem) {
 		return -EINVAL;
 	} else if (sem->lost == NULL) {
 		sem->repair->fatal++;
-		aal_error("No 'lost+found' directory openned.");
+		fsck_mess("No 'lost+found' directory opened.");
 		return RE_FATAL;
 	}
 
@@ -727,44 +752,22 @@ static errno_t repair_semantic_lost_prepare(repair_semantic_t *sem) {
 	return res;
 }
 
-static void repair_semantic_setup(repair_semantic_t *sem) {
-	aal_memset(sem->progress, 0, sizeof(*sem->progress));
-
-	if (!sem->progress_handler)
-		return;
-
-	sem->progress->type = GAUGE_SEM;
-	sem->progress->text = "***** Semantic Traverse Pass: reiser4 "
-		"semantic tree checking.";
-	sem->progress->state = PROGRESS_STAT;
-	time(&sem->stat.time);
-	sem->progress_handler(sem->progress);
-	sem->progress->text = NULL;
-}
-
 static void repair_semantic_update(repair_semantic_t *sem) {
 	repair_semantic_stat_t *stat;
 	aal_stream_t stream;
 	char *time_str;
 
-	if (!sem->progress_handler)
-		return;
-	
 	stat = &sem->stat;
 	aal_stream_init(&stream, NULL, &memory_stream);
 	
-	if (stat->dirs || stat->files || stat->syms || stat->spcls) {
-		aal_stream_format(&stream, "\tObject found:\n");
-		aal_stream_format(&stream, "\tDirectories %llu, Files %llu, "
-				  "Symlinks %llu, Special %llu\n", stat->dirs, 
-				  stat->files, stat->syms, stat->spcls);
+	if (stat->reached_files) {
+		aal_stream_format(&stream, "\tFound %llu objects.\n",
+				  stat->reached_files);
 	}
 
-	if (stat->ldirs || stat->lfiles || stat->lsyms || stat->lspcls) {
-		aal_stream_format(&stream, "\tLost&found of them:\n");
-		aal_stream_format(&stream, "\tDirectories %llu, Files %llu, "
-				  "Symlinks %llu, Special %llu\n", stat->ldirs, 
-				  stat->lfiles, stat->lsyms, stat->lspcls);
+	if (stat->lost_files) {
+		aal_stream_format(&stream, "\tLost&found %llu objects.\n",
+				  stat->lost_files);
 	}
 
 	if (stat->shared)
@@ -786,16 +789,11 @@ static void repair_semantic_update(repair_semantic_t *sem) {
 	time_str = ctime(&sem->stat.time);
 	time_str[aal_strlen(time_str) - 1] = '\0';
 	aal_stream_format(&stream, time_str);
-
-	sem->progress->state = PROGRESS_STAT;
-	sem->progress->text = (char *)stream.entity;
-	sem->progress_handler(sem->progress);
-
+	aal_mess(stream.entity);
 	aal_stream_fini(&stream);
 }
 
 errno_t repair_semantic(repair_semantic_t *sem) {
-	repair_progress_t progress;
 	reiser4_tree_t *tree;
 	errno_t res = 0;
 	
@@ -804,22 +802,25 @@ errno_t repair_semantic(repair_semantic_t *sem) {
 	aal_assert("vpf-1027", sem->repair->fs != NULL);
 	aal_assert("vpf-1028", sem->repair->fs->tree != NULL);
 	
-	
 	tree = sem->repair->fs->tree;
 	
 	if (reiser4_tree_fresh(tree)) {
-		aal_warn("No reiser4 metadata were found. Semantic "
-			 "pass is skipped.");
+		aal_fatal("No reiser4 metadata were found. "
+			  "Semantic pass is skipped.");
 		sem->repair->fatal++;
 		goto error;
 	}
 	
-	sem->progress = &progress;
-	repair_semantic_setup(sem);
-	
 	if ((res = reiser4_tree_load_root(tree)))
 		return res;
-	
+
+	aal_mess("CHECKING SEMANTIC TREE");
+	sem->gauge = aal_gauge_create(aux_gauge_handlers[GT_PROGRESS], 
+				      NULL, NULL, 500, NULL);
+	aal_gauge_set_value(sem->gauge, 0);
+	aal_gauge_touch(sem->gauge);
+	time(&sem->stat.time);
+
 	if (tree->root == NULL) {
 		res = -EINVAL;
 		goto error_update;
@@ -829,6 +830,9 @@ errno_t repair_semantic(repair_semantic_t *sem) {
 	if ((res = repair_semantic_root_prepare(sem)))
 		goto error_update;
 	
+	if ((res = reiser4_opset_init(tree, 0)))
+		goto error_close_root;
+	
 	/* Open "lost+found" directory in BUILD mode. */
 	if (sem->repair->mode == RM_BUILD) {
 		if ((res = repair_semantic_lost_prepare(sem)))
@@ -836,7 +840,7 @@ errno_t repair_semantic(repair_semantic_t *sem) {
 	}
 
 	/* Traverse "/" and recover all reachable subtree. */
-	res = reiser4_object_traverse(sem->root, callback_object_traverse, sem);
+	res = reiser4_object_traverse(sem->root, cb_object_traverse, sem);
 	if (res) goto error_close_lost;
 
 	reiser4_object_close(sem->root);
@@ -845,7 +849,7 @@ errno_t repair_semantic(repair_semantic_t *sem) {
 	/* Connect lost objects to their parents -- if parents can be 
 	   identified -- or to "lost+found". */
 	if (sem->repair->mode == RM_BUILD) {
-		if ((res = repair_tree_scan(tree, callback_tree_scan, sem)))
+		if ((res = reiser4_tree_scan(tree, NULL, cb_tree_scan, sem)))
 			goto error_close_lost;
 	}
 	
@@ -862,6 +866,8 @@ errno_t repair_semantic(repair_semantic_t *sem) {
 	}
 	
  error_update:
+	aal_gauge_done(sem->gauge);
+	aal_gauge_free(sem->gauge);
 	repair_semantic_update(sem);	
 	
  error:
