@@ -21,7 +21,7 @@ bool_t repair_tree_legal_level(reiser4_plug_t *plug, uint8_t level) {
 	return level == LEAF_LEVEL;
 }
 
-static errno_t callback_data_level(reiser4_plug_t *plug, void *data) {
+static errno_t cb_data_level(reiser4_plug_t *plug, void *data) {
 	uint8_t *level = (uint8_t *)data;
 	
 	aal_assert("vpf-746", data != NULL);
@@ -39,8 +39,7 @@ bool_t repair_tree_data_level(uint8_t level) {
 	if (level == 0)
 		return 0;
 	
-	return (reiser4_factory_cfind(callback_data_level,
-				      &level) != NULL);
+	return (reiser4_factory_cfind(cb_data_level, &level) != NULL);
 }
 
 /* Get the max real key existed in the tree. Go down through all right-most 
@@ -61,7 +60,7 @@ static errno_t repair_tree_maxreal_key(reiser4_tree_t *tree,
 	
 	if (reiser4_place_fetch(&place)) {
 		aal_error("Node (%llu): Failed to open the item (%u).", 
-			  node_blocknr(node), place.pos.item);
+			  node->block->nr, place.pos.item);
 		return -EINVAL;
 	}
 	
@@ -115,24 +114,6 @@ errno_t repair_tree_parent_lkey(reiser4_tree_t *tree,
 	return 0;
 }
 
-/* Gets the key of the next item. */
-errno_t repair_tree_next_key(reiser4_tree_t *tree, 
-			     reiser4_place_t *place, 
-			     reiser4_key_t *key) 
-{
-	reiser4_place_t temp;
-	
-	aal_assert("vpf-1427", tree != NULL);
-	aal_assert("vpf-1427", place != NULL);
-	aal_assert("vpf-1427", key != NULL);
-
-	temp = *place;
-	temp.pos.item++;
-	temp.pos.unit = MAX_UINT32;
-
-	return reiser4_tree_place_key(tree, &temp, key);
-}
-
 reiser4_node_t *repair_tree_load_node(reiser4_tree_t *tree, 
 				      reiser4_node_t *parent,
 				      blk_t blk, bool_t check)
@@ -182,7 +163,7 @@ errno_t repair_tree_dknode_check(reiser4_tree_t *tree,
 	/* Get the right delimiting key. */
 	if ((res = reiser4_tree_place_key(tree, &place, &dkey))) {
 		aal_error("Node (%llu): Failed to get the right "
-			  "delimiting key.", node_blocknr(node));
+			  "delimiting key.", node->block->nr);
 		return res;
 	}
 
@@ -194,16 +175,16 @@ errno_t repair_tree_dknode_check(reiser4_tree_t *tree,
 	/* Get the maxreal key of the last item. */
 	if ((res = reiser4_item_maxreal_key(&place, &key_max)) < 0) {
 		aal_error("Node (%llu): Failed to get the max real "
-			  "key of the last item.", node_blocknr(node));
+			  "key of the last item.", node->block->nr);
 		return res;
 	}
 	
 	/* Fatal error if the maxreal key greater than the right 
 	   delimiting key. */
 	if (reiser4_key_compfull(&key_max, &dkey) > 0) {
-		aal_error("Node (%llu): The last key [%s] in the node "
+		fsck_mess("Node (%llu): The last key [%s] in the node "
 			  "is greater then the right delimiting key "
-			  "[%s].", node_blocknr(node), 
+			  "[%s].", node->block->nr, 
 			  reiser4_print_key(&key_max, PO_DEFAULT),
 			  reiser4_print_key(&dkey, PO_DEFAULT));
 		return RE_FATAL;
@@ -212,7 +193,7 @@ errno_t repair_tree_dknode_check(reiser4_tree_t *tree,
 	/* Get the left delimiting key. */
 	if ((res = repair_tree_parent_lkey(tree, node, &dkey))) {
 		aal_error("Node (%llu): Failed to get the left "
-			  "delimiting key.", node_blocknr(node));
+			  "delimiting key.", node->block->nr);
 		return res;
 	}
 
@@ -228,8 +209,8 @@ errno_t repair_tree_dknode_check(reiser4_tree_t *tree,
 	if (comp > 0) {
 		/* The left delimiting key is much then the left key in the 
 		   node - not legal */
-		aal_error("Node (%llu): The first key [%s] is not equal to the "
-			  "left delimiting key [%s].", node_blocknr(node), 
+		fsck_mess("Node (%llu): The first key [%s] is not equal to the "
+			  "left delimiting key [%s].", node->block->nr, 
 			  reiser4_print_key(&place.key, PO_DEFAULT),
 			  reiser4_print_key(&dkey, PO_DEFAULT));
 		return RE_FATAL;
@@ -242,11 +223,11 @@ errno_t repair_tree_dknode_check(reiser4_tree_t *tree,
 	if (node->p.node == NULL) 
 		return 0;
 
-	aal_error("Node (%llu): The left delimiting key [%s] in the "
+	fsck_mess("Node (%llu): The left delimiting key [%s] in the "
 		  "parent node (%llu), pos (%u/%u) does not match the "
-		  "first key [%s] in the node.%s", node_blocknr(node),
+		  "first key [%s] in the node.%s", node->block->nr,
 		  reiser4_print_key(&place.key, PO_DEFAULT),
-		  node_blocknr(node->p.node), place.pos.item, 
+		  place_blknr(&node->p), place.pos.item, 
 		  place.pos.unit, reiser4_print_key(&dkey, PO_DEFAULT),
 		  mode == RM_BUILD ? " Fixed." : "");
 
@@ -398,9 +379,10 @@ errno_t repair_tree_copy(reiser4_tree_t *tree, reiser4_place_t *dst,
 }
 
 /* Lookup for the correct @place place by the @start key in the @tree. */
-static errno_t repair_tree_insert_lookup(reiser4_tree_t *tree, 
-					 reiser4_place_t *place,
-					 trans_hint_t *hint)
+static errno_t repair_tree_lookup(reiser4_tree_t *tree, 
+				  reiser4_place_t *dst,
+				  reiser4_place_t *src,
+				  reiser4_key_t *key)
 {
 	reiser4_key_t dkey, end;
 	lookup_hint_t lhint;
@@ -409,44 +391,34 @@ static errno_t repair_tree_insert_lookup(reiser4_tree_t *tree,
 	errno_t res;
 	
 	aal_assert("vpf-1364", tree  != NULL);
-	aal_assert("vpf-1365", place != NULL);
-	aal_assert("vpf-1367", hint  != NULL);
+	aal_assert("vpf-1365", dst != NULL);
+	aal_assert("vpf-1367", src != NULL);
+	aal_assert("vpf-1689", key != NULL);
 
 	aal_memset(&lhint, 0, sizeof(lhint));
 	lhint.level = LEAF_LEVEL;
-	lhint.key = &hint->offset;
+	lhint.key = key;
 	lhint.collision = NULL;
 	lhint.hint = NULL;
 	
-	switch ((res = reiser4_tree_lookup(tree, &lhint, FIND_EXACT, place))) {
-	case PRESENT:
-		/* The whole item can not be inserted. */
-		if (place->pos.unit == MAX_UINT32)
-			place->pos.unit = 0;
-
-		if (reiser4_place_fetch(place))
-			return -EIO;
-
-		return 0;
-	case ABSENT:
-		break;
-	default:
+	res = reiser4_tree_lookup(tree, &lhint, FIND_EXACT, dst);
+	
+	if (res != ABSENT)
 		return res;
-	}
 
 	/* Absent. If non-existent unit or item, there is nothing mergable 
 	   from the right side--lookup would go down there in that case.  */
-	if (reiser4_place_right(place))
+	if (reiser4_place_right(dst))
 		/* Step to right. */
-		reiser4_place_inc(place, 1);
+		reiser4_place_inc(dst, 1);
 
-	if (reiser4_place_rightmost(place)) {
-		prev = *place;
+	if (reiser4_place_rightmost(dst)) {
+		prev = *dst;
 
 		reiser4_node_lock(prev.node);
 		
-		if ((res = reiser4_tree_next_place(tree, place, place))) {
-			aal_error("vpf-1363: Failed to get the next node.");
+		if ((res = reiser4_tree_next_place(tree, dst, dst))) {
+			aal_error("Failed to get the next node.");
 			reiser4_node_unlock(prev.node);
 			return res;
 		}
@@ -454,45 +426,48 @@ static errno_t repair_tree_insert_lookup(reiser4_tree_t *tree,
 		reiser4_node_unlock(prev.node);
 		
 		/* No right node. */
-		if (!place->node) {
-			*place = prev;
-			return 0;
-		}
-	} else 
+		if (!dst->node)
+			goto finish;
+	} else {
 		aal_memset(&prev, 0, sizeof(prev));
+	}
 
-	/* Get the current key of the @place. */
-	if (reiser4_place_fetch(place))
+	/* Get the current key of the @dst. */
+	if (reiser4_place_fetch(dst))
 		return -EIO;
 
-	if ((res = reiser4_item_get_key(place, &dkey)))
+	if ((res = reiser4_item_get_key(dst, &dkey)))
 		return res;
 
-	if ((res = reiser4_item_maxreal_key((reiser4_place_t *)hint->specific, &end)))
+	if ((res = reiser4_item_maxreal_key(src, &end)))
 		return res;
 	
 	/* If @end key is not less than the lookuped, items are overlapped. 
 	   Othewise move to the previous pos. */
-	if ((res = reiser4_key_compfull(&end, &dkey)) >= 0) {
-		if (place->pos.unit == MAX_UINT32)
-			place->pos.unit = 0;
-	} else if (prev.node) {
-		*place = prev;
-	}
+	if ((res = reiser4_key_compfull(&end, &dkey)) >= 0) 
+		return PRESENT;
 
-	return 0;
+ finish:
+	if (prev.node)
+		*dst = prev;
+
+	/* Do not care about the  */
+	dst->plug = NULL;
+
+	return ABSENT;
 }
 
-static errno_t callback_prep_merge(reiser4_place_t *place, trans_hint_t *hint) {
-	return plug_call(hint->plug->o.item_ops->repair,
-			 prep_merge, place, hint);
-}
-
-static errno_t callback_merge(reiser4_node_t *node, pos_t *pos,
-			      trans_hint_t *hint)
+static errno_t cb_prep_insert_raw(reiser4_place_t *place, 
+				  trans_hint_t *hint) 
 {
-	return plug_call(node->plug->o.node_ops, 
-			 merge, node, pos, hint);
+	return plug_call(hint->plug->o.item_ops->repair, 
+			 prep_insert_raw, place, hint);
+}
+
+static errno_t cb_insert_raw(reiser4_node_t *node, pos_t *pos, 
+			     trans_hint_t *hint) 
+{
+	return plug_call(node->plug->o.node_ops, insert_raw, node, pos, hint);
 }
 
 /* Insert the item into the tree overwriting an existent in the tree item 
@@ -537,76 +512,75 @@ errno_t repair_tree_insert(reiser4_tree_t *tree, reiser4_place_t *src,
 	level = reiser4_node_get_level(src->node);
 	
 	while (1) {
-		if ((res = repair_tree_insert_lookup(tree, &dst, &hint)))
+		if ((res = repair_tree_lookup(tree, &dst, 
+					      src, &hint.offset)) < 0)
+		{
 			return res;
+		}
 		
 		/* Convert @dst if needed. */
-		if (dst.pos.unit != MAX_UINT32) {
+		if (dst.plug) {
 			bool_t conv;
 
 			conv = repair_tree_need_conv(tree, dst.plug, src->plug);
 			
-			switch (conv) {
-			case 1:
+			if (conv < 0) {
+				/* Convertion is not possible. */
+				fsck_mess("Node (%llu), item (%u): the item (%s) "
+					  "[%s] is overlapped by keys with the "
+					  "item (%s) [%s] being inserted [node "
+					  "%llu, item %u]. Skip insertion.",
+					  place_blknr(&dst), dst.pos.item,
+					  dst.plug->label,
+					  reiser4_print_key(&dst.key, PO_DEFAULT),
+					  src->plug->label,
+					  reiser4_print_key(&src->key, PO_DEFAULT),
+					  place_blknr(src), src->pos.item);
+
+				   return RE_FATAL;
+			}
+			
+			if (conv) {
 				res = repair_tree_conv(tree, &dst, src);
 				if (res) return res;
 
 				/* Repeat lookup after @dst conversion. */
 				continue;
-			case 0:
-				break;
-			default:
-				/* Conversion cannot be performed. */
-				aal_error("Node (%llu), item (%u): the item (%s) "
-					  "[%s] is overlapped by keys with the "
-					  "item (%s) [%s] being inserted [node "
-					  "%llu, item %u]. Skip insertion.",
-					  node_blocknr(dst.node), dst.pos.item,
-					  dst.plug->label,
-					  reiser4_print_key(&dst.key, PO_DEFAULT),
-					  src->plug->label,
-					  reiser4_print_key(&src->key, PO_DEFAULT),
-					  node_blocknr(src->node), src->pos.item);
-
-				   return 0;
 			}
+			
+			/* Convertion is not needed, places are 
+			   overlapped, adjust the position. */
+			if (dst.pos.unit == MAX_UINT32)
+				dst.pos.unit = 0;
 		}
 
-		/* If dst points to some existent item of the same plugin type
-		   as src item and merge method is implemented for this plugin,
-		   call merge. Call it also if dst points to not existent item. 
-		 */
-		if (!hint.plug->o.item_ops->repair->merge) {
-			/* For items without merge method implemented, like SD. */
-			return 0;
-		} else if ((dst.plug && plug_equal(dst.plug, src->plug)) || 
-			   !dst.plug)
+		/* Insert_raw can insert the whole item (!dst.plug) or if dst &
+		   src items are of the same plugin. Call tree_copy otherwise. */
+		if ((dst.plug && plug_equal(dst.plug, src->plug)) || !dst.plug)
 		{
-			if ((res = reiser4_tree_modify(tree, &dst, &hint, level,
-						       callback_prep_merge,
-						       callback_merge)))
-				goto error;
+			res = reiser4_tree_modify(tree, &dst, &hint, level,
+						  cb_prep_insert_raw,
+						  cb_insert_raw);
+			
+			if (res) goto error;
 		} else {
 			/* For not equal plugins do coping. */
-			aal_error("Node (%llu), item (%u): the item "
-				  "(%s) [%s] is overlapped by keys "
-				  "with the item (%s) [%s] being "
-				  "inserted [node %llu, item %u]. "
-				  "Copying is not ready yet, skip "
-				  "insertion.", 
-				  node_blocknr(dst.node), dst.pos.item,
-				  dst.plug->label, 
+			fsck_mess("Node (%llu), item (%u): the item (%s) [%s] "
+				  "is overlapped by keys with the item (%s) "
+				  "[%s] being inserted [node %llu, item %u]. "
+				  "Copying is not ready yet, skip insertion.",
+				  place_blknr(&dst), dst.pos.item, 
+				  dst.plug->label,
 				  reiser4_print_key(&dst.key, PO_DEFAULT),
 				  src->plug->label,
 				  reiser4_print_key(&src->key, PO_DEFAULT),
-				  node_blocknr(src->node), 
-				  src->pos.item);
+				  place_blknr(src), src->pos.item);
 
 			/* FIXME: Copying must exist for tail->extent.
 			if ((res = repair_tree_copy(tree, &dst, src, &key)))
 				return res;
 			*/
-			return 0;
+			return RE_FATAL;
 		}
 
 		if (!src->plug->o.item_ops->balance->lookup)
@@ -632,90 +606,16 @@ errno_t repair_tree_insert(reiser4_tree_t *tree, reiser4_place_t *src,
 	if (dst.pos.unit == MAX_UINT32) {
 		aal_error("Node (%llu), item (%u): insertion to the "
 			  "node (%llu), item (%u) failed.", 
-			  node_blocknr(src->node), src->pos.item, 
-			  node_blocknr(dst.node),  dst.pos.item);
+			  place_blknr(src), src->pos.item, 
+			  place_blknr(&dst),  dst.pos.item);
 	} else {
 		aal_error("Node (%llu), item (%u): merging with node "
 			  "(%llu), item (%u) by the key [%s] failed.",
-			  node_blocknr(src->node), src->pos.item, 
-			  node_blocknr(dst.node), dst.pos.item,
+			  place_blknr(src), src->pos.item, 
+			  place_blknr(&dst), dst.pos.item,
 			  reiser4_print_key(&hint.offset, PO_INODE));
 	}
 	
 	return res;
 }
 
-errno_t repair_tree_scan(reiser4_tree_t *tree, place_func_t func, void *data) {
-        reiser4_key_t key, max;
-        uint32_t count;
-        errno_t res;
-
-        aal_assert("vpf-1423", tree != NULL);
-        aal_assert("vpf-1424", func != NULL);
-
-        if (reiser4_tree_fresh(tree))
-                return -EINVAL;
-
-        if ((res = reiser4_tree_load_root(tree)))
-                return res;
-
-        if (tree->root == NULL)
-                return -EINVAL;
-
-        /* Prepare the start and the end keys. */
-        key.plug = max.plug = tree->key.plug;
-        reiser4_key_minimal(&key);
-        reiser4_key_maximal(&max);
-
-        /* While not the end of the tree. */
-        while (reiser4_key_compfull(&key, &max)) {
-                reiser4_place_t place;
-		lookup_hint_t hint;
-                lookup_t lookup;
-		pos_t *pos;
-
-                /* FIXME-VITALY: This is not key-collision-safe. */
-		hint.key = &key;
-		hint.level = LEAF_LEVEL;
-		hint.collision = NULL;
-
-                /* Lookup the key. */
-                if ((lookup = reiser4_tree_lookup(tree, &hint, FIND_EXACT, 
-						  &place)) < 0)
-                        return lookup;
-
-		pos = &place.pos;
-
-                for (; pos->item < reiser4_node_items(place.node); pos->item++)	{
-                        if ((res = reiser4_place_fetch(&place)))
-                                return res;
-			
-			/* Go down to the child if branch. */
-			if ((res = reiser4_item_branch(place.plug))) {
-				place.node = 
-					reiser4_tree_child_node(tree, &place);
-				
-				if (!place.node) return -EIO;
-
-				count = reiser4_node_items(place.node);
-				place.pos.item = -1;
-				
-				continue;
-			}
-			
-                        /* Get the key of the next item. */
-                        if ((res = repair_tree_next_key(tree, &place, &key)))
-                                return res;
-
-                        /* Call func for the item. */
-                        if ((res = func(&place, data)) < 0)
-                                return res;
-
-                        /* If res != 0, lookup is needed. */
-                        if (res) break;
-                }
-        }
-
-        return 0;
-}
-
