@@ -10,7 +10,7 @@
 #include "alloc40_repair.h"
 
 #define ALLOC40_BLOCKNR(blksize) \
-        ((REISER4_MASTER_OFFSET / blksize) + 2)
+        (REISER4_MASTER_BLOCKNR(blksize) + 2)
 
 /* Calculates the adler32 checksum for the data pointed by "buff" of the length
    "n". This function was originally taken from zlib, version 1.1.3, July 9th,
@@ -104,7 +104,7 @@ errno_t alloc40_layout(generic_entity_t *entity,
 	for (blk = start; blk < start + alloc->bitmap->total;
 	     blk = ((blk / bpb) + 1) * bpb) 
 	{
-		res = region_func(entity, blk, 1, data);
+		res |= region_func(entity, blk, 1, data);
 		
 		if (res && res != -ESTRUCT)
 			return res;
@@ -117,8 +117,8 @@ errno_t alloc40_layout(generic_entity_t *entity,
    saves it in allocator checksums area. Actually this function is callback one
    which is called by alloc40_layout function in order to load all bitmap map
    from the device. See alloc40_open for details. */
-static errno_t callback_fetch_bitmap(void *entity, blk_t start,
-				     count_t width, void *data)
+static errno_t cb_fetch_bitmap(void *entity, blk_t start,
+			       count_t width, void *data)
 {
 	errno_t res;
 	uint64_t offset;
@@ -201,9 +201,7 @@ static generic_entity_t *alloc40_open(fs_desc_t *desc, uint64_t blocks) {
 
 	/* Calling alloc40_layout() method with fetch_bitmap() callback to load
 	   all bitmap blocks. */
-	if (alloc40_layout((generic_entity_t *)alloc,
-			   callback_fetch_bitmap, alloc))
-	{
+	if (alloc40_layout((generic_entity_t *)alloc, cb_fetch_bitmap, alloc)) {
 		aal_error("Can't load ondisk bitmap.");
 		goto error_free_bitmap;
 	}
@@ -295,8 +293,8 @@ static errno_t alloc40_extract(generic_entity_t *entity, void *data) {
 }
 
 /* Callback for saving one bitmap block onto device */
-static errno_t callback_sync_bitmap(void *entity, blk_t start,
-				    count_t width, void *data)
+static errno_t cb_sync_bitmap(void *entity, blk_t start,
+			      count_t width, void *data)
 {
 	errno_t res;
 	alloc40_t *alloc;
@@ -371,7 +369,7 @@ static errno_t alloc40_sync(generic_entity_t *entity) {
 
 	/* Calling layout() function for saving all bitmap blocks to device
 	   block allocator lies on. */
-	if ((res = alloc40_layout(entity, callback_sync_bitmap, alloc))) {
+	if ((res = alloc40_layout(entity, cb_sync_bitmap, alloc))) {
 		aal_error("Can't save bitmap to device.");
 		return res;
 	}
@@ -505,24 +503,21 @@ static int alloc40_available(generic_entity_t *entity,
 				      start, count, 0);
 }
 
-static void callback_inval_warn(blk_t start, uint32_t ladler,
-				uint32_t cadler)
-{
-	aal_warn("Checksum missmatch in bitmap block %llu. Checksum "
-		 "is 0x%x, should be 0x%x.", start, ladler, cadler);
+static void cb_inval_warn(blk_t start, uint32_t ladler, uint32_t cadler) {
+	aal_error("Checksum missmatch in bitmap block %llu. Checksum "
+		  "is 0x%x, should be 0x%x.", start, ladler, cadler);
 }
 
 typedef void (*inval_func_t) (blk_t, uint32_t, uint32_t);
 
 /* Callback function for checking one bitmap block on validness. Here we just
    calculate actual checksum and compare it with loaded one. */
-errno_t callback_valid_block(void *entity, blk_t start,
-			     count_t width, void *data)
-{
+errno_t cb_valid_block(void *entity, blk_t start, count_t width, void *data) {
 	uint32_t chunk;
 	uint64_t offset;
 	alloc40_t *alloc;
-	
+	errno_t res;
+
 	uint32_t size, free;
 	char *current, *map;
 	inval_func_t inval_func;
@@ -560,12 +555,12 @@ errno_t callback_valid_block(void *entity, blk_t start,
 
 	/* If loaded checksum and calculated one are not equal, we have
 	   corrupted bitmap. */
-	if (ladler != cadler && inval_func) {
+	res = (ladler != cadler) ? -ESTRUCT : 0;
+	
+	if (res && inval_func)
 		inval_func(start, ladler, cadler);
-		return -ESTRUCT;
-	}
 
-	return 0;
+	return res;
 }
 
 /* Checks allocator on validness  */
@@ -578,8 +573,7 @@ errno_t alloc40_valid(generic_entity_t *entity) {
 	/* Calling layout function for traversing all the bitmap blocks with
 	   checking callback function. */
 	return alloc40_layout((generic_entity_t *)alloc,
-			      callback_valid_block,
-			      callback_inval_warn);
+			      cb_valid_block, cb_inval_warn);
 }
 
 /* Call @func for all blocks which belong to the same bitmap block as passed
@@ -638,7 +632,8 @@ static reiser4_alloc_ops_t alloc40_ops = {
 	.region		= alloc40_region,
 	.occupy	        = alloc40_occupy,
 	.allocate       = alloc40_allocate,
-	.release        = alloc40_release
+	.release        = alloc40_release,
+	.check_struct   = alloc40_check_struct
 };
 
 reiser4_plug_t alloc40_plug = {
