@@ -1,14 +1,14 @@
 /* Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
-   tail40_repair.c -- reiser4 default tail plugin. */
+   bbox40_repair.c -- reiser4 default black box plugin. */
 
 
 #ifndef ENABLE_STAND_ALONE
 #include <repair/plugin.h>
 #include "bbox40_repair.h"
 
-errno_t bbox40_check_struct(reiser4_place_t *place, uint8_t mode) {
+errno_t bbox40_check_struct(reiser4_place_t *place, repair_hint_t *hint) {
 	uint64_t type;
 	uint8_t size;
 	
@@ -16,9 +16,11 @@ errno_t bbox40_check_struct(reiser4_place_t *place, uint8_t mode) {
 	type = plug_call(place->key.plug->o.key_ops, get_offset, &place->key);
 
 	if (type >= SL_LAST) {
-		aal_error("Node (%llu), item (%u): safe link item (%s) of the "
-			  "unknown type (%llu) found.", place->node->block->nr, 
-			  place->pos.item, place->plug->label, type);
+		fsck_mess("Node (%llu), item (%u), [%s]: safe link "
+			  "item (%s) of the unknown type (%llu) found.",
+			  place_blknr(place), place->pos.item, 
+			  print_key(bbox40_core, &place->key),
+			  place->plug->label, type);
 		
 		return RE_FATAL;
 	}
@@ -30,9 +32,10 @@ errno_t bbox40_check_struct(reiser4_place_t *place, uint8_t mode) {
 		size += sizeof(uint64_t);
 
 	if (size != place->len) {
-		aal_error("Node (%llu), item (%u): safe link item (%s) of "
-			  "the wrong length (%u) found. Should be (%u).", 
-			  place->node->block->nr, place->pos.item, 
+		fsck_mess("Node (%llu), item (%u), [%s]: safe link item (%s) "
+			  "of the wrong length (%u) found. Should be (%u).",
+			  place_blknr(place), place->pos.item, 
+			  print_key(bbox40_core, &place->key),
 			  place->plug->label, place->len, size);
 		
 		return RE_FATAL;
@@ -65,7 +68,7 @@ void bbox40_print(reiser4_place_t *place, aal_stream_t *stream,
 	aal_memcpy(&key.body, place->body, size);
 
 	aal_stream_format(stream, "UNITS=1\n    %s  %s",
-			  bbox40_core->key_ops.print(&key, PO_DEFAULT),
+			  print_key(bbox40_core, &key),
 			  reiser4_slink_name[type]);
 	
 	if (type == SL_TRUNCATE) {
@@ -76,4 +79,42 @@ void bbox40_print(reiser4_place_t *place, aal_stream_t *stream,
 	aal_stream_format(stream, "\n");
 }
 
+errno_t bbox40_prep_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
+	reiser4_place_t *src;
+	
+	aal_assert("vpf-1662", place != NULL);
+	aal_assert("vpf-1663", hint != NULL);
+	aal_assert("vpf-1664", hint->specific != NULL);
+	
+	src = (reiser4_place_t *)hint->specific;
+
+	hint->overhead = 0;
+	hint->bytes = 0;
+
+	if (place->pos.unit == MAX_UINT32) {
+		hint->count = 1;
+		hint->len = src->len;
+	} else {
+		hint->count = hint->len = 0;
+	}
+	
+	return 0;
+}
+
+errno_t bbox40_insert_raw(reiser4_place_t *place, trans_hint_t *hint) {
+	reiser4_place_t *src;
+	
+	aal_assert("vpf-1665", place != NULL);
+	aal_assert("vpf-1666", hint != NULL);
+	aal_assert("vpf-1667", hint->specific != NULL);
+
+	if (!hint->len) return 0;
+	
+	src = (reiser4_place_t *)hint->specific;
+	aal_memcpy(place->body, src->body, hint->len);
+	place_mkdirty(place);
+	
+	return 0;
+}
+
 #endif
