@@ -32,9 +32,9 @@ static inline void *cde40_hash(reiser4_place_t *place, uint32_t pos) {
 static void cde40_get_obj(reiser4_place_t *place, uint32_t pos,
 			  reiser4_key_t *key)
 {
+	aal_memset(key, 0, sizeof(*key));
+	
 	key->plug = place->key.plug;
-	plug_call(key->plug->o.key_ops, clean, key);
-
 	aal_memcpy(key->body, cde40_objid(place, pos),
 		   ob_size(cde40_key_pol(place)));
 }
@@ -62,6 +62,7 @@ errno_t cde40_get_hash(reiser4_place_t *place, uint32_t pos,
 	return 0;
 }
 
+#ifndef ENABLE_STAND_ALONE
 /* Set the key for the entry->offset. It is needed for fixing entry keys if
    repair code detects it is wrong. */
 errno_t cde40_set_hash(reiser4_place_t *place, uint32_t pos,
@@ -91,6 +92,7 @@ errno_t cde40_set_hash(reiser4_place_t *place, uint32_t pos,
 	
 	return 0;
 }
+#endif
 
 /* Extracts entry name from the passed @entry to passed @buff */
 char *cde40_get_name(reiser4_place_t *place, uint32_t pos,
@@ -139,6 +141,16 @@ static uint32_t cde40_get_len(reiser4_place_t *place, uint32_t pos) {
 	
 	return len;
 }
+
+/* Updates entry offset. It is needed for fixing entry keys if repair code
+   detects it is wrong. */
+errno_t cde40_update_key(reiser4_place_t *place, reiser4_key_t *key) {
+	aal_assert("vpf-1228", key != NULL);
+	aal_assert("vpf-1229", place != NULL);
+	aal_assert("vpf-1230", place->body != NULL);
+
+	return cde40_set_hash(place, place->pos.unit, key);
+}
 #endif
 
 /* Builds full key by entry components. It is needed for updating keys after
@@ -151,16 +163,6 @@ errno_t cde40_fetch_key(reiser4_place_t *place, reiser4_key_t *key) {
 	return cde40_get_hash(place, place->pos.unit, key);
 }
 
-/* Updates entry offset. It is needed for fixing entry keys if repair code
-   detects it is wrong. */
-errno_t cde40_update_key(reiser4_place_t *place, reiser4_key_t *key) {
-	aal_assert("vpf-1228", key != NULL);
-	aal_assert("vpf-1229", place != NULL);
-	aal_assert("vpf-1230", place->body != NULL);
-
-	return cde40_set_hash(place, place->pos.unit, key);
-}
-
 /* Returns the number of units. */
 uint32_t cde40_units(reiser4_place_t *place) {
 	aal_assert("umka-865", place != NULL);
@@ -322,6 +324,7 @@ static uint32_t cde40_shrink(reiser4_place_t *place,
 
 	aal_assert("vpf-1561", place != NULL);
 
+	pol = cde40_key_pol(place);
 	units = cde_get_units(place);
 
 	aal_assert("vpf-1562", pos < units);
@@ -946,7 +949,7 @@ lookup_t cde40_collision(reiser4_place_t *place, coll_hint_t *hint) {
 	
 	aal_assert("vpf-1548", place != NULL);
 	aal_assert("vpf-1549", hint != NULL);
-	aal_assert("vpf-1549", hint->type == DIRENTRY_ITEM);
+	aal_assert("vpf-1549", hint->type == DIR_ITEM);
 
 	if (place->pos.unit == MAX_UINT32)
 		place->pos.unit = 0;
@@ -1022,8 +1025,8 @@ int cde40_comp_hash(reiser4_place_t *place, uint32_t pos, reiser4_key_t *key) {
 
 /* Helper function that is used by lookup method for 
    comparing given key with passed entry hash. */
-static int callback_comp_hash(void *array, uint32_t pos,
-			      void *key, void *data)
+static int cb_comp_hash(void *array, uint32_t pos,
+			void *key, void *data)
 {
 	return cde40_comp_hash((reiser4_place_t *)data, pos, 
 			       (reiser4_key_t *)key);
@@ -1044,7 +1047,7 @@ lookup_t cde40_lookup(reiser4_place_t *place,
 	/* Bin search within the cde item to get the position of 
 	   the wanted key. */
 	switch (aux_bin_search(place->body, cde40_units(place),
-			       hint->key, callback_comp_hash,
+			       hint->key, cb_comp_hash,
 			       place, &place->pos.unit))
 	{
 	case 1:
@@ -1103,28 +1106,25 @@ static item_object_ops_t object_ops = {
 	.trunc_units	  = NULL,
 	.layout		  = NULL,
 #endif
-	.read_units	  = NULL,
-	.object_plug	  = NULL
+	.read_units	  = NULL
 };
 
-static item_debug_ops_t debug_ops = {
 #ifndef ENABLE_STAND_ALONE
+static item_debug_ops_t debug_ops = {
 	.print		  = cde40_print
-#endif
 };
 
 static item_repair_ops_t repair_ops = {
-#ifndef ENABLE_STAND_ALONE
 	.check_struct	  = cde40_check_struct,
 	.check_layout	  = NULL,
 
-	.prep_merge	  = cde40_prep_merge,
-	.merge		  = cde40_merge,
+	.prep_insert_raw  = cde40_prep_insert_raw,
+	.insert_raw	  = cde40_insert_raw,
 
 	.pack		  = NULL,
 	.unpack		  = NULL
-#endif
 };
+#endif
 
 static item_tree_ops_t tree_ops = {
 	.down_link	  = NULL,
@@ -1135,15 +1135,17 @@ static item_tree_ops_t tree_ops = {
 
 static reiser4_item_ops_t cde40_ops = {
 	.tree		  = &tree_ops,
-	.debug		  = &debug_ops,
 	.object		  = &object_ops,
+	.balance	  = &balance_ops,
+#ifndef ENABLE_STAND_ALONE
 	.repair		  = &repair_ops,
-	.balance	  = &balance_ops
+	.debug		  = &debug_ops
+#endif
 };
 
 static reiser4_plug_t cde40_plug = {
 	.cl    = class_init,
-	.id    = {ITEM_CDE40_ID, DIRENTRY_ITEM, ITEM_PLUG_TYPE},
+	.id    = {ITEM_CDE40_ID, DIR_ITEM, ITEM_PLUG_TYPE},
 #ifndef ENABLE_STAND_ALONE
 	.label = "cde40",
 	.desc  = "Compound direntry for reiser4, ver. " VERSION,
