@@ -18,9 +18,9 @@ uint32_t extent40_units(reiser4_place_t *place) {
 
 #ifdef ENABLE_DEBUG
 	if (place->len % sizeof(extent40_t) != 0) {
-		aal_error("Invalid extent item size (%u) detected. "
-			  "Node %llu, item %u.", place->len,
-			  place->node->block->nr, place->pos.item);
+		aal_error("Node (%llu), item (%u): Invalid extent "
+			  "item size (%u) detected.", place_blknr(place),
+			  place->pos.item, place->len);
 		return 0;
 	}
 #endif
@@ -31,7 +31,8 @@ uint32_t extent40_units(reiser4_place_t *place) {
 /* Calculates extent size. */
 uint64_t extent40_offset(reiser4_place_t *place, uint32_t pos) {
 	extent40_t *extent;
-	uint32_t i, blocks = 0;
+	uint64_t blocks = 0;
+	uint32_t i;
     
 	aal_assert("umka-2204", place != NULL);
 	
@@ -40,7 +41,7 @@ uint64_t extent40_offset(reiser4_place_t *place, uint32_t pos) {
 	for (i = 0; i < pos; i++, extent++)
 		blocks += et40_get_width(extent);
     
-	return (blocks * extent40_blksize(place));
+	return blocks * extent40_blksize(place);
 }
 
 /* Builds the key of the unit at @pos and stores it inside passed @key
@@ -341,8 +342,8 @@ lookup_t extent40_lookup(reiser4_place_t *place,
 			   get_offset, &place->key);
 	
 	for (i = 0; i < units; i++, extent++) {
-		offset += et40_get_width(extent) *
-			extent40_blksize(place);
+		offset += (et40_get_width(extent) *
+			extent40_blksize(place));
 
 		if (offset > wanted) {
 			place->pos.unit = i;
@@ -374,7 +375,7 @@ static int64_t extent40_read_units(reiser4_place_t *place,
 	uint64_t block_offset;
 
 	aal_assert("umka-1421", place != NULL);
-	aal_assert("umka-1422", buff != NULL);
+	aal_assert("umka-1422", hint != NULL);
 
 	count = hint->count;
 	buff = hint->specific;
@@ -769,7 +770,7 @@ static errno_t extent40_prep_write(reiser4_place_t *place,
 
 	hint->len = 0;
 	hint->overhead = 0;
-		
+	
 	if (place->pos.unit == MAX_UINT32) {
 		/* Assigning maxkey to key of new created item. */
 		plug_call(hint->offset.plug->o.key_ops,
@@ -1385,7 +1386,6 @@ static item_balance_ops_t balance_ops = {
 };
 
 static item_object_ops_t object_ops = {
-	.object_plug	  = NULL,
 #ifndef ENABLE_STAND_ALONE
 	.remove_units	  = extent40_remove_units,
 	.update_units	  = extent40_update_units,
@@ -1403,24 +1403,22 @@ static item_object_ops_t object_ops = {
 	.fetch_units	  = extent40_fetch_units
 };
 
-static item_repair_ops_t repair_ops = {
 #ifndef ENABLE_STAND_ALONE
+static item_repair_ops_t repair_ops = {
 	.check_layout	  = extent40_check_layout,
 	.check_struct	  = extent40_check_struct,
 	
-	.prep_merge	  = extent40_prep_merge,
-	.merge		  = extent40_merge,
+	.prep_insert_raw  = extent40_prep_insert_raw,
+	.insert_raw	  = extent40_insert_raw,
 
 	.pack		  = NULL,
 	.unpack		  = NULL
-#endif
 };
 
 static item_debug_ops_t debug_ops = {
-#ifndef ENABLE_STAND_ALONE
 	.print		  = extent40_print,
-#endif
 };
+#endif
 
 static item_tree_ops_t tree_ops = {
 	.down_link	  = NULL,
@@ -1431,10 +1429,12 @@ static item_tree_ops_t tree_ops = {
 
 static reiser4_item_ops_t extent40_ops = {
 	.tree		  = &tree_ops,
-	.debug		  = &debug_ops,
 	.object		  = &object_ops,
+	.balance	  = &balance_ops,
+#ifndef ENABLE_STAND_ALONE
 	.repair		  = &repair_ops,
-	.balance	  = &balance_ops
+	.debug		  = &debug_ops,
+#endif
 };
 
 static reiser4_plug_t extent40_plug = {
