@@ -8,22 +8,24 @@
 
 #include <aal/libaal.h>
 #include <reiser4/plugin.h>
+#include "extent40.h"
+
+extern errno_t extent40_prep_insert_raw(reiser4_place_t *place,
+				   trans_hint_t *hint);
+
+extern errno_t extent40_insert_raw(reiser4_place_t *place,
+				   trans_hint_t *hint);
+
+extern errno_t extent40_check_struct(reiser4_place_t *place,
+				     repair_hint_t *hint);
+
+extern errno_t extent40_check_layout(reiser4_place_t *place, 
+ 				     repair_hint_t *hint, 
+				     region_func_t region_func,
+				     void *data);
 
 extern void extent40_print(reiser4_place_t *place,
 			   aal_stream_t *stream, 
 			   uint16_t options);
 
-extern errno_t extent40_merge(reiser4_place_t *place,
-			      trans_hint_t *hint);
-
-extern errno_t extent40_check_struct(reiser4_place_t *place,
-				     uint8_t mode);
-
-extern errno_t extent40_prep_merge(reiser4_place_t *place,
-				   trans_hint_t *hint);
-
-extern errno_t extent40_check_layout(reiser4_place_t *place,
-				     region_func_t region_func,
- 				     void *data, uint8_t mode);
-
 #endif
