@@ -9,8 +9,14 @@
 #include <aal/libaal.h>
 #include <reiser4/plugin.h>
 
+extern errno_t stat40_prep_insert_raw(reiser4_place_t *place, 
+				      trans_hint_t *hint);
+
+extern errno_t stat40_insert_raw(reiser4_place_t *place, 
+				 trans_hint_t *hint);
+
 extern errno_t stat40_check_struct(reiser4_place_t *place,
-				   uint8_t mode);
+				   repair_hint_t *hint);
 
 extern void stat40_print(reiser4_place_t *place, 
 			 aal_stream_t *stream, 
