@@ -94,8 +94,8 @@ static char *__blk_type_name(journal40_block_t blk_type) {
 }
 
 /* Callback for format.layout. Returns 1 for all fotmat blocks. */
-static errno_t callback_check_format_block(void *entity, blk_t start,
-					   count_t width, void *data)
+static errno_t cb_check_format_block(void *entity, blk_t start,
+				     count_t width, void *data)
 {
 	blk_t blk = *(blk_t *)data;
 	return (blk >= start && blk < start + width);
@@ -118,7 +118,7 @@ static errno_t journal40_blk_format_check(journal40_t *journal, blk_t blk,
 		return 0;
 	
 	/* blk belongs to format area */
-	return data->fs_layout(data->layout, callback_check_format_block, &blk) ? 
+	return data->fs_layout(data->layout, cb_check_format_block, &blk) ? 
 		-ESTRUCT : 0;
 }
 
@@ -126,8 +126,7 @@ static errno_t journal40_blk_format_check(journal40_t *journal, blk_t blk,
    block number equals to wanted blk. Set found_type then to TxH. Returns 1 
    when traverse should be stopped, zeroes found_type if no satisfied 
    transaction was found. */
-static errno_t callback_find_txh_blk(generic_entity_t *entity, blk_t blk, 
-				     void *data) 
+static errno_t cb_find_txh_blk(generic_entity_t *entity, blk_t blk, void *data)
 {
 	journal40_check_t *check_data = (journal40_check_t *)data;
 	
@@ -152,11 +151,8 @@ static errno_t callback_find_txh_blk(generic_entity_t *entity, blk_t blk,
 /* Secondary (not TxH) blocks callback for nested traverses. Should find the 
    transaction which contains block number equal to wanted blk. Set wanted_blk 
    to TxH block number and found_type to the type of found blk. */
-static errno_t callback_find_sec_blk(generic_entity_t *entity, 
-				     aal_block_t *txh_block, 
-				     blk_t blk, 
-				     journal40_block_t blk_type, 
-				     void *data) 
+static errno_t cb_find_sec_blk(generic_entity_t *entity, aal_block_t *txh_block,
+			       blk_t blk, journal40_block_t blk_type,void *data)
 {
 	journal40_check_t *check_data = (journal40_check_t *)data;
 	
@@ -172,8 +168,8 @@ static errno_t callback_find_sec_blk(generic_entity_t *entity,
 /* TxH callback for traverse. Returns 1 if blk is a block out of format bound 
    or of format area or is met more then once. data->cur_txh = 0 all the way
    here to explain the traverse caller that the whole journal is invalid. */
-static errno_t callback_journal_txh_check(generic_entity_t *entity, blk_t blk, 
-					  void *data) 
+static errno_t cb_journal_txh_check(generic_entity_t *entity, 
+				    blk_t blk, void *data) 
 {
 	journal40_t *journal = (journal40_t *)entity;
 	journal40_check_t *check_data = (journal40_check_t *)data;
@@ -184,7 +180,7 @@ static errno_t callback_journal_txh_check(generic_entity_t *entity, blk_t blk,
 	check_data->flags = 1 << TF_DATA_AREA_ONLY;
 	
 	if (journal40_blk_format_check(journal, blk, check_data)) {
-		aal_error("Transaction header lies in the illegal block "
+		fsck_mess("Transaction header lies in the illegal block "
 			  "(%llu) for the used format (%s).", blk, 
 			  journal->format->plug->label);
 		return -ESTRUCT;
@@ -192,7 +188,7 @@ static errno_t callback_journal_txh_check(generic_entity_t *entity, blk_t blk,
 	
 	if (aux_bitmap_test(check_data->journal_layout, blk)) {
 		/* TxH block is met not for the 1 time. Kill the journal. */
-		aal_error("Transaction header in the block (%llu) was "
+		fsck_mess("Transaction header in the block (%llu) was "
 			  "met already.", blk);
 		return -ESTRUCT;
 	}
@@ -204,11 +200,9 @@ static errno_t callback_journal_txh_check(generic_entity_t *entity, blk_t blk,
 
 /* Secondary blocks callback for traverse. Does all the work described above for 
    all block types except TxH. */
-static errno_t callback_journal_sec_check(generic_entity_t *entity, 
-					  aal_block_t *txh_block, 
-					  blk_t blk, 
-					  journal40_block_t blk_type, 
-					  void *data) 
+static errno_t cb_journal_sec_check(generic_entity_t *entity, 
+				    aal_block_t *txh_block, blk_t blk, 
+				    journal40_block_t blk_type, void *data)
 {
 	journal40_t *journal = (journal40_t *)entity;
 	journal40_check_t *check_data = (journal40_check_t *)data;
@@ -231,7 +225,7 @@ static errno_t callback_journal_sec_check(generic_entity_t *entity,
 	check_data->flags = blk_type == JB_ORG ? 0 : 1 << TF_DATA_AREA_ONLY;
 	
 	if (journal40_blk_format_check(journal, blk, check_data)) {
-		aal_error("%s lies in the illegal block (%llu) for the "
+		fsck_mess("%s lies in the illegal block (%llu) for the "
 			  "used format (%s).", __blk_type_name(blk_type), 
 			  blk, journal->format->plug->label);
 		return -ESTRUCT;
@@ -254,7 +248,7 @@ static errno_t callback_journal_sec_check(generic_entity_t *entity,
 		lr_header = (journal40_lr_header_t *)log_block->data;
 		
 		if (aal_memcmp(lr_header->magic, LGR_MAGIC, LGR_MAGIC_SIZE)) {
-			aal_error("Transaction Header (%llu), Log record "
+			fsck_mess("Transaction Header (%llu), Log record "
 				  "(%llu): Log Record Magic was not found.", 
 				  check_data->cur_txh, blk);
 			aal_block_free(log_block);
@@ -267,7 +261,7 @@ static errno_t callback_journal_sec_check(generic_entity_t *entity,
 	if (aux_bitmap_test(check_data->journal_layout, blk)) {
 		/* blk was met in the current trans more then once. */
 		if (aux_bitmap_test(check_data->current_layout, blk)) {
-			aal_error("Transaction Header (%llu): %s block "
+			fsck_mess("Transaction Header (%llu): %s block "
 				  "(%llu) was met in the transaction "
 				  "more then once.", check_data->cur_txh, 
 				  __blk_type_name(blk_type), blk);
@@ -295,20 +289,20 @@ static errno_t callback_journal_sec_check(generic_entity_t *entity,
 				check_data->wanted_blk = blk;
 				
 				ret = journal40_traverse(journal, NULL, NULL, 
-							 callback_find_sec_blk, 
+							 cb_find_sec_blk, 
 							 check_data);
 				
 				if (ret != -ESTRUCT) {
-					aal_bug("vpf-3063", "Traverse failed to "
-						"find a transaction the "
-						"block (%llu) was met for "
-						"the first time.", blk);
+					aal_error("Traverse failed to find "
+						  "a transaction the block "
+						  "(%llu) was met for the "
+						  "first time.", blk);
 					return ret;
 				}
 				
 				/* Found trans is the oldest problem, return it 
 				   to caller. */				
-				aal_error("Transaction Header (%llu): "
+				fsck_mess("Transaction Header (%llu): "
 					  "transaction looks correct but "
 					  "uses the block (%llu) already "
 					  "used in the transaction (%llu).", 
@@ -331,18 +325,18 @@ static errno_t callback_journal_sec_check(generic_entity_t *entity,
 			check_data->wanted_blk = blk;
 			check_data->flags = 0;
 			
-			ret = journal40_traverse(journal, callback_find_txh_blk, 
-						 NULL, callback_find_sec_blk, 
+			ret = journal40_traverse(journal, cb_find_txh_blk, 
+						 NULL, cb_find_sec_blk, 
 						 check_data);
 			
 			if (ret != -ESTRUCT) {
-				aal_bug("vpf-3066", "Traverse failed to find a "
-					"transaction the block (%llu) was "
-					"met for the first time.", blk);
+				aal_error("Traverse failed to find a "
+					  "transaction the block (%llu) was "
+					  "met for the first time.", blk);
 				return ret;
 			}
 			
-			aal_error("Transaction Header (%llu): transaction "
+			fsck_mess("Transaction Header (%llu): transaction "
 				  "looks correct but uses the block (%llu) "
 				  "already used in the transaction (%llu) "
 				  "as a %s block.", check_data->cur_txh,
@@ -366,20 +360,20 @@ static errno_t callback_journal_sec_check(generic_entity_t *entity,
 			/* Stop looking through TxH's when reach the current trans. */
 			check_data->flags = (1 << TF_SAME_TXH_BREAK);
 			
-			ret = journal40_traverse(journal, callback_find_txh_blk, 
+			ret = journal40_traverse(journal, cb_find_txh_blk, 
 						 NULL, NULL, check_data);
 			
 			if (ret != -ESTRUCT) {
-				aal_bug("vpf-3067", "Traverse failed to find a "
-					"transaction the block (%llu) was "
-					"met for the first time.", blk);
+				aal_error("Traverse failed to find a "
+					  "transaction the block (%llu) was "
+					  "met for the first time.", blk);
 				return ret;
 			}
 			
 			/* If TxH was found, the current transaction is the oldest 
 			   problem trans. */
 			if (check_data->found_type != JB_INV) {
-				aal_error("Transaction Header (%llu): "
+				fsck_mess("Transaction Header (%llu): "
 					  "original location (%llu) was "
 					  "met before as a Transaction "
 					  "Header of one of the next "
@@ -430,8 +424,8 @@ errno_t journal40_check_struct(generic_entity_t *entity,
 		return -ENOMEM;
 	}
 	
-	ret = journal40_traverse((journal40_t *)entity, callback_journal_txh_check,
-				 NULL, callback_journal_sec_check, &data);
+	ret = journal40_traverse((journal40_t *)entity, cb_journal_txh_check,
+				 NULL, cb_journal_sec_check, &data);
 	
 	if (ret && ret != -ESTRUCT)
 		return ret;
@@ -439,7 +433,7 @@ errno_t journal40_check_struct(generic_entity_t *entity,
 	if (ret) {
 		/* Journal should be updated */
 		if (!data.cur_txh) {
-			aal_error("Journal has broken list of transaction "
+			fsck_mess("Journal has broken list of transaction "
 				  "headers. Reinitialize the journal.");
 			
 			data.cur_txh = 
@@ -466,7 +460,7 @@ errno_t journal40_check_struct(generic_entity_t *entity,
 				return -EIO;
 			}
 			
-			aal_error("Corrupted transaction (%llu) was found. The last "
+			fsck_mess("Corrupted transaction (%llu) was found. The last "
 				  "valid transaction is (%llu).", data.cur_txh,
 				  get_th_prev_tx((journal40_tx_header_t *)tx_block->data));
 			
@@ -514,8 +508,8 @@ static void extract_string(char *stor, char *orig, uint32_t max) {
 }
 
 /* Helper function for printing transaction header. */
-static errno_t callback_print_txh(generic_entity_t *entity,
-				  blk_t blk, void *data)
+static errno_t cb_print_txh(generic_entity_t *entity,
+			    blk_t blk, void *data)
 {
 	aal_block_t *block;
 	journal40_t *journal;
@@ -573,20 +567,19 @@ static errno_t callback_print_txh(generic_entity_t *entity,
 }
 
 /* Printing pair (wandered and original) blocks */
-static errno_t callback_print_par(generic_entity_t *entity,
-				  aal_block_t *block,
-				  blk_t orig, void *data)
+static errno_t cb_print_par(generic_entity_t *entity,
+			    aal_block_t *block,
+			    blk_t orig, void *data)
 {
 	aal_stream_format((aal_stream_t *)data,
-			  "%llu -> %llu\n",
-			  orig, block->nr);
+			  "%llu -> %llu\n", orig, block->nr);
 	return 0;
 }
 
 /* hellper function for printing log record. */
-static errno_t callback_print_lgr(generic_entity_t *entity,
-				  aal_block_t *block, blk_t blk,
-				  journal40_block_t bel, void *data)
+static errno_t cb_print_lgr(generic_entity_t *entity,
+			    aal_block_t *block, blk_t blk,
+			    journal40_block_t bel, void *data)
 {
 	aal_stream_t *stream;
 	char magic[LGR_MAGIC_SIZE];
@@ -647,12 +640,14 @@ void journal40_print(generic_entity_t *entity,
 	aal_stream_format(stream, "description:\t%s\n\n",
 			  entity->plug->desc);
 	
-	aal_stream_format(stream, "Journal header:\n");
+	aal_stream_format(stream, "Journal header block (%llu):\n", 
+			  journal->header->nr);
 
 	aal_stream_format(stream, "last commited:\t%llu\n\n",
 			  get_jh_last_commited(header));
 		
-	aal_stream_format(stream, "Journal footer:\n");
+	aal_stream_format(stream, "Journal footer block (%llu):\n",
+			  journal->footer->nr);
 
 	aal_stream_format(stream, "last flushed:\t%llu\n",
 			  get_jf_last_flushed(footer));
@@ -667,11 +662,8 @@ void journal40_print(generic_entity_t *entity,
 			  get_jf_used_oids(footer));
 
 	/* Print all transactions. */
-	journal40_traverse((journal40_t *)entity, 
-			   callback_print_txh,
-			   callback_print_par,
-			   callback_print_lgr,
-			   (void *)stream);
+	journal40_traverse((journal40_t *)entity, cb_print_txh,
+			   cb_print_par, cb_print_lgr, (void *)stream);
 }
 #endif
 
