@@ -12,7 +12,6 @@
 
 reiser4_core_t *dir40_core = NULL;
 
-#ifndef ENABLE_STAND_ALONE
 /* Return current position in directory into passed @offset. */
 static errno_t dir40_telldir(object_entity_t *entity,
 			     reiser4_key_t *position)
@@ -26,13 +25,14 @@ static errno_t dir40_telldir(object_entity_t *entity,
 	plug_call(dir->position.plug->o.key_ops,
 		  assign, position, &dir->position);
 
+#ifndef ENABLE_STAND_ALONE
 	/* Adjust is offset inside collided keys arrays and needed for
 	   positioning right in such a case. In normal case it is zero. */
 	position->adjust = dir->position.adjust;
+#endif
 	
 	return 0;
 }
-#endif
 
 /* Close directiry instance. */
 static void dir40_close(object_entity_t *entity) {
@@ -81,8 +81,10 @@ errno_t dir40_reset(object_entity_t *entity) {
 
 	/* Building key itself. */
 	plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops, 
-		  build_hashed, &dir->position, dir->hash, 
-		  dir->fibre, obj40_locality(&dir->obj), 
+		  build_hashed, &dir->position, 
+		  entity->opset.plug[OPSET_HASH],
+		  entity->opset.plug[OPSET_FIBRE], 
+		  obj40_locality(&dir->obj), 
 		  obj40_objectid(&dir->obj), ".");
 
 	return 0;
@@ -193,9 +195,8 @@ lookup_t dir40_update_body(object_entity_t *entity, int check_group) {
 #endif
 	
 	/* Making lookup by current dir key. */
-	if ((res = obj40_find_item(&dir->obj, &dir->position, 
-				   FIND_EXACT, NULL, NULL,
-				   &dir->body)) < 0)
+	if ((res = obj40_find_item(&dir->obj, &dir->position, FIND_EXACT, 
+				   NULL, NULL, &dir->body)) < 0)
 		return res;
 
 	if (res == ABSENT) {
@@ -206,9 +207,9 @@ lookup_t dir40_update_body(object_entity_t *entity, int check_group) {
 			return ABSENT;
 		
 		/* If key matches, but this is not directory item. */
-		if (check_group && dir->body.plug->id.group != DIRENTRY_ITEM)
+		if (check_group && dir->body.plug->id.group != DIR_ITEM)
 			return ABSENT;
-		
+
 #ifndef ENABLE_STAND_ALONE
 		/* No adjusting for the ABSENT result. */
 		adjust = 0;
@@ -345,8 +346,10 @@ static lookup_t dir40_search(object_entity_t *entity, char *name,
 {
 	dir40_t *dir;
 	lookup_t res;
+#ifndef ENABLE_STAND_ALONE
 	coll_hint_t hint;
 	coll_func_t func;
+#endif
 
 	aal_assert("umka-1118", name != NULL);
 	aal_assert("umka-1117", entity != NULL);
@@ -356,18 +359,25 @@ static lookup_t dir40_search(object_entity_t *entity, char *name,
 	/* Preparing key to be used for lookup. It is generating from the
 	   directory oid, locality and name by menas of using hash plugin. */
 	plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops, 
-		  build_hashed, &dir->body.key, dir->hash, 
-		  dir->fibre, obj40_locality(&dir->obj), 
+		  build_hashed, &dir->body.key, 
+		  entity->opset.plug[OPSET_HASH],
+		  entity->opset.plug[OPSET_FIBRE], 
+		  obj40_locality(&dir->obj),
 		  obj40_objectid(&dir->obj), name);
 
 #ifndef ENABLE_STAND_ALONE
 	hint.specific = name;
-	hint.type = DIRENTRY_ITEM;
+	hint.type = DIR_ITEM;
 	func = dir40_core->tree_ops.collision;
 #endif
 	
 	if ((res = obj40_find_item(&dir->obj, &dir->body.key, bias, 
-				   func, &hint, &dir->body)) < 0)
+#ifndef ENABLE_STAND_ALONE
+				   func, &hint,
+#else
+				   NULL, NULL,
+#endif
+				   &dir->body)) < 0)
 	{
 		return res;
 	}
@@ -406,43 +416,26 @@ static object_entity_t *dir40_open(object_info_t *info) {
 	aal_assert("umka-836", info != NULL);
 	aal_assert("umka-837", info->tree != NULL);
 	
-	if (info->start.plug->id.group != STATDATA_ITEM)
+	if (info->start.plug->id.group != STAT_ITEM)
+		return NULL;
+	
+	if (info->opset.plug[OPSET_OBJ] != &dir40_plug)
 		return NULL;
 	
 	if (!(dir = aal_calloc(sizeof(*dir), 0)))
 		return NULL;
 
 	/* Initializing obj handle for the directory */
-	obj40_init(&dir->obj, &dir40_plug, dir40_core, info);
+	obj40_init(&dir->obj, info, dir40_core);
 	
-	if (obj40_pid(&dir->obj, OBJECT_PLUG_TYPE,
-		      "directory") !=  dir40_plug.id.id)
-	{
-		goto error_free_dir;
-	}
-	
-	/* Getting hash plugin basing on stat data and/or param set. */
-	if (!(dir->hash = obj40_plug(&dir->obj, HASH_PLUG_TYPE, "hash")))
-                goto error_free_dir;
-	
-	/* Getting fibre plugin basing on stat data and/or param set. */
-	if (!(dir->fibre = obj40_plug(&dir->obj, FIBRE_PLUG_TYPE, "fibre")))
-                goto error_free_dir;
-
 	/* Positioning to the first directory unit. */
 	dir40_reset((object_entity_t *)dir);
 	
 	return (object_entity_t *)dir;
-
- error_free_dir:
-	aal_free(dir);
-	return NULL;
 }
 
 /* Loads stat data to passed @hint. */
-static errno_t dir40_stat(object_entity_t *entity,
-			  statdata_hint_t *hint)
-{
+static errno_t dir40_stat(object_entity_t *entity, stat_hint_t *hint) {
 	dir40_t *dir;
 	
 	aal_assert("umka-2563", entity != NULL);
@@ -471,64 +464,35 @@ static uint64_t dir40_size(object_entity_t *entity) {
 /* Creates dir40 instance. Creates its stat data item, and body item with one
    "." unit. Yet another unit ".." will be inserted latter, then directiry will
    be attached to a parent object. */
-static object_entity_t *dir40_create(object_info_t *info,
-				     object_hint_t *hint)
-{
-	rid_t pid;
-	dir40_t *dir;
-	uint64_t mask;
-	entry_hint_t entry;
+static object_entity_t *dir40_create(object_hint_t *hint) {
 	trans_hint_t body_hint;
-	reiser4_plug_t *body_plug;
+	entry_hint_t entry;
+	reiser4_key_t *key;
+	dir40_t *dir;
     
-	aal_assert("umka-835", info != NULL);
 	aal_assert("umka-1739", hint != NULL);
-	aal_assert("vpf-1095", info->tree != NULL);
+	aal_assert("vpf-1095", hint->info.tree != NULL);
 
 	if (!(dir = aal_calloc(sizeof(*dir), 0)))
 		return NULL;
 	
 	/* Initializing obj handle. */
-	obj40_init(&dir->obj, &dir40_plug, dir40_core, info);
-
-	/* Getting hash plugin */
-	if (!(dir->hash = dir40_core->factory_ops.ifind(HASH_PLUG_TYPE, 
-							hint->body.dir.hash))) 
-	{
-		aal_error("Can't find hash plugin by its "
-			  "id 0x%x.", hint->body.dir.hash);
-		goto error_free_dir;
-	}
-
-	/* Getting fibration plugin */
-	if (!(dir->fibre = dir40_core->factory_ops.ifind(FIBRE_PLUG_TYPE, 
-							 hint->body.dir.fibre))) 
-	{
-		aal_error("Can't find fibration plugin by its "
-			  "id 0x%x.", hint->body.dir.fibre);
-		goto error_free_dir;
-	}
-
-	/* Initializing body plugin. */
-	pid = hint->body.dir.direntry;
+	obj40_init(&dir->obj, &hint->info, dir40_core);
 	
-	if (!(body_plug = dir40_core->factory_ops.ifind(ITEM_PLUG_TYPE, pid))) {
-		aal_error("Can't find direntry item plugin by "
-			  "its id 0x%x.", pid);
-		goto error_free_dir;
-	}
-    
 	aal_memset(&body_hint, 0, sizeof(body_hint));
 	
 	/* Initializing direntry item hint. This should be done before the stat
 	   data item hint, because we will need size of direntry item during
 	   stat data initialization. */
    	body_hint.count = 1;
-	body_hint.plug = body_plug;
+	body_hint.plug = hint->info.opset.plug[OPSET_DIRITEM];
+	key = &hint->info.object;
 	
-	plug_call(info->object.plug->o.key_ops, 
-		  build_hashed, &body_hint.offset, dir->hash, 
-		  dir->fibre, obj40_locality(&dir->obj), 
+	plug_call(key->plug->o.key_ops, 
+		  build_hashed, &body_hint.offset, 
+		  dir->obj.info.opset.plug[OPSET_HASH], 
+		  dir->obj.info.opset.plug[OPSET_FIBRE],
+		  obj40_locality(&dir->obj), 
 		  obj40_objectid(&dir->obj), ".");
 
 	/* Preparing hint for the empty directory. It consists only "." for
@@ -536,16 +500,13 @@ static object_entity_t *dir40_create(object_info_t *info,
 	aal_strncpy(entry.name, ".", 1);
 
 	/* Initializing entry stat data key. */
-	plug_call(info->object.plug->o.key_ops, assign,
-		  &entry.object, &info->object);
+	plug_call(key->plug->o.key_ops, assign, &entry.object, key);
 
 	/* Initializing entry hash key. */
-	plug_call(info->object.plug->o.key_ops, assign,
+	plug_call(key->plug->o.key_ops, assign,
 		  &entry.offset, &body_hint.offset);
 
 	body_hint.specific = &entry;
-	body_hint.place_func = NULL;
-	body_hint.region_func = NULL;
 	body_hint.shift_flags = SF_DEFAULT;
 	
 	dir40_reset((object_entity_t *)dir);
@@ -568,10 +529,8 @@ static object_entity_t *dir40_create(object_info_t *info,
 	}
 
 	/* Create stat data item. */
-	mask = (1 << SDEXT_UNIX_ID | 1 << SDEXT_LW_ID);
-	
-	if (obj40_create_stat(&dir->obj, hint->label.statdata, mask,
-			      1, body_hint.len, 0, 1, S_IFDIR, NULL))
+	if (obj40_create_stat(&dir->obj, 1, body_hint.len, 
+			      0, 1, S_IFDIR, NULL))
 	{
 	
 		/* Removing body item. */	
@@ -594,9 +553,7 @@ static object_entity_t *dir40_create(object_info_t *info,
 }
 
 /* Removes all directory body items. */
-static errno_t dir40_truncate(object_entity_t *entity,
-			      uint64_t n)
-{
+static errno_t dir40_truncate(object_entity_t *entity, uint64_t n) {
 	errno_t res;
 	dir40_t *dir;
 	reiser4_key_t key;
@@ -615,8 +572,8 @@ static errno_t dir40_truncate(object_entity_t *entity,
 
 	/* Creating maximal possible key in order to find last directory item
 	   and remove it from the tree. Thanks to Nikita for this idea. */
-	plug_call(dir->body.plug->o.key_ops,
-		  set_offset, &key, MAX_UINT64);
+	plug_call(dir->body.key.plug->o.key_ops, assign, &key, &dir->body.key);
+	plug_call(dir->body.key.plug->o.key_ops, set_offset, &key, MAX_UINT64);
 
 	while (1) {
 		trans_hint_t hint;
@@ -637,6 +594,7 @@ static errno_t dir40_truncate(object_entity_t *entity,
 		hint.place_func = NULL;
 		hint.region_func = NULL;
 		hint.shift_flags = SF_DEFAULT;
+		place.pos.unit = MAX_UINT32;
 		
 		/* Removing item from the tree */
 		if ((res = obj40_remove(&dir->obj, &place, &hint)))
@@ -679,44 +637,43 @@ static errno_t dir40_attach(object_entity_t *entity,
 
 	/* Adding ".." pointing to parent to @entity object. */
 	plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops,
-		  assign, &entry.object, &parent->info.object);
+		  assign, &entry.object, &parent->object);
 
-	if ((res = plug_call(entity->plug->o.object_ops,
+	if ((res = plug_call(entity->opset.plug[OPSET_OBJ]->o.object_ops,
 			     add_entry, entity, &entry)))
 	{
 		return res;
 	}
 
 	/* Increasing parent's @nlink by one */
-	return plug_call(parent->plug->o.object_ops, link, parent);
+	return plug_call(parent->opset.plug[OPSET_OBJ]->o.object_ops, 
+			 link, parent);
 }
 
 /* Detaches @entity from @parent. */
 static errno_t dir40_detach(object_entity_t *entity,
 			    object_entity_t *parent)
 {
-	dir40_t *dir;
+	reiser4_plug_t *plug;
 	entry_hint_t entry;
+	dir40_t *dir;
 
 	aal_assert("umka-2291", entity != NULL);
 
 	dir = (dir40_t *)entity;
 
+	plug = entity->opset.plug[OPSET_OBJ];
+	
 	/* Removing ".." from child if it is found */
-	switch (plug_call(entity->plug->o.object_ops,
-			  lookup, entity, "..", &entry))
-	{
+	switch (plug_call(plug->o.object_ops, lookup, entity, "..", &entry)) {
 	case PRESENT:
-		plug_call(entity->plug->o.object_ops,
-			  rem_entry, entity, &entry);
+		return plug_call(plug->o.object_ops, rem_entry, entity, &entry);
 	default:
-		if (parent) {
-			/* Decreasing parent's @nlink by one */
-			return plug_call(parent->plug->o.object_ops,
-					 unlink, parent);
-		}
-
-		return 0;
+		if (!parent) 
+			return 0;
+		
+		/* Decreasing parent's @nlink by one */
+		return plug_call(plug->o.object_ops, unlink, parent);
 	}
 }
 
@@ -750,7 +707,7 @@ static errno_t dir40_unlink(object_entity_t *entity) {
 	return obj40_unlink(&dir->obj);
 }
 
-/* Helper function. Builds entry key by entry name. */
+/* Helper function. Builds @entry->offset key by @entry->name. */
 static errno_t dir40_build_entry(object_entity_t *entity, 
 				 entry_hint_t *entry)
 {
@@ -767,8 +724,10 @@ static errno_t dir40_build_entry(object_entity_t *entity,
 	objectid = obj40_objectid(&dir->obj);
 	
 	return plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops, 
-			 build_hashed, &entry->offset, dir->hash, 
-			 dir->fibre, locality, objectid, entry->name);
+			 build_hashed, &entry->offset, 
+			 entity->opset.plug[OPSET_HASH],
+			 entity->opset.plug[OPSET_FIBRE], 
+			 locality, objectid, entry->name);
 }
 
 /* Add new entry to directory. */
@@ -794,6 +753,9 @@ static errno_t dir40_add_entry(object_entity_t *entity,
 			     FIND_EXACT, &temp))
 	{
 	case ABSENT:
+		if (!obj40_valid_item(&temp.place))
+			return -EINVAL;
+		
 		if ((res = obj40_fetch_item(&temp.place)))
 			return res;
 		
@@ -902,8 +864,8 @@ struct layout_hint {
 
 typedef struct layout_hint layout_hint_t;
 
-static errno_t callback_item_layout(void *place, blk_t start,
-				    count_t width, void *data)
+static errno_t cb_item_layout(void *place, blk_t start,
+			      count_t width, void *data)
 {
 	layout_hint_t *hint = (layout_hint_t *)data;
 	return hint->region_func(hint->entity, start,
@@ -945,7 +907,7 @@ static errno_t dir40_layout(object_entity_t *entity,
 		if (dir->body.plug->o.item_ops->object->layout) {
 			/* Calling item's layout method */
 			if ((res = plug_call(place->plug->o.item_ops->object,
-					     layout, place, callback_item_layout,
+					     layout, place, cb_item_layout,
 					     &hint)))
 			{
 				return res;
@@ -953,13 +915,10 @@ static errno_t dir40_layout(object_entity_t *entity,
 		} else {
 			/* Layout method is not implemented. Counting item
 			   itself. */
-			blk_t blk = place->node->block->nr;
+			blk_t blk = place_blknr(place);
 			
-			if ((res = callback_item_layout(place, blk,
-							1, &hint)))
-			{
+			if ((res = cb_item_layout(place, blk, 1, &hint)))
 				return res;
-			}
 		}
 
 		/* Getting next directory item. */
@@ -1018,7 +977,7 @@ static errno_t dir40_metadata(object_entity_t *entity,
 
 /* Updates stat data from passed @hint */
 static errno_t dir40_update(object_entity_t *entity,
-			    statdata_hint_t *hint)
+			    stat_hint_t *hint)
 {
 	dir40_t *dir;
 	
@@ -1048,7 +1007,6 @@ static reiser4_object_ops_t dir40_ops = {
 	.detach		= dir40_detach,
 	.clobber	= dir40_clobber,
 	.recognize	= dir40_recognize,
-	.form		= dir40_form,
 	.fake		= dir40_fake,
 	.check_struct	= dir40_check_struct,
 	.check_attach	= dir40_check_attach,
@@ -1068,12 +1026,7 @@ static reiser4_object_ops_t dir40_ops = {
 	.lookup		= dir40_lookup,
 	.seekdir	= dir40_seekdir,
 	.readdir	= dir40_readdir,
-
-#ifndef ENABLE_STAND_ALONE
 	.telldir	= dir40_telldir,
-#else
-	.telldir	= NULL
-#endif
 };
 
 reiser4_plug_t dir40_plug = {
