@@ -7,13 +7,10 @@
 #define DIR40_H
 
 #include <aal/libaal.h>
-#include <sys/stat.h>
-#include <reiser4/plugin.h>
-#include <plugin/object/obj40/obj40.h>
+#include "plugin/object/obj40/obj40.h"
 
 /* Compaund directory structure */
 struct dir40 {
-	
 	/* Common fields for all files (statdata, etc) */
 	obj40_t obj;
 
@@ -22,17 +19,13 @@ struct dir40 {
 
 	/* Current directory offset. */
 	reiser4_key_t position;
-
-	/* Hash plugin in use */
-	reiser4_plug_t *hash;
-	
-	/* Fibration plugin in use */
-	reiser4_plug_t *fibre;
 };
 
 typedef struct dir40 dir40_t;
 
 extern reiser4_plug_t dir40_plug;
+
+#ifndef ENABLE_STAND_ALONE
 extern reiser4_core_t *dir40_core;
 
 extern lookup_t dir40_next(dir40_t *dir, int first);
@@ -44,5 +37,5 @@ extern lookup_t dir40_lookup(object_entity_t *entity,
 extern errno_t dir40_fetch(dir40_t *dir, entry_hint_t *entry);
 
 extern lookup_t dir40_update_body(object_entity_t *entity, int check_group);
-
+#endif
 #endif
