@@ -3,13 +3,6 @@
    
    obj40.c -- reiser4 file 40 plugins common code. */
 
-#include <sys/stat.h>
-
-#ifndef ENABLE_STAND_ALONE
-#  include <time.h>
-#  include <unistd.h>
-#endif
-
 #include "obj40.h"
 
 /* Returns file's oid */
@@ -104,11 +97,9 @@ lookup_t obj40_find_item(obj40_t *obj, reiser4_key_t *key,
 }
 
 /* Reads one stat data extension to @data. */
-errno_t obj40_read_ext(reiser4_place_t *place, rid_t id,
-		       void *data)
-{
+errno_t obj40_read_ext(reiser4_place_t *place, rid_t id, void *data) {
 	trans_hint_t trans;
-	statdata_hint_t stat;
+	stat_hint_t stat;
 
 	aal_memset(&stat, 0, sizeof(stat));
 
@@ -134,16 +125,16 @@ errno_t obj40_read_ext(reiser4_place_t *place, rid_t id,
 
 /* Gets size field from the stat data */
 uint64_t obj40_get_size(obj40_t *obj) {
-	sdext_lw_hint_t lw_hint;
+	sdhint_lw_t lwh;
 
-	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lw_hint))
+	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh))
 		return 0;
 	
-	return lw_hint.size;
+	return lwh.size;
 }
 
 /* Loads stat data to passed @hint. */
-errno_t obj40_load_stat(obj40_t *obj, statdata_hint_t *hint) {
+errno_t obj40_load_stat(obj40_t *obj, stat_hint_t *hint) {
 	trans_hint_t trans;
 	
 	aal_assert("umka-2553", obj != NULL);
@@ -166,7 +157,7 @@ errno_t obj40_load_stat(obj40_t *obj, statdata_hint_t *hint) {
 
 #ifndef ENABLE_STAND_ALONE
 /* Saves stat data to passed @hint. */
-errno_t obj40_save_stat(obj40_t *obj, statdata_hint_t *hint) {
+errno_t obj40_save_stat(obj40_t *obj, stat_hint_t *hint) {
 	trans_hint_t trans;
 	
 	aal_assert("umka-2554", obj != NULL);
@@ -189,66 +180,87 @@ errno_t obj40_save_stat(obj40_t *obj, statdata_hint_t *hint) {
 
 /* Create stat data item basing on passed extensions @mask, @size, @bytes,
    @nlinks, @mode and @path for symlinks. Returns error or zero for success. */
-errno_t obj40_create_stat(obj40_t *obj, rid_t pid, uint64_t mask,
-			  uint64_t size, uint64_t bytes, uint64_t rdev,
-			  uint32_t nlink, uint16_t mode, char *path)
+errno_t obj40_create_stat(obj40_t *obj, uint64_t size, uint64_t bytes, 
+			  uint64_t rdev, uint32_t nlink, uint16_t mode, 
+			  char *path)
 {
-	int64_t res;
-	lookup_t lookup;
+	sdhint_unix_t unixh;
+	sdhint_plug_t plugh;
+	sdhint_lw_t lwh;
+	stat_hint_t stat;
 	trans_hint_t hint;
-	statdata_hint_t stat;
-	sdext_lw_hint_t lw_ext;
-	sdext_unix_hint_t unix_ext;
+	lookup_t lookup;
+	int64_t res;
+	
+	aal_assert("vpf-1592", obj != NULL);
+	aal_assert("vpf-1593", obj->info.opset.plug[OPSET_STAT] != NULL);
 	
 	aal_memset(&hint, 0, sizeof(hint));
 	
 	/* Getting statdata plugin */
-	if (!(hint.plug = obj->core->factory_ops.ifind(ITEM_PLUG_TYPE, pid))) {
-		aal_error("Can't find stat data item plugin by "
-			  "its id 0x%x.", pid);
-		return -EIO;
-	}
+	hint.plug = obj->info.opset.plug[OPSET_STAT];
 
 	hint.count = 1;
-	hint.place_func = NULL;
-	hint.region_func = NULL;
 	hint.shift_flags = SF_DEFAULT;
 	
 	plug_call(obj->info.object.plug->o.key_ops, assign, 
 		  &hint.offset, &obj->info.object);
     
 	/* Initializing stat data item hint. */
-	stat.extmask = mask;
+	stat.extmask = (1 << SDEXT_UNIX_ID | 1 << SDEXT_LW_ID);
     	
 	/* Light weight hint initializing. */
-	lw_ext.size = size;
-	lw_ext.nlink = nlink;
-	lw_ext.mode = mode | 0755;
+	lwh.size = size;
+	lwh.nlink = nlink;
+	lwh.mode = mode | 0755;
 	
 	/* Unix extension hint initializing */
 	if (rdev && bytes) {
 		aal_error("Invalid stat data params (rdev or bytes).");
 		return -EINVAL;
 	} else {
-		unix_ext.rdev = rdev;
-		unix_ext.bytes = bytes;
+		unixh.rdev = rdev;
+		unixh.bytes = bytes;
 	}
 	
-	unix_ext.uid = getuid();
-	unix_ext.gid = getgid();
+	unixh.uid = getuid();
+	unixh.gid = getgid();
 	
-	unix_ext.atime = time(NULL);
-	unix_ext.mtime = unix_ext.atime;
-	unix_ext.ctime = unix_ext.atime;
+	unixh.atime = time(NULL);
+	unixh.mtime = unixh.atime;
+	unixh.ctime = unixh.atime;
 
+	aal_memcpy(&plugh, &obj->info, sizeof(plugh));
+#if 0
+	for (i = 0, plcount = 0; i < OPSET_LAST; i++) {
+		plugs.pset[i] = (obj->info.opmask & (1 << i)) ? 
+			obj->info.opset.plug[i] : NULL;
+
+		/* Smth to be stored. */
+		if (plugs.pset[i])
+			plcount++;
+	}
+#endif
+	
 	aal_memset(&stat.ext, 0, sizeof(stat.ext));
 
 	/* Initializing extensions array */
-	stat.ext[SDEXT_LW_ID] = &lw_ext;
-	stat.ext[SDEXT_UNIX_ID] = &unix_ext;
+	stat.ext[SDEXT_LW_ID] = &lwh;
+	stat.ext[SDEXT_UNIX_ID] = &unixh;
 
-	if ((1 << SDEXT_SYMLINK_ID) & mask)
+	/* Get plugins that must exists in the PLUGID extention. */
+	obj->core->pset_ops.diff(obj->info.tree, &plugh);
+	
+	if (plugh.mask) {
+		stat.extmask |= (1 << SDEXT_PLUG_ID);
+		stat.ext[SDEXT_PLUG_ID] = &plugh;
+		obj->info.opset.mask = plugh.mask;
+	}
+
+	if (path) {
+		stat.extmask |= (1 << SDEXT_SYMLINK_ID);
 		stat.ext[SDEXT_SYMLINK_ID] = path;
+	}
 	
 	hint.specific = &stat;
 
@@ -270,8 +282,8 @@ errno_t obj40_create_stat(obj40_t *obj, rid_t pid, uint64_t mask,
 
 /* Updates size and bytes fielsds */
 errno_t obj40_touch(obj40_t *obj, uint64_t size, uint64_t bytes) {
+	sdhint_unix_t unixh;
 	errno_t res;
-	sdext_unix_hint_t unix_hint;
 
 	/* Updating stat data place */
 	if ((res = obj40_update(obj)))
@@ -286,22 +298,20 @@ errno_t obj40_touch(obj40_t *obj, uint64_t size, uint64_t bytes) {
 	}
 
 	/* Updating bytes */
-	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unix_hint)))
+	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh)))
 		return res;
 
 	/* Updating values and write unix extension back. */
-	unix_hint.rdev = 0;
-	unix_hint.bytes = bytes;
+	unixh.rdev = 0;
+	unixh.bytes = bytes;
 
-	return obj40_write_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unix_hint);
+	return obj40_write_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh);
 }
 
 /* Writes one stat data extension. */
-errno_t obj40_write_ext(reiser4_place_t *place, rid_t id,
-			void *data)
-{
+errno_t obj40_write_ext(reiser4_place_t *place, rid_t id, void *data) {
 	trans_hint_t hint;
-	statdata_hint_t stat;
+	stat_hint_t stat;
 
 	aal_memset(&stat, 0, sizeof(stat));
 
@@ -330,7 +340,7 @@ errno_t obj40_write_ext(reiser4_place_t *place, rid_t id,
 /* Returns extensions mask from stat data item at @place. */
 uint64_t obj40_extmask(reiser4_place_t *place) {
 	trans_hint_t hint;
-	statdata_hint_t stat;
+	stat_hint_t stat;
 
 	aal_memset(&stat, 0, sizeof(stat));
 
@@ -352,161 +362,131 @@ uint64_t obj40_extmask(reiser4_place_t *place) {
 
 /* Gets mode field from the stat data */
 uint16_t obj40_get_mode(obj40_t *obj) {
-	sdext_lw_hint_t lw_hint;
+	sdhint_lw_t lwh;
 
-	if (obj40_read_ext(STAT_PLACE(obj),
-			   SDEXT_LW_ID, &lw_hint))
-	{
+	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh))
 		return 0;
-	}
 	
-	return lw_hint.mode;
+	return lwh.mode;
 }
 
 /* Updates mode field in statdata */
 errno_t obj40_set_mode(obj40_t *obj, uint16_t mode) {
+	sdhint_lw_t lwh;
 	errno_t res;
-	sdext_lw_hint_t lw_hint;
 
-	if ((res = obj40_read_ext(STAT_PLACE(obj),
-				  SDEXT_LW_ID, &lw_hint)))
-	{
+	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh)))
 		return res;
-	}
 
-	lw_hint.mode = mode;
+	lwh.mode = mode;
 	
-	return obj40_write_ext(STAT_PLACE(obj),
-			       SDEXT_LW_ID, &lw_hint);
+	return obj40_write_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh);
 }
 
 /* Updates size field in the stat data */
 errno_t obj40_set_size(obj40_t *obj, uint64_t size) {
+	sdhint_lw_t lwh;
 	errno_t res;
-	sdext_lw_hint_t lw_hint;
 
-	if ((res = obj40_read_ext(STAT_PLACE(obj),
-				  SDEXT_LW_ID, &lw_hint)))
-	{
+	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh)))
 		return res;
-	}
 
-	lw_hint.size = size;
+	lwh.size = size;
 	
-	return obj40_write_ext(STAT_PLACE(obj),
-			       SDEXT_LW_ID, &lw_hint);
+	return obj40_write_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh);
 }
 
 /* Gets nlink field from the stat data */
 uint32_t obj40_get_nlink(obj40_t *obj) {
-	sdext_lw_hint_t lw_hint;
+	sdhint_lw_t lwh;
 
-	if (obj40_read_ext(STAT_PLACE(obj),
-			   SDEXT_LW_ID, &lw_hint))
-	{
+	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh))
 		return 0;
-	}
 	
-	return lw_hint.nlink;
+	return lwh.nlink;
 }
 
 /* Updates nlink field in the stat data */
 errno_t obj40_set_nlink(obj40_t *obj, uint32_t nlink) {
+	sdhint_lw_t lwh;
 	errno_t res;
-	sdext_lw_hint_t lw_hint;
 
-	if ((res = obj40_read_ext(STAT_PLACE(obj),
-				  SDEXT_LW_ID, &lw_hint)))
-	{
+	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh)))
 		return res;
-	}
 
-	lw_hint.nlink = nlink;
+	lwh.nlink = nlink;
 	
-	return obj40_write_ext(STAT_PLACE(obj),
-			       SDEXT_LW_ID, &lw_hint);
+	return obj40_write_ext(STAT_PLACE(obj), SDEXT_LW_ID, &lwh);
 }
 
 /* Gets atime field from the stat data */
 uint32_t obj40_get_atime(obj40_t *obj) {
-	sdext_unix_hint_t unix_hint;
+	sdhint_unix_t unixh;
 
-	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unix_hint))
+	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh))
 		return 0;
 	
-	return unix_hint.atime;
+	return unixh.atime;
 }
 
 /* Updates atime field in the stat data */
 errno_t obj40_set_atime(obj40_t *obj, uint32_t atime) {
+	sdhint_unix_t unixh;
 	errno_t res;
-	sdext_unix_hint_t unix_hint;
 
-	if ((res = obj40_read_ext(STAT_PLACE(obj), 
-				  SDEXT_UNIX_ID, &unix_hint)))
-	{
+	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh)))
 		return res;
-	}
 
-	unix_hint.atime = atime;
+	unixh.atime = atime;
 	
-	return obj40_write_ext(STAT_PLACE(obj),
-			       SDEXT_UNIX_ID, &unix_hint);
+	return obj40_write_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh);
 }
 
 /* Gets mtime field from the stat data */
 uint32_t obj40_get_mtime(obj40_t *obj) {
-	sdext_unix_hint_t unix_hint;
+	sdhint_unix_t unixh;
 
-	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unix_hint))
+	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh))
 		return 0;
 	
-	return unix_hint.mtime;
+	return unixh.mtime;
 }
 
 /* Updates mtime field in the stat data */
 errno_t obj40_set_mtime(obj40_t *obj, uint32_t mtime) {
+	sdhint_unix_t unixh;
 	errno_t res;
-	sdext_unix_hint_t unix_hint;
 
-	if ((res = obj40_read_ext(STAT_PLACE(obj), 
-				  SDEXT_UNIX_ID, &unix_hint)))
-	{
+	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh)))
 		return res;
-	}
 
-	unix_hint.mtime = mtime;
+	unixh.mtime = mtime;
 	
-	return obj40_write_ext(STAT_PLACE(obj),
-			       SDEXT_UNIX_ID, &unix_hint);
+	return obj40_write_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh);
 }
 
 /* Gets bytes field from the stat data */
 uint64_t obj40_get_bytes(obj40_t *obj) {
-	sdext_unix_hint_t unix_hint;
+	sdhint_unix_t unixh;
 
-	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unix_hint))
+	if (obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh))
 		return 0;
 	
-	return unix_hint.bytes;
+	return unixh.bytes;
 }
 
 /* Updates bytes field in the stat data */
 errno_t obj40_set_bytes(obj40_t *obj, uint64_t bytes) {
+	sdhint_unix_t unixh;
 	errno_t res;
-	sdext_unix_hint_t unix_hint;
 
-	if ((res = obj40_read_ext(STAT_PLACE(obj),
-				  SDEXT_UNIX_ID, &unix_hint)))
-	{
+	if ((res = obj40_read_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh)))
 		return res;
-	}
 
-	unix_hint.rdev = 0;
-	unix_hint.bytes = bytes;
+	unixh.rdev = 0;
+	unixh.bytes = bytes;
 	
-	return obj40_write_ext(STAT_PLACE(obj),
-			       SDEXT_UNIX_ID, &unix_hint);
+	return obj40_write_ext(STAT_PLACE(obj), SDEXT_UNIX_ID, &unixh);
 }
 
 /* Changes nlink field in statdata by passed @value */
@@ -537,7 +517,6 @@ errno_t obj40_clobber(obj40_t *obj) {
 errno_t obj40_layout(obj40_t *obj, region_func_t region_func,
 		     void *data)
 {
-	blk_t blk;
 	errno_t res;
 
 	aal_assert("umka-2547", obj != NULL);
@@ -546,8 +525,7 @@ errno_t obj40_layout(obj40_t *obj, region_func_t region_func,
 	if ((res = obj40_update(obj)))
 		return res;
 	
-	blk = STAT_PLACE(obj)->node->block->nr;
-	return region_func(obj, blk, 1, data);
+	return region_func(obj, place_blknr(STAT_PLACE(obj)), 1, data);
 }
 
 /* Enumerates object metadata. */
@@ -599,62 +577,30 @@ errno_t obj40_unlink(obj40_t *obj) {
 }
 #endif
 
-/* Obtains the plugin of the plugid returned by obj40_pid(). */
-reiser4_plug_t *obj40_plug(obj40_t *obj, rid_t type, char *name) {
-	rid_t pid = obj40_pid(obj, type, name);
-
-	/* Obtain the plugin by id. */
-	if (pid == INVAL_PID)
-		return NULL;
+/* Initializes object handle by plugin, key, core operations and opaque pointer
+   to tree file is going to be opened/created in. */
+errno_t obj40_init(obj40_t *obj, object_info_t *info, reiser4_core_t *core) {
+	uint8_t i;
 	
-	return obj->core->factory_ops.ifind(type, pid);
-}
-
-/* Obtains plugid of the type @type from the SD if it is kept there, othewise
-   obtains the default one from the params. */
-rid_t obj40_pid(obj40_t *obj, rid_t type, char *name) {
-	rid_t pid;
-	
-	aal_assert("vpf-1235", obj != NULL);
-	aal_assert("vpf-1236", STAT_PLACE(obj)->plug != NULL);
-	
-	pid = plug_call(STAT_PLACE(obj)->plug->o.item_ops->object,
-			object_plug, STAT_PLACE(obj), type);
-
-	/* If nothing found in SD, obtain the default one. */
-	if (pid == INVAL_PID) {
-#ifndef ENABLE_STAND_ALONE
-		pid = obj->core->param_ops.value(name);
-#else
-		if (type == HASH_PLUG_TYPE)
-			pid = HASH_R5_ID;
-		
-		if (type == FIBRE_PLUG_TYPE)
-			pid = FIBRE_DOT_O_ID;
-#endif
-	}
-	
-	return pid;
-}
-
-/*
-  Initializes object handle by plugin, key, core operations and opaque pointer
-  to tree file is going to be opened/created in. */
-errno_t obj40_init(obj40_t *obj, reiser4_plug_t *plug,
-		   reiser4_core_t *core, object_info_t *info)
-{
 	aal_assert("umka-1574", obj != NULL);
-	aal_assert("umka-1756", plug != NULL);
 	aal_assert("umka-1757", info != NULL);
-
-	obj->core = core;
-	obj->plug = plug;
+	aal_assert("umka-1757", info->tree != NULL);
 	
 	aal_memcpy(&obj->info, info, sizeof(*info));
+	obj->core = core;
+	
+	if (!info->start.plug) {
+		plug_call(info->object.plug->o.key_ops, assign, 
+			  STAT_KEY(obj), &info->object);
+	}
 
-	if (info->object.plug) {
-		plug_call(info->object.plug->o.key_ops,
-			  assign, STAT_KEY(obj), &info->object);
+	/* Set missed plugins to opset from the fs global opset and build
+	   the mask of the specific plugins. */
+	for (i = 0; i < OPSET_LAST; i++) {
+		aal_assert("vpf-1647", info->opset.plug[i] != INVAL_PTR);
+		
+		if (info->opset.plug[i] == NULL)
+			obj->info.opset.plug[i] = info->tree->opset[i];
 	}
 	
 	return 0;
@@ -669,7 +615,7 @@ errno_t obj40_update(obj40_t *obj) {
 	aal_assert("umka-1905", obj != NULL);
 		
 	/* Looking for stat data place by */
-	switch ((res = obj40_find_item(obj, &STAT_PLACE(obj)->key, 
+	switch ((res = obj40_find_item(obj, &obj->info.object,
 				       FIND_EXACT, NULL, NULL,
 				       STAT_PLACE(obj))))
 	{
