@@ -55,15 +55,12 @@ static void reg40_close(object_entity_t *entity) {
 }
 
 /* Updates body place in correspond to file offset. */
-lookup_t reg40_update_body(object_entity_t *entity) {
-	reg40_t *reg = (reg40_t *)entity;
-
-	aal_assert("umka-1161", entity != NULL);
+lookup_t reg40_update_body(reg40_t *reg) {
+	aal_assert("umka-1161", reg != NULL);
 	
 	/* Getting the next body item from the tree */
 	return obj40_find_item(&reg->obj, &reg->position, 
-			       FIND_EXACT, NULL, NULL,
-			       &reg->body);
+			       FIND_EXACT, NULL, NULL, &reg->body);
 }
 
 /* Resets file position. As fire position is stored inside @reg->position, it
@@ -138,47 +135,43 @@ static object_entity_t *reg40_open(object_info_t *info) {
 
 	/* Checking if passed info contains correct stat data coord. What if
 	   file has not stat data? */
-	if (info->start.plug->id.group != STATDATA_ITEM)
+	if (info->start.plug->id.group != STAT_ITEM)
 		return NULL;
 
+	if (info->opset.plug[OPSET_OBJ] != &reg40_plug)
+		return NULL;
+	
 	if (!(reg = aal_calloc(sizeof(*reg), 0)))
 		return NULL;
 
 	/* Initializing file handle. It is needed for working with file stat
 	   data item. */
-	obj40_init(&reg->obj, &reg40_plug, reg40_core, info);
+	obj40_init(&reg->obj, info, reg40_core);
 
-	/* Checking if stat data contains points to object, which can be handled
-	   by current plugin. */
-	if (obj40_pid(&reg->obj, OBJECT_PLUG_TYPE,
-		      "regular") !=  reg40_plug.id.id)
-	{
-		goto error_free_reg;
-	}
-
-	/* Initializing tail policy plugin. */
-#ifndef ENABLE_STAND_ALONE
-	if (!(reg->policy = obj40_plug(&reg->obj, POLICY_PLUG_TYPE,
-				       "policy")))
-	{
-		aal_error("Can't get file policy plugin.");
-		goto error_free_reg;
-	}
-#endif
-	
 	/* Reseting file (setting offset to 0) */
 	reg40_reset((object_entity_t *)reg);
+
+#ifndef ENABLE_STAND_ALONE
+	{
+		lookup_t lookup;
+
+		/* Get the body plugin in use. */
+		if ((lookup = reg40_update_body(reg)) < 0) {
+			aal_free(reg);
+			return NULL;
+		}
+
+		if (lookup) {
+			reg->body_plug = reg->body.plug;
+		}
+	}
+#endif
+
 	return (object_entity_t *)reg;
-	
- error_free_reg:
-	aal_free(reg);
-	return NULL;
 }
 
 /* Loads stat data to passed @hint */
-static errno_t reg40_stat(object_entity_t *entity,
-			  statdata_hint_t *hint)
-{
+static errno_t reg40_stat(object_entity_t *entity, stat_hint_t *hint) {
 	reg40_t *reg;
 	
 	aal_assert("umka-2561", entity != NULL);
@@ -191,61 +184,25 @@ static errno_t reg40_stat(object_entity_t *entity,
 #ifndef ENABLE_STAND_ALONE
 /* Create the file described by pased @hint. That is create files stat data
    item. */
-static object_entity_t *reg40_create(object_info_t *info,
-				     object_hint_t *hint)
-{
+static object_entity_t *reg40_create(object_hint_t *hint) {
 	reg40_t *reg;
-	uint64_t mask;
 	
-	aal_assert("umka-1169", info != NULL);
 	aal_assert("umka-1738", hint != NULL);
-	aal_assert("vpf-1093",  info->tree != NULL);
+	aal_assert("vpf-1093",  hint->info.tree != NULL);
 
 	if (!(reg = aal_calloc(sizeof(*reg), 0)))
 		return NULL;
 	
 	/* Initializing file handle. */
-	obj40_init(&reg->obj, &reg40_plug, reg40_core, info);
+	obj40_init(&reg->obj, &hint->info, reg40_core);
 
-	/* Getting tail policy plugin id. */
-	if (hint->body.reg.policy == INVAL_PID) {
-		/* Getting default tail policy from param if passed hint
-		   contains no valid tail policy plugin id. */
-		hint->body.reg.policy = reg40_core->param_ops.value("policy");
-
-		if (hint->body.reg.policy == INVAL_PID) {
-			aal_error("Invalid default tail policy "
-				  "plugin id has been detected.");
-			goto error_free_reg;
-		}
-	}
-
-	/* Initializing tail policy plugin. */
-	if (!(reg->policy = reg40_core->factory_ops.ifind(POLICY_PLUG_TYPE,
-						          hint->body.reg.policy)))
-	{
-		aal_error("Can't find tail policy plugin by "
-			  "its id 0x%x.", hint->body.reg.policy);
+	/* Create stat data item with size, bytes, nlinks equal to zero. */
+	if (obj40_create_stat(&reg->obj, 0, 0, 0, 0, S_IFREG, NULL))
 		goto error_free_reg;
-	}
-
-	/* Create stat data item with size, bytes, nlinks equal to zero and mask
-	   equal to @mask. */
-	mask = (1 << SDEXT_UNIX_ID | 1 << SDEXT_LW_ID);
-
-	if (obj40_create_stat(&reg->obj, hint->label.statdata,
-			      mask, 0, 0, 0, 0, S_IFREG, NULL))
-	{
-		goto error_free_reg;
-	}
-
-	/* Copy new created stat data coord to @info->start. It is needed on
-	   higher levels of abstraction and for fsck. */
-	aal_memcpy(&info->start, STAT_PLACE(&reg->obj),
-		   sizeof(info->start));
 
 	/* Reset file. */
 	reg40_reset((object_entity_t *)reg);
+
 	return (object_entity_t *)reg;
 
  error_free_reg:
@@ -288,22 +245,23 @@ static errno_t reg40_unlink(object_entity_t *entity) {
    @size -- new file size. This function will use tail policy plugin to find
    what kind of next body item should be writen. */
 reiser4_plug_t *reg40_policy_plug(reg40_t *reg, uint64_t new_size) {
-	aal_assert("umka-2394", reg != NULL);
-	aal_assert("umka-2393", reg->policy != NULL);
-
-	/* FIXME-UMKA: Here is not enough to have only plugin type to get it
-	   from stat data, as for tails and extents plugin type is the same and
-	   namely ITEM_PLUG_TYPE. */
+	reiser4_plug_t *policy;
 	
-	/* Calling tail policy plugin to detect body plugin. */
-	if (plug_call(reg->policy->o.policy_ops, tails, new_size)) {
+	aal_assert("umka-2394", reg != NULL);
+
+	policy = reg->obj.info.opset.plug[OPSET_POLICY];
+	
+	aal_assert("umka-2393", policy != NULL);
+
+	/* Calling formatting policy plugin to detect body plugin. */
+	if (plug_call(policy->o.policy_ops, tails, new_size)) {
 		/* Trying to get non-standard tail plugin from stat data. And if
 		   it is not found, default one from params will be taken. */
-		return obj40_plug(&reg->obj, ITEM_PLUG_TYPE, "tail");
-	} else {
-		/* The same for extent plugin */
-		return obj40_plug(&reg->obj, ITEM_PLUG_TYPE, "extent");
+		return reg->obj.info.opset.plug[OPSET_TAIL];
 	}
+	
+	/* The same for extent plugin */
+	return reg->obj.info.opset.plug[OPSET_EXTENT];
 }
 
 /* Makes tail2extent and extent2tail conversion. */
@@ -351,44 +309,38 @@ static errno_t reg40_convert(object_entity_t *entity,
 static errno_t reg40_check_body(object_entity_t *entity,
 				uint64_t new_size)
 {
+	reiser4_plug_t *plug;
 	reg40_t *reg;
-	uint64_t fsize;
-
-	reiser4_plug_t *body_plug;
-	reiser4_plug_t *policy_plug;
 	
 	aal_assert("umka-2395", entity != NULL);
 
 	reg = (reg40_t *)entity;
-	fsize = reg40_size(entity);
 	
 	/* There is nothing to convert? */
-	if (!fsize || !new_size)
+	if (!new_size)
 		return 0;
 
-	/* Getting policy plugin. It will be used for new file body items and
-	   old body items will be converted to. */
-	if (!(policy_plug = reg40_policy_plug(reg, new_size))) {
-		aal_error("Can't get body plugin for new file size %llu.",
-			  new_size);
+	/* Getting item plugin that should be used according to 
+	   the current tail policy plugin. */
+	if (!(plug = reg40_policy_plug(reg, new_size))) {
+		aal_error("Can't get body plugin for new "
+			  "file size %llu.", new_size);
 		return -EIO;
 	}
 
-	/* Getting old file body plugin. This is needed for comparing with new
-	   body plugin, told us by tail policy plugin. */
-	if (!(body_plug = reg40_policy_plug(reg, fsize))) {
-		aal_error("Can't get body plugin for old file size %llu.",
-			  fsize);
-		return -EIO;
+	if (!reg->body_plug) {
+		reg->body_plug = plug;
+		return 0;
 	}
 		
 	/* Comparing new plugin and old one. If they are the same, conversion if
 	   not needed. */
-	if (plug_equal(policy_plug, body_plug))
+	if (plug_equal(plug, reg->body_plug))
 		return 0;
 
 	/* Convert file. */
-	return reg40_convert(entity, policy_plug);
+	reg->body_plug = plug;
+	return reg40_convert(entity, plug);
 }
 
 /* Writes passed data to the file. Returns amount of data on disk, that is
@@ -399,7 +351,6 @@ int64_t reg40_put(object_entity_t *entity, void *buff,
 	reg40_t *reg;
 	int64_t written;
 	trans_hint_t hint;
-	uint64_t new_offset;
 
 	reg = (reg40_t *)entity;
 
@@ -413,16 +364,11 @@ int64_t reg40_put(object_entity_t *entity, void *buff,
 	hint.region_func = NULL;
 	hint.shift_flags = SF_DEFAULT;
 	hint.place_func = place_func;
+	hint.plug = reg->body_plug;
 	
 	plug_call(reg->position.plug->o.key_ops,
 		  assign, &hint.offset, &reg->position);
 
-	/* Getting file plugin for new offset. */
-	new_offset = reg40_offset(entity) + n;
-	
-	if (!(hint.plug = reg40_policy_plug(reg, new_offset)))
-		return -EIO;
-
 	/* Write data to tree. */
 	if ((written = obj40_write(&reg->obj, &hint)) < 0)
 		return written;
@@ -476,15 +422,17 @@ static int64_t reg40_write(object_entity_t *entity,
 	int64_t bytes;
 	uint64_t size;
 	uint64_t offset;
-	
+	uint64_t new_size;
+
 	aal_assert("umka-2281", entity != NULL);
 
 	reg = (reg40_t *)entity;
 	size = reg40_size(entity);
 	offset = reg40_offset(entity);
-
+	new_size = offset + n > size ? offset + n : size;
+	
 	/* Convert body items if needed. */
-	if ((res = reg40_check_body(entity, offset + n))) {
+	if ((res = reg40_check_body(entity, new_size))) {
 		aal_error("Can't perform tail conversion.");
 		return res;
 	}
@@ -493,15 +441,13 @@ static int64_t reg40_write(object_entity_t *entity,
 	if (offset > size) {
 		uint64_t hole = offset - size;
 
-		/* Seek back to size of hole, as reg40_put() uses @reg->position
-		   as data write offset. */
-		reg40_seek(entity, offset - hole);
+		/* Seek back to size of hole, as reg40_put() uses 
+		   @reg->position as data write offset. */
+		reg40_seek(entity, size);
 
 		/* Put a hole of size @hole. */
 		if ((bytes = reg40_put(entity, NULL, hole, NULL)) < 0)
 			return bytes;
-
-		size += hole;
 	} else {
 		bytes = 0;
 	}
@@ -512,18 +458,13 @@ static int64_t reg40_write(object_entity_t *entity,
 
 	bytes += res;
 
-	/* Updating stat data fields. */
+	/* Updating the SD place and update size, bytes there. */
 	if ((res = obj40_update(&reg->obj)))
 		return res;
 
-	/* Size should be updated only if we write beyond of size. */
-	if (offset >= size)
-		size += n;
-
 	/* Calculating new @bytes and updating stat data fields. */
 	bytes += obj40_get_bytes(&reg->obj);
-	
-	return obj40_touch(&reg->obj, size, bytes);
+	return obj40_touch(&reg->obj, new_size, bytes);
 }
 
 /* Truncates file to passed size @n. */
@@ -604,8 +545,8 @@ struct layout_hint {
 
 typedef struct layout_hint layout_hint_t;
 
-static errno_t callback_item_layout(void *place, blk_t start,
-				    count_t width, void *data)
+static errno_t cb_item_layout(void *place, blk_t start,
+			      count_t width, void *data)
 {
 	layout_hint_t *hint = (layout_hint_t *)data;
 	return hint->region_func(hint->entity, start,
@@ -643,7 +584,7 @@ static errno_t reg40_layout(object_entity_t *entity,
 		reiser4_place_t *place = &reg->body;
 		
 		/* Update current body coord. */
-		if ((res = reg40_update_body(entity)) < 0)
+		if ((res = reg40_update_body(reg)) < 0)
 			return res;
 
 		/* Check if file stream is over. */
@@ -653,15 +594,15 @@ static errno_t reg40_layout(object_entity_t *entity,
 		/* Calling item enumerator funtion for current body item. */
 		if (place->plug->o.item_ops->object->layout) {
 			if ((res = plug_call(place->plug->o.item_ops->object,
-					     layout, place, callback_item_layout,
+					     layout, place, cb_item_layout,
 					     &hint)))
 			{
 				return res;
 			}
 		} else {
-			blk_t blk = place->node->block->nr;
+			blk_t blk = place_blknr(place);
 			
-			if ((res = callback_item_layout(place, blk, 1, &hint)))
+			if ((res = cb_item_layout(place, blk, 1, &hint)))
 				return res;
 		}
 
@@ -705,7 +646,7 @@ static errno_t reg40_metadata(object_entity_t *entity,
 		reiser4_key_t maxkey;
 		
 		/* Update body place. */
-		if ((res = reg40_update_body(entity)) < 0)
+		if ((res = reg40_update_body(reg)) < 0)
 			return res;
 
 		/* Check if file stream is over. */
@@ -730,9 +671,7 @@ static errno_t reg40_metadata(object_entity_t *entity,
 }
 
 /* Updates stat data from passed @hint */
-static errno_t reg40_update(object_entity_t *entity,
-			    statdata_hint_t *hint)
-{
+static errno_t reg40_update(object_entity_t *entity, stat_hint_t *hint) {
 	reg40_t *reg;
 	
 	aal_assert("umka-2559", entity != NULL);
@@ -752,7 +691,6 @@ static reiser4_object_ops_t reg40_ops = {
 	.layout         = reg40_layout,
 	.metadata       = reg40_metadata,
 	.convert        = reg40_convert,
-	.form		= reg40_form,
 	.update         = reg40_update,
 	.link           = reg40_link,
 	.unlink         = reg40_unlink,
