@@ -7,13 +7,11 @@
 #define REG40_H
 
 #include <aal/libaal.h>
-#include <sys/stat.h>
 #include <reiser4/plugin.h>
 #include <plugin/object/obj40/obj40.h>
 
 /* Regular file structure */
 struct reg40 {
-
 	/* Common fields (statdata, etc) */
 	obj40_t obj;
 
@@ -22,10 +20,10 @@ struct reg40 {
 
 	/* Current position in the reg file */
 	reiser4_key_t position;
-
+	
 #ifndef ENABLE_STAND_ALONE
-	/* Tail policy plugin */
-	reiser4_plug_t *policy;
+	/* File body plugin is use. */
+	reiser4_plug_t *body_plug;
 #endif
 };
 
@@ -42,7 +40,7 @@ extern errno_t reg40_seek(object_entity_t *entity,
 
 extern errno_t reg40_reset(object_entity_t *entity);
 extern uint64_t reg40_offset(object_entity_t *entity);
-extern lookup_t reg40_update_body(object_entity_t *entity);
+extern lookup_t reg40_update_body(reg40_t *reg);
 
 extern reiser4_plug_t *reg40_policy_plug(reg40_t *reg,
 					 uint64_t new_size);
