@@ -26,10 +26,11 @@ static void fsck_print_usage(char *name) {
 		"  -v, --verbose                 makes fsck to be verbose\n"
 		"  -r                            ignored\n"
 		"Plugins options:\n"
-		"  -p, --print-params            prints default params.\n"
+		"  -p, --print-profile           prints the plugin profile.\n"
 		"  -l, --print-plugins           prints all known plugins.\n"
 		"  -o, --override TYPE=PLUGIN    overrides the default plugin of the type\n"
-	        "                                \"TYPE\" by the plugin \"PLUGIN\".\n"
+	        "                                \"TYPE\" by the plugin \"PLUGIN\" in the\n"
+		"                                profile.\n"
 		"Common options:\n"
 		"  -?, -h, --help                prints program usage.\n"
 		"  -V, --version                 prints current version.\n"
@@ -42,24 +43,24 @@ static void fsck_print_usage(char *name) {
 
 #define WARNING \
 "*******************************************************************\n"\
-"This is an EXPERIMENTAL version of fsck.reiser4. Read REAMDE first.\n"\
-"*******************************************************************\n\n"
+"This is an EXPERIMENTAL version of fsck.reiser4. Read README first.\n"\
+"*******************************************************************\n"
 
 static errno_t fsck_ask_confirmation(fsck_parse_t *data, char *host_name) {
-	fprintf(stderr, WARNING);
-	fprintf(stderr, "Fscking the %s block device.\n", host_name);
+	fprintf(stderr, WARNING "\n");
+	aal_mess("Fscking the %s block device.", host_name);
 	
 	switch (data->sb_mode) {
 	case RM_CHECK:
-		fprintf(stderr, "Will check the consistency of the Reiser4 "
-			"SuperBlock.\n");
+		aal_mess("Will check the consistency of "
+			 "the Reiser4 SuperBlock.");
 		break;
 	case RM_FIX:
-		fprintf(stderr, "Will fix minor corruptions of the Reiser4 "
-			"SuperBblock.\n");
+		aal_mess("Will fix minor corruptions of "
+			 "the Reiser4 SuperBblock.");
 		break;
 	case RM_BUILD:
-		fprintf(stderr, "Will build the Reiser4 SuperBlock.\n");
+		aal_mess("Will build the Reiser4 SuperBlock.");
 		break;
 	default:
 		break;
@@ -67,19 +68,19 @@ static errno_t fsck_ask_confirmation(fsck_parse_t *data, char *host_name) {
 	
 	switch (data->fs_mode) {
 	case RM_CHECK:
-		fprintf(stderr, "Will check the consistency of the Reiser4 "
-			"FileSystem.\n");
+		aal_mess("Will check the consistency of "
+			 "the Reiser4 FileSystem.");
 		break;
 	case RM_FIX:
-		fprintf(stderr, "Will fix minor corruptions of the Reiser4 "
-			"FileSystem.\n");
+		aal_mess("Will fix minor corruptions of "
+			 "the Reiser4 FileSystem.");
 		break;
 	case RM_BUILD:
-		fprintf(stderr, "Will build the Reiser4 FileSystem.\n");
+		aal_mess("Will build the Reiser4 FileSystem.");
 		break;
 	case RM_BACK:
-		fprintf(stderr, "Will rollback changes saved in '%s' back "
-			"onto (%s).\n", data->backup_file, host_name);
+		aal_mess("Will rollback changes saved in '%s' back "
+			"onto (%s).", data->backup_file, host_name);
 		break;
 	default:
 		break;
@@ -91,17 +92,12 @@ static errno_t fsck_ask_confirmation(fsck_parse_t *data, char *host_name) {
 	return NO_ERROR; 
 }
 
-static void fsck_init_streams(fsck_parse_t *data) {
+static void fsck_init_streams() {
+	int ex;
+
+	for (ex = 0; ex < EXCEPTION_TYPE_LAST; ex++)
+		misc_exception_set_stream(ex, stderr);
 	
-	misc_exception_set_stream(EXCEPTION_TYPE_INFO, 
-				  aal_test_bit(&data->options,
-					       FSCK_OPT_VERBOSE) ? stderr : NULL);
-	
-	misc_exception_set_stream(EXCEPTION_TYPE_ERROR, data->logfile);
-	misc_exception_set_stream(EXCEPTION_TYPE_WARNING, data->logfile);
-	misc_exception_set_stream(EXCEPTION_TYPE_MESSAGE, stdout);
-	misc_exception_set_stream(EXCEPTION_TYPE_FATAL, stderr);
-	misc_exception_set_stream(EXCEPTION_TYPE_BUG, stderr);
 	misc_exception_set_stream(EXCEPTION_TYPE_INFO, stdout);
 }
 
@@ -120,7 +116,7 @@ static errno_t fsck_init(fsck_parse_t *data,
 	int option_index;
 	errno_t ret = 0;
 	uint32_t cache;
-	int c;
+	int mounted, c;
 
 	static struct option options[] = {
 		/* FSCK modes. */
@@ -128,7 +124,7 @@ static errno_t fsck_init(fsck_parse_t *data,
 		{"fix", no_argument, &mode, RM_FIX},
 		{"build-sb", no_argument, &sb_mode, RM_BUILD},
 		{"build-fs", no_argument, &fs_mode, RM_BUILD},
-		{"print-params", no_argument, &mode, RM_SHOW_PARM},
+		{"print-profile", no_argument, &mode, RM_SHOW_PARM},
 		{"print-plugins", no_argument, &mode, RM_SHOW_PLUG},
 		/* Fsck hidden modes. */
 		{"rollback", no_argument, &mode, RM_BACK},
@@ -150,7 +146,8 @@ static errno_t fsck_init(fsck_parse_t *data,
 		{0, 0, 0, 0}
 	};
 
-	misc_exception_set_stream(EXCEPTION_TYPE_FATAL, stderr);
+	fsck_init_streams();
+	aux_gauge_set_handler(misc_progress_handler, GT_PROGRESS);
 	memset(override, 0, sizeof(override));
 	data->logfile = stderr;
 
@@ -201,9 +198,7 @@ static errno_t fsck_init(fsck_parse_t *data,
 			misc_print_banner(argv[0]);
 			goto user_error;
 		case 'q':
-			aal_gauge_set_handler(GAUGE_PERCENTAGE, NULL);
-			aal_gauge_set_handler(GAUGE_INDICATOR, NULL);
-			aal_gauge_set_handler(GAUGE_SILENT, NULL);
+			aux_gauge_set_handler(NULL, GT_PROGRESS);
 			break;
 		case 'r':
 			break;
@@ -215,7 +210,7 @@ static errno_t fsck_init(fsck_parse_t *data,
 			break;
 		case 'c':
 			if ((cache = misc_str2long(optarg, 10)) == INVAL_DIG) {
-				aal_error("Invalid cache value specified (%s).",
+				aal_fatal("Invalid cache value specified (%s).",
 					  optarg);
 				return USER_ERROR;
 			}
@@ -241,8 +236,18 @@ static errno_t fsck_init(fsck_parse_t *data,
 		goto oper_error;
 	}
 
+	if (aal_strlen(override) > 0) {
+		override[aal_strlen(override) - 1] = '\0';
+
+		aal_mess("Overriding the plugin profile by \"%s\".", 
+			 override);
+		
+		if (misc_profile_override(override))
+			goto oper_error;
+	}
+
 	if (mode == RM_SHOW_PARM) {
-		misc_param_print();
+		misc_profile_print();
 		goto user_error;
 	}
 
@@ -251,13 +256,16 @@ static errno_t fsck_init(fsck_parse_t *data,
 		goto user_error;
 	}
 
-	fsck_init_streams(data);
-    
 	if (optind != argc - 1) {
 		fsck_print_usage(argv[0]);
 		goto user_error;
 	}
 	
+	if (aal_test_bit(&data->options, FSCK_OPT_AUTO)) {
+		aal_mess("%s %s", argv[0], argv[optind]);
+		exit(NO_ERROR);
+	}
+
 	data->sb_mode = sb_mode ? sb_mode : mode;
 	data->fs_mode = fs_mode ? fs_mode : mode;
   
@@ -274,14 +282,22 @@ static errno_t fsck_init(fsck_parse_t *data,
 	}
 	
 	/* Check if device is mounted and we are able to fsck it. */ 
-	if (misc_dev_mounted(argv[optind], NULL)) {
-		if (!misc_dev_mounted(argv[optind], "ro")) {
-			aal_fatal("The partition (%s) is mounted "
-				  "w/ write permissions, cannot "
-				  "fsck it.", argv[optind]);
+	if ((mounted = misc_dev_mounted(argv[optind])) > 0) {
+		if (mounted == MF_RW) {
+			aal_fatal("The partition (%s) is mounted with write "
+				  "permissions, cannot fsck it.", argv[optind]);
 			goto user_error;
-		} else {
-			aal_set_bit(&data->options, FSCK_OPT_READ_ONLY);
+		}
+		
+		aal_set_bit(&data->options, FSCK_OPT_RO);
+
+		if (data->sb_mode != RM_CHECK || data->fs_mode != RM_CHECK) {
+			/* If not CHECK mode, lock the process in the memory. */
+			if (mlockall(MCL_CURRENT)) {
+				aal_fatal("Failed to lock the process in the "
+					  "memory: %s.", strerror(errno));
+				goto oper_error;
+			}
 		}
 	}
     
@@ -293,23 +309,7 @@ static errno_t fsck_init(fsck_parse_t *data,
 		goto oper_error;
 	}
 
-	if (aal_test_bit(&data->options, FSCK_OPT_AUTO)) {
-		misc_print_banner(argv[0]);
-		exit(NO_ERROR);
-	}
-
-	aal_gauge_set_handler(GAUGE_PERCENTAGE, gauge_rate);
-	aal_gauge_set_handler(GAUGE_TREE, gauge_tree);
-
-	if (aal_strlen(override) > 0) {
-		override[aal_strlen(override) - 1] = '\0';
-
-		aal_mess("Overriding default params by \"%s\".", override);
-		
-		if (misc_param_override(override))
-			goto oper_error;
-	}
-	
+	misc_exception_set_stream(EXCEPTION_TYPE_FSCK, data->logfile);
 	return fsck_ask_confirmation(data, argv[optind]);
 	
  user_error:
@@ -349,12 +349,10 @@ static errno_t fsck_check_init(repair_data_t *repair,
 			       uint8_t fs_mode) 
 {
 	aal_stream_t stream;
+	int flags = 0;
 	int reop = 0;
 	count_t len;
 	errno_t res;
-	int flags;
-	
-	fprintf(stderr, "***** Openning the fs.\n");
 	
 	/* Reopen device RW for fixing SB. */
 	if (sb_mode != RM_CHECK) {
@@ -406,14 +404,9 @@ static errno_t fsck_check_init(repair_data_t *repair,
 	repair->fixable = 0;
 	repair->mode = fs_mode;
 
-	/* Check the openned fs. */
-	if ((res = repair_fs_valid(repair->fs, fs_mode)) < 0)
-		goto error_close_fs;
-	
 	repair_error_count(repair, res);
 	
 	repair->fs->tree->mpc_func = misc_mpressure_detect;
-	repair->progress_handler = gauge_handler;    
 	
 	aal_stream_init(&stream, NULL, &memory_stream);
 	
@@ -425,7 +418,7 @@ static errno_t fsck_check_init(repair_data_t *repair,
 	repair_format_print(repair->fs->format, &stream);
 	aal_stream_format(&stream, "\n");
 	
-	fprintf(stderr, "Reiser4 fs was detected on the %s.\n%s",
+	aal_mess("Reiser4 fs was detected on %s.\n%s",
 		repair->fs->device->name, (char *)stream.entity);
 	
 	aal_stream_fini(&stream);
@@ -508,6 +501,8 @@ int main(int argc, char *argv[]) {
 		goto free_device;
 	}
 	
+	fsck_time("fsck.reiser4 started at");
+	
 	/* SB_mode is specified, otherwise  */
 	repair.debug_flag = aal_test_bit(&parse_data.options, FSCK_OPT_DEBUG);
 	repair.bitmap_file = parse_data.bitmap_file;
@@ -549,7 +544,7 @@ int main(int argc, char *argv[]) {
 	/* Report about the results. */
 	if (res < 0 || ex == OPER_ERROR) {
 		ex = OPER_ERROR;
-		aal_mess("Operational error occured while fscking.");
+		fprintf(stderr, "Operational error occured while fscking.\n");
 		goto free_fsck;
 	} 
 	
@@ -568,7 +563,9 @@ int main(int argc, char *argv[]) {
 	if (repair.fatal) {
 		/* Some fatal corruptions in disk format or filesystem. */
 		if (parse_data.fs_mode == RM_BUILD) {
-			fprintf(stderr, "Failed to build the reiser4 filesystem.\n");
+			/* Only if no metadata are found. */
+			fprintf(stderr, "NO REISER4 METADATA WERE FOUND. "
+				"FS RECOVERY IS NOT POSSIBLE.\n");
 		} else {
 			fprintf(stderr, "%llu fatal corruptions were detected in %s. "
 				"Run with %s option to fix them.\n", repair.fatal, 
@@ -583,12 +580,21 @@ int main(int argc, char *argv[]) {
 			repair.fixable);
 		
 		ex = FIXABLE_ERROR;
-	} else if (!repair.sb_fixable)
-		fprintf(stderr, "No corruption found.\n\n");
+	} else if (!repair.sb_fixable) {
+		if (parse_data.fs_mode != RM_CHECK && 
+		    aal_test_bit(&parse_data.options, FSCK_OPT_RO))
+		{
+			fprintf(stderr, "\nThe filesystem was mounted RO. It is "
+				"better to umount and mount it again.\n");
+		}
+		
+		fprintf(stderr, "FS is consistent.\n");
+	}
+	
+	fprintf(stderr, "\n");
 
  free_fsck:
 	fsck_fini(&parse_data);
 	
 	return ex;
 }
-
