@@ -49,10 +49,11 @@ static void measurefs_print_usage(char *name) {
 		"                                during calclulation if --data-frag is\n"
 		"                                specified.\n"
 		"Plugins options:\n"
-		"  -p, --print-params            prints default params.\n"
+		"  -p, --print-profile           prints default profile.\n"
 		"  -l, --print-plugins           prints known plugins.\n"
 	        "  -o, --override TYPE=PLUGIN    overrides the default plugin of the type\n"
-	        "                                \"TYPE\" by the plugin \"PLUGIN\".\n"
+	        "                                \"TYPE\" by the plugin \"PLUGIN\" in the\n"
+		"                                profile.\n"
 		"Common options:\n"
 		"  -?, -h, --help                prints program usage.\n"
 		"  -V, --version                 prints current version.\n"
@@ -133,9 +134,7 @@ static errno_t tree_frag_process_item(void *entity, uint64_t start,
 /* Traverse passed @node and calculate tree fragmentation for it. The results
    are stored in @frag_hint structure. This function is called from the tree
    traversal routine for each internal node. See bellow for details. */
-static errno_t tree_frag_process_node(reiser4_tree_t *tree,
-				      reiser4_node_t *node, void *data)
-{
+static errno_t tree_frag_process_node(reiser4_node_t *node, void *data) {
 	pos_t pos;
 	static int bogus = 0;
 	tree_frag_hint_t *frag_hint;
@@ -150,12 +149,13 @@ static errno_t tree_frag_process_node(reiser4_tree_t *tree,
 		/* Initializing item at @place */
 		if (reiser4_place_open(&place, node, &pos)) {
 			aal_error("Can't open item %u in node %llu.",
-				  pos.item, node_blocknr(node));
+				  pos.item, node->block->nr);
 			return -EINVAL;
 		}
 
+		/* FIXME: touch eaery time. show once per second. */
 		if (frag_hint->gauge && pos.item == 0 && bogus++ % 128 == 0)
-			aal_gauge_update(frag_hint->gauge, 0);
+			aal_gauge_touch(frag_hint->gauge);
 		
 		/* Checking and calling item's layout method with function
 		   tfrag_process_item() as a function for handling one block the
@@ -171,9 +171,7 @@ static errno_t tree_frag_process_node(reiser4_tree_t *tree,
 	return 0;
 }
 
-static errno_t tree_frag_update_node(reiser4_tree_t *tree,
-				     reiser4_place_t *place, void *data)
-{
+static errno_t tree_frag_update_node(reiser4_place_t *place, void *data) {
 	((tree_frag_hint_t *)data)->level++;
 	return 0;
 }
@@ -184,8 +182,8 @@ static errno_t tree_frag_update_node(reiser4_tree_t *tree,
    traverse node, etc). Actual statistics collecting is performed in the passed
    callbacks and subcallbacks (for item traversing). */
 errno_t measurefs_tree_frag(reiser4_fs_t *fs, uint32_t flags) {
-	errno_t res;
 	tree_frag_hint_t frag_hint;
+	errno_t res;
 
 	aal_memset(&frag_hint, 0, sizeof(frag_hint));
 	
@@ -193,13 +191,12 @@ errno_t measurefs_tree_frag(reiser4_fs_t *fs, uint32_t flags) {
 		/* Initializing gauge, because it is a long process and user
 		   should be informed what the stage of the process is going at
 		   the moment. */
-		if (!(frag_hint.gauge = aal_gauge_create(GAUGE_INDICATOR,
-							 NULL)))
-		{
+		frag_hint.gauge = 
+			aal_gauge_create(aux_gauge_handlers[GT_PROGRESS],
+					 NULL, NULL, 0, "Tree fragmentation "
+					 "... ");
+		if (!frag_hint.gauge)
 			return -ENOMEM;
-		}
-
-		aal_gauge_rename(frag_hint.gauge, "Tree fragmentation");
 	}
 	
 	/* Preparing serve structure, statistics will be stored in  */
@@ -207,7 +204,7 @@ errno_t measurefs_tree_frag(reiser4_fs_t *fs, uint32_t flags) {
 	frag_hint.level = reiser4_tree_get_height(fs->tree);
 	
 	if (frag_hint.gauge)
-		aal_gauge_start(frag_hint.gauge);
+		aal_gauge_touch(frag_hint.gauge);
 
 	/* Calling tree traversal with callbacks for processing internal nodes
 	   and items in order to calculate tree fragmentation. */
@@ -220,10 +217,12 @@ errno_t measurefs_tree_frag(reiser4_fs_t *fs, uint32_t flags) {
 	}
 
 	/* Printing results. */
-	if (frag_hint.gauge)
+	if (frag_hint.gauge) {
+		aal_gauge_done(frag_hint.gauge);
 		aal_gauge_free(frag_hint.gauge);
-	else
+	} else {
 		printf("Tree fragmentation: ");
+	}
 
 	printf("%.6f\n", frag_hint.total > 0 ?
 	       (double)frag_hint.bad / frag_hint.total : 0);
@@ -287,15 +286,15 @@ static errno_t stat_item_layout(void *entity, uint64_t start,
 
 /* Processing one formatted node and calculate number of internal pointers,
    extent ones, packing, etc. */
-static errno_t stat_process_node(reiser4_tree_t *tree,
-				 reiser4_node_t *node, void *data)
-{
+static errno_t stat_process_node(reiser4_node_t *node, void *data) {
 	uint8_t level;
 	uint32_t blksize;
 	uint32_t twigs_used;
 	uint32_t leaves_used;
 	uint32_t branches_used;
 	uint32_t formatted_used;
+	
+	reiser4_tree_t *tree = (reiser4_tree_t *)node->tree;
 
 	tree_stat_hint_t *stat_hint;
 	pos_t pos = {MAX_UINT32, MAX_UINT32};
@@ -305,8 +304,9 @@ static errno_t stat_process_node(reiser4_tree_t *tree,
 
 	blksize = reiser4_master_get_blksize(tree->fs->master);
 
+	/* FIXME: touch eaery time. show once per second. */
 	if (stat_hint->gauge && stat_hint->formatted % 128 == 0)
-		aal_gauge_update(stat_hint->gauge, 0);
+		aal_gauge_touch(stat_hint->gauge);
 
 	formatted_used = blksize - reiser4_node_space(node);
 
@@ -352,7 +352,7 @@ static errno_t stat_process_node(reiser4_tree_t *tree,
 		/* Fetching item data. */
 		if ((res = reiser4_place_open(&place, node, &pos))) {
 			aal_error("Can't open item %u in node %llu.",
-				  pos.item, node_blocknr(node));
+				  pos.item, node->block->nr);
 			return res;
 		}
 
@@ -361,19 +361,19 @@ static errno_t stat_process_node(reiser4_tree_t *tree,
 		stat_hint->items++;
 
 		switch (place.plug->id.group) {
-		case STATDATA_ITEM:
+		case STAT_ITEM:
 			stat_hint->statdatas++;
 			break;
 		case EXTENT_ITEM:
 			stat_hint->extents++;
 			break;
-		case NODEPTR_ITEM:
+		case PTR_ITEM:
 			stat_hint->nodeptrs++;
 			break;
 		case TAIL_ITEM:
 			stat_hint->tails++;
 			break;
-		case DIRENTRY_ITEM:
+		case DIR_ITEM:
 			stat_hint->direntries++;
 			break;
 		}
@@ -401,26 +401,24 @@ static errno_t stat_process_node(reiser4_tree_t *tree,
 
 /* Entry point function for calculating tree statistics */
 errno_t measurefs_tree_stat(reiser4_fs_t *fs, uint32_t flags) {
-	errno_t res;
-	uint32_t blksize;
 	tree_stat_hint_t stat_hint;
+	uint32_t blksize;
+	errno_t res;
 
 	aal_memset(&stat_hint, 0, sizeof(stat_hint));
 
 	/* Creating gauge. */
 	if (!(flags & BF_QUIET)) {
-		if (!(stat_hint.gauge = aal_gauge_create(GAUGE_INDICATOR,
-							 NULL)))
-		{
+		stat_hint.gauge = 
+			aal_gauge_create(aux_gauge_handlers[GT_PROGRESS],
+					 NULL, NULL, 0, "Tree statistics ... ");
+		if (!stat_hint.gauge)
 			return -ENOMEM;
-		}
-
-		aal_gauge_rename(stat_hint.gauge, "Tree statistics");
 	}
 
 	/* Traversing tree with callbacks for calculating tree statistics. */
 	if (stat_hint.gauge)
-		aal_gauge_start(stat_hint.gauge);
+		aal_gauge_touch(stat_hint.gauge);
 
 	stat_hint.tree = fs->tree;
 	
@@ -431,6 +429,7 @@ errno_t measurefs_tree_stat(reiser4_fs_t *fs, uint32_t flags) {
 	}
 
 	if (stat_hint.gauge) {
+		aal_gauge_done(stat_hint.gauge);
 		aal_gauge_free(stat_hint.gauge);
 		misc_wipe_line(stdout);
 	}
@@ -519,16 +518,13 @@ static errno_t file_frag_process_blk(void *entity, blk_t start,
 }
 
 /* Calculates the passed file fragmentation. */
-errno_t measurefs_file_frag(reiser4_fs_t *fs,
-			    char *filename,
-			    uint32_t gauge)
-{
+errno_t measurefs_file_frag(reiser4_fs_t *fs, char *filename, uint32_t gauge) {
 	errno_t res = 0;
 	reiser4_object_t *object;
 	file_frag_hint_t frag_hint;
 
 	/* Opens object by its name */
-	if (!(object = reiser4_object_open(fs->tree, filename, 0)))
+	if (!(object = reiser4_semantic_open(fs->tree, filename, NULL, 0)))
 		return -EINVAL;
 
 	/* Initializing serve structures */
@@ -560,12 +556,10 @@ errno_t measurefs_file_frag(reiser4_fs_t *fs,
 
 /* Processes leaf node in order to find all stat data items which are start of
    corresponding files and calculate file fragmentation for each of them. */
-static errno_t data_frag_process_node(reiser4_tree_t *tree,
-				      reiser4_node_t *node,
-				      void *data)
-{
-	pos_t pos;
+static errno_t data_frag_process_node(reiser4_node_t *node, void *data) {
+	reiser4_tree_t *tree = (reiser4_tree_t *)node->tree;
 	file_frag_hint_t *frag_hint;
+	pos_t pos;
 
 	pos.unit = MAX_UINT32;
 	frag_hint = (file_frag_hint_t *)data;
@@ -586,7 +580,7 @@ static errno_t data_frag_process_node(reiser4_tree_t *tree,
 		/* Initialiing the item at @place */
 		if ((res = reiser4_place_open(&place, node, &pos))) {
 			aal_error("Can't open item %u in node %llu.",
-				  pos.item, node_blocknr(node));
+				  pos.item, node->block->nr);
 			return res;
 		}
 
@@ -594,7 +588,7 @@ static errno_t data_frag_process_node(reiser4_tree_t *tree,
 			continue;
 
 		/* Opening object by its stat data item denoded by @place */
-		if (!(object = reiser4_object_realize(tree, NULL, &place)))
+		if (!(object = reiser4_object_open(tree, NULL, &place)))
 			continue;
 
 		/* Initializing per-file counters */
@@ -603,13 +597,13 @@ static errno_t data_frag_process_node(reiser4_tree_t *tree,
 		frag_hint->total = 0;
 
 		if (frag_hint->gauge && pos.item == 0)
-			aal_gauge_update(frag_hint->gauge, 0);
+			aal_gauge_touch(frag_hint->gauge);
 
 		/* Calling calculating the file fragmentation by emans of using
 		   the function we have seen abowe. */
 		if (reiser4_object_layout(object, file_frag_process_blk, data)) {
-			aal_error("Can't enumerate data blocks "
-				  "occupied by %s", object->name);
+			aal_error("Can't enumerate data blocks occupied by %s", 
+				  reiser4_print_key(&object->ent->object, PO_INODE));
 			goto error_close_object;
 		}
 
@@ -630,7 +624,8 @@ static errno_t data_frag_process_node(reiser4_tree_t *tree,
 				(double)frag_hint->current / frag_hint->files : 0;
 			
 			aal_mess("Fragmentation for %s: %.6f [av. %.6f ]",
-				 object->name, file_factor, curr_factor);
+				 reiser4_print_key(&object->ent->object, PO_INODE),
+				 file_factor, curr_factor);
 		}
 
 	error_close_object:
@@ -640,47 +635,43 @@ static errno_t data_frag_process_node(reiser4_tree_t *tree,
 	return 0;
 }
 
-static errno_t data_frag_update_node(reiser4_tree_t *tree,
-				     reiser4_place_t *place,
-				     void *data)
-{
+static errno_t data_frag_update_node(reiser4_place_t *place, void *data) {
 	((file_frag_hint_t *)data)->level++;
 	return 0;
 }
 
 /* Entry point function for data fragmentation. */
-errno_t measurefs_data_frag(reiser4_fs_t *fs,
-			    uint32_t flags)
-{
-	errno_t res;
+errno_t measurefs_data_frag(reiser4_fs_t *fs, uint32_t flags) {
 	file_frag_hint_t frag_hint;
+	errno_t res;
 
 	aal_memset(&frag_hint, 0, sizeof(frag_hint));
 
 	/* Create gauge. */
 	if (!(flags & BF_QUIET)) {
-		if (!(frag_hint.gauge = aal_gauge_create(GAUGE_INDICATOR,
-							 NULL)))
-		{
+		frag_hint.gauge = 
+			aal_gauge_create(aux_gauge_handlers[GT_PROGRESS],
+					 NULL, NULL, 0, "Data fragmentation "
+					 "... ");
+		if (!frag_hint.gauge)
 			aal_fatal("Out of memory!");
 			return -ENOMEM;
-		}
-
-		aal_gauge_rename(frag_hint.gauge, "Data fragmentation");
 	}
 
 	frag_hint.flags = flags;
 	frag_hint.level = reiser4_tree_get_height(fs->tree);
 	
 	if (frag_hint.gauge)
-		aal_gauge_start(frag_hint.gauge);
+		aal_gauge_touch(frag_hint.gauge);
 	
 	if ((res = reiser4_tree_trav(fs->tree, NULL, data_frag_process_node,
 				     data_frag_update_node, NULL, &frag_hint)))
 		return res;
 
-	if (frag_hint.gauge)
+	if (frag_hint.gauge) {
+		aal_gauge_done(frag_hint.gauge);
 		aal_gauge_free(frag_hint.gauge);
+	}
 
 	if (frag_hint.flags & BF_SHOW_FILE || !frag_hint.gauge)
 		printf("Data fragmentation is: ");
@@ -713,7 +704,7 @@ int main(int argc, char *argv[]) {
 		{"file-frag", required_argument, NULL, 'F'},
 		{"data-frag", no_argument, NULL, 'D'},
 		{"show-file", no_argument, NULL, 'E'},
-		{"print-params", no_argument, NULL, 'p'},
+		{"print-profile", no_argument, NULL, 'p'},
 		{"print-plugins", no_argument, NULL, 'l'},
 		{"override", required_argument, NULL, 'o'},
 		{"cache", required_argument, NULL, 'c'},
@@ -802,16 +793,16 @@ int main(int argc, char *argv[]) {
 		override[aal_strlen(override) - 1] = '\0';
 		
 		if (!(flags & BF_QUIET)) {
-			aal_mess("Overriding default params "
-				 "by \"%s\".", override);
+			aal_mess("Overriding the plugin profile by \"%s\".", 
+				 override);
 		}
 		
-		if (misc_param_override(override))
+		if (misc_profile_override(override))
 			goto error_free_libreiser4;
 	}
 	
 	if (flags & BF_SHOW_PARM)
-		misc_param_print();
+		misc_profile_print();
 
 	if (flags & BF_SHOW_PLUG)
 		misc_plugins_print();
@@ -826,7 +817,7 @@ int main(int argc, char *argv[]) {
 	host_dev = argv[optind];
 
 	/* Checking if passed partition is mounted */
-	if (misc_dev_mounted(host_dev, NULL) && !(flags & BF_FORCE)) {
+	if (misc_dev_mounted(host_dev) > 0 && !(flags & BF_FORCE)) {
 		aal_error("Device %s is mounted at the moment. "
 			  "Use -f to force over.", host_dev);
 		goto error_free_libreiser4;
