@@ -54,10 +54,11 @@ static void mkfs_print_usage(char *name) {
 		"  -L, --label LABEL             volume label lets to mount\n"
 		"                                filesystem by its label.\n"
 		"Plugins options:\n"
-		"  -p, --print-params            prints default params.\n"
+		"  -p, --print-profile           prints the plugin profile.\n"
 		"  -l, --print-plugins           prints all known plugins.\n"
 	        "  -o, --override TYPE=PLUGIN    overrides the default plugin of the type\n"
-	        "                                \"TYPE\" by the plugin \"PLUGIN\".\n"
+	        "                                \"TYPE\" by the plugin \"PLUGIN\" in the\n"
+		"                                profile.\n"
 		"Common options:\n"
 		"  -?, -h, --help                prints program usage.\n"
 		"  -V, --version                 prints current version.\n"
@@ -76,6 +77,28 @@ static void mkfs_init(void) {
 		misc_exception_set_stream(ex, stderr);
 }
 
+static reiser4_object_t *reiser4_root_create(reiser4_fs_t *fs) {
+	entry_hint_t entry;
+	object_hint_t hint;
+	
+	aal_assert("vpf-1625", fs != NULL);
+	aal_assert("vpf-1626", fs->tree != NULL);
+	
+	aal_memset(&hint, 0, sizeof(hint));
+
+	/* Preparing object hint. INVAL_PTR means that this kind of plugin 
+	   is not ready yet but needs to be stored, */
+	hint.mode = 0;
+	reiser4_opset_profile(hint.info.opset.plug);
+	hint.info.opset.plug[OPSET_OBJ] = hint.info.opset.plug[OPSET_MKDIR];
+
+	/* Preparing entry hint. */
+	entry.name[0] = '\0';
+	reiser4_key_assign(&entry.offset, &fs->tree->key);
+
+	return reiser4_object_create(fs->tree, &entry, &hint);
+}
+
 int main(int argc, char *argv[]) {
 	int c;
 	struct stat st;
@@ -107,7 +130,7 @@ int main(int argc, char *argv[]) {
 		{"label", required_argument, NULL, 'L'},
 		{"uuid", required_argument, NULL, 'U'},
 		{"lost-found", required_argument, NULL, 's'},
-		{"print-params", no_argument, NULL, 'p'},
+		{"print-profile", no_argument, NULL, 'p'},
 		{"print-plugins", no_argument, NULL, 'l'},
 		{"override", required_argument, NULL, 'o'},
 		{0, 0, 0, 0}
@@ -221,16 +244,16 @@ int main(int argc, char *argv[]) {
 		override[aal_strlen(override) - 1] = '\0';
 
 		if (!(flags & BF_QUIET)) {
-			aal_mess("Overriding default params "
-				 "by \"%s\".", override);
+			aal_mess("Overriding the plugin profile by \"%s\".",
+				 override);
 		}
 
-		if (misc_param_override(override))
+		if (misc_profile_override(override))
 			goto error_free_libreiser4;
 	}
 
 	if (flags & BF_SHOW_PARM)
-		misc_param_print();
+		misc_profile_print();
 	
 	if (flags & BF_SHOW_PLUG)
 		misc_plugins_print();
@@ -268,6 +291,12 @@ int main(int argc, char *argv[]) {
 	}
 #endif
 	
+	if (hint.blksize > REISER4_MAX_BLKSIZE) {
+		aal_error("Invalid blocksize (%u). It must not be greater then "
+			  "%u.", hint.blksize, REISER4_MAX_BLKSIZE);
+		return USER_ERROR;	
+	}
+	
 #ifdef HAVE_UNAME
 	/* Guessing system type */
 	if (uname(&sysinfo) == -1) {
@@ -326,7 +355,8 @@ int main(int argc, char *argv[]) {
 	}
 
 	if (!(flags & BF_QUIET) && aal_list_len(devices)) {
-		if (!(gauge = aal_gauge_create(GAUGE_SILENT, NULL)))
+		if (!(gauge = aal_gauge_create(aux_gauge_handlers[GT_PROGRESS],
+					       NULL, NULL, 0, NULL)))
 			goto error_free_libreiser4;
 	}
     
@@ -367,7 +397,7 @@ int main(int argc, char *argv[]) {
 		}
    
 		/* Checking if passed partition is mounted */
-		if (misc_dev_mounted(host_dev, NULL) && !(flags & BF_FORCE)) {
+		if (misc_dev_mounted(host_dev) > 0 && !(flags & BF_FORCE)) {
 			aal_error("Device %s is mounted at the moment. "
 				  "Use -f to force over.", host_dev);
 			goto error_free_libreiser4;
@@ -419,10 +449,9 @@ int main(int argc, char *argv[]) {
 		}
     
 		if (gauge) {
-			aal_gauge_rename(gauge, "Creating reiser4 on %s",
+			aal_gauge_rename(gauge, "Creating reiser4 on %s ... ",
 					 host_dev);
-
-			aal_gauge_start(gauge);
+			aal_gauge_touch(gauge);
 		}
 
 		/* Creating filesystem */
@@ -437,7 +466,7 @@ int main(int argc, char *argv[]) {
 			goto error_free_fs;
 
 		/* Creating root directory */
-		if (!(fs->root = reiser4_dir_create(fs, NULL, NULL))) {
+		if (!(fs->root = reiser4_root_create(fs))) {
 			aal_error("Can't create filesystem "
 				  "root directory.");
 			goto error_free_journal;
@@ -447,8 +476,15 @@ int main(int argc, char *argv[]) {
 		if (reiser4_object_link(fs->root, fs->root, NULL)) {
 			aal_error("Can't link root directory "
 				  "to itself.");
+			goto error_free_journal;
 		}
 	
+		if (reiser4_opset_init(fs->tree, 1)) {
+			aal_error("Can't initialize the fs-global "
+				  "object plugin set.");
+			goto error_free_journal;
+		}
+
 		/* Creating lost+found directory */
 		if (flags & BF_LOST) {
 			reiser4_object_t *object;
@@ -464,10 +500,9 @@ int main(int argc, char *argv[]) {
 			reiser4_object_close(object);
 		}
 	
-		if (gauge) {
+		if (gauge)
 			aal_gauge_done(gauge);
-		}
-	
+
 		/* Zeroing uuid in order to force mkfs to generate it on its own
 		   for next device form built device list. */
 		aal_memset(hint.uuid, 0, sizeof(hint.uuid));
